<?php

namespace App\Http\Controllers\admin\AppSetting\Category;

use App\Http\Controllers\Controller;
use App\Models\Category;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

class CategoryController extends Controller
{
    public function index()
    {
        return view('admin.app-setting.category.index');
    }

    public function getCategory(Request $request)
    {
        if ($request->ajax()) {
            $data = Category::get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="'.url('/').'/admin/app-setting/category/edit/'.$row->id.'" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="'.$row->id.'">Delete</a>';
                    return $btn;
                })

                ->rawColumns(['action'])

                ->make(true);
        }
    }

    public function add()
    {
        return view('admin.app-setting.category.add');
    }

    public function store(Request $request)
    {
        $category =  Category::create([
                'title_english' => $request->title_english,
                'title_arabic' => $request->title_arabic,
                'display_order' => $request->display_order,
            ]);
        if ($category) {
            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function edit($id)
    {
        $category = Category::where('id', $id)->first();
        return view('admin.app-setting.category.edit', compact('category'));
    }

    public function update(Request $request)
    {
        $category = Category::where('id', $request->id)->update([
            'title_english' => $request->title_english,
            'title_arabic' => $request->title_arabic,
            'display_order' => $request->display_order,
        ]);
        return response()->json([
            'success' => true,
            'msg' => 'Record updated successfully.'
        ]);
    }

    public function delete(Request $request)
    {
        $category = Category::where('id', $request->id)->delete();
        return response()->json([
            'success' => true,
            'msg' => 'Record deleted successfully.'
        ]);
    }
}
