<?php

namespace App\Http\Controllers\admin\AppSetting\Category;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\CategoryFAQ;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

class CategoryFAQController extends Controller
{
    public function index()
    {
        return view('admin.app-setting.categoryFAQ.index');
    }

    public function getCategory(Request $request)
    {
        if ($request->ajax()) {
            $data = CategoryFAQ::with('category')->get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="'.url('/').'/admin/app-setting/categoryFAQ/edit/'.$row->id.'" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="'.$row->id.'">Delete</a>';
                    return $btn;
                })
                ->addColumn('category', function ($row) {
                    if($row->category){
                      $category = $row->category->title_english;
                        return $category;  
                    }else{
                        return "";
                    }
                    
                })
                ->editColumn('question_english', function ($row) {
                    $question_english = strip_tags($row->question_english);
                    return $question_english;
                })
                ->editColumn('question_arabic', function ($row) {
                    $question_arabic = strip_tags($row->question_arabic);
                    return $question_arabic;
                })
                ->rawColumns(['category','action'])

                ->make(true);
        }
    }

    public function add()
    {
        $category = Category::all();
        return view('admin.app-setting.categoryFAQ.add', compact('category'));
    }

    public function store(Request $request)
    {
        $category =  CategoryFAQ::create([
                'question_english'  => $request->question_english,
                'answer_english'    => $request->answer_english,
                'question_arabic'   => $request->question_arabic,
                'answer_arabic'     => $request->answer_arabic,
                'category_id'       => $request->category,
                'display_order'     => $request->display_order,
            ]);
        if ($category) {
            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function edit($id)
    {
        // echo "category FAQ";die;
        $categoryFAQ = CategoryFAQ::where('id', $id)->first();
        return view('admin.app-setting.categoryFAQ.edit', compact('categoryFAQ'));
    }

    public function update(Request $request)
    {
        $category = CategoryFAQ::where('id', $request->id)->update([
            'question_english'  => $request->question_english,
            'answer_english'    => $request->answer_english,
            'question_arabic'   => $request->question_arabic,
            'answer_arabic'     => $request->answer_arabic,
            'category_id'       => $request->category_id,
            'display_order'     => $request->display_order,
        ]);
        return response()->json([
            'success' => true,
            'msg' => 'Record updated successfully.'
        ]);
    }

    public function delete(Request $request)
    {
        $category = CategoryFAQ::where('id', $request->id)->delete();
        return response()->json([
            'success' => true,
            'msg' => 'Record deleted successfully.'
        ]);
    }
    public function getCategoryData()
    {
        $category = Category::all();
        return response()->json([
            'success' => true,
            'category' => $category
        ]);
    }
}
