<?php

namespace App\Http\Controllers\Admin\AppSetting\Challenge;

use App\Http\Controllers\Controller;
use App\Models\Challenge;
use App\Models\ChallengeContent;
use App\Models\ChallengeSection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\Facades\DataTables;

class ChallengeController extends Controller
{
    public function index()
    {
        return view('admin.app-setting.challenge.index');
    }
    public function getAllChallengeSection()
    {
        $challenge = ChallengeSection::get();
        return response()->json([
            'success' => true,
            'challenge' => $challenge
        ]);
    }

    public function getAllContent(Request $request)
    {
        $content = ChallengeContent::where("challenge_id", $request->id)->get();
        return response()->json([
            'success' => true,
            'content' => $content
        ]);
    }

    public function getChallenge(Request $request){
        if ($request->ajax()) {
            $data = Challenge::latest()->get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="'.url('/').'/admin/app-setting/challenge/edit/'.$row->id.'" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="'.$row->id.'">Delete</a>';
                    return $btn;
                })
                ->editColumn('active', function ($menu) {
                    if ($menu->status == 1) {
                        $btn = '<div class="custom-control custom-switch">
                        <input type="checkbox" class="custom-control-input activeSwitch" checked  value="' . $menu->status . '" id="customSwitch' . $menu->id . '" data-id="' . $menu->id . '" data-language="' . $menu->language . '">
                        <label class="custom-control-label" for="customSwitch' . $menu->id . '"></label>
                      </div>';
                    } else {
                        $btn = '<div class="custom-control custom-switch">
                        <input type="checkbox" class="custom-control-input activeSwitch"  value="' . $menu->status . '" id="customSwitch' . $menu->id . '" data-id="' . $menu->id . '" data-language="' . $menu->language . '">
                        <label class="custom-control-label" for="customSwitch' . $menu->id . '"></label>
                      </div>';
                    }
                    return $btn;
                })
                ->rawColumns(['active','action'])
                ->make(true);
        }
    }

    public function add()
    {
        $challengeSection = ChallengeSection::get();
        return view('admin.app-setting.challenge.add', compact('challengeSection'));
    }

    public function store(Request $request)
    {

        $challenges = $request->challenge;
        $contents = $request->contents;
        $contentData = [];
        $contentCreateData = [];

        $content_count =  count($contents);

        $challenge =  Challenge::create([
            'title_english' => $challenges['title_english'],
            'title_arabic' => $challenges['title_arabic'],
            'description_english' => $challenges['description_english'],
            'description_arabic' => $challenges['description_arabic'],
            'cover_image' => $challenges['cover_image'],
            'description_image' => $challenges['description_image'],
            'final_upload_type' => $challenges['final_upload_type'],
            'final_cover_image' => $challenges['final_cover_image'],
            'final_url' => $challenges['final_url'],
            'badge' => $challenges['badge'],
            'access_type' => $challenges['access_type'],
            'status' => $challenges['status'],
            'challenge_type' => $challenges['challenge_type'],
            'content_count' => $content_count,
            'challenge_section_id' => $challenges['challenge_section_id'],
            ]);
        if ($challenge) {
            foreach($contents as $content){
                $contentData = [];
                $contentData["notification_english"] = $content['notification_english'];
                $contentData["notification_arabic"] = $content['notification_arabic'];
                $contentData["upload_type"] = $content['upload_type'];
                $contentData["cover_image"] = $content['cover_image'];
                $contentData["url"] = $content['url'];
                $contentData["challenge_id"] = $challenge->id;
                $contentData["created_at"] =  now();
                $contentData["updated_at"] =  now();
                $contentCreateData[] = $contentData;
            }
            if($contentCreateData){
                $result = ChallengeContent::insert($contentCreateData);
                if($result){
                    return response()->json([
                        'success' => true,
                        'msg' => 'Record saved successfully.'
                    ]);
                }else{
                    return response()->json([
                        'success' => false,
                        'msg' => 'Something went wrong.'
                    ]);
                }
            }else{
                return response()->json([
                    'success' => false,
                    'msg' => 'Something went wrong.'
                ]);
            }

            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function edit($id)
    {
        $challenge = Challenge::where('id', $id)->first();
        $challenge['error'] = array("final_url" => "");
        return view('admin.app-setting.challenge.edit', compact('challenge'));
    }

    public function update(Request $request)
    {

        $challenges = $request->challenge;
        $contents = $request->contents;

        $content_count =  count($contents);
        $challenge_id =  $challenges['id'];

        $challenge = Challenge::where('id', $challenges['id'])->update([
            'title_english' => $challenges['title_english'],
            'title_arabic' => $challenges['title_arabic'],
            'description_english' => $challenges['description_english'],
            'description_arabic' => $challenges['description_arabic'],
            'cover_image' => $challenges['cover_image'],
            'description_image' => $challenges['description_image'],
            'final_upload_type' => $challenges['final_upload_type'],
            'final_cover_image' => $challenges['final_cover_image'],
            'final_url' => $challenges['final_url'],
            'badge' => $challenges['badge'],
            'access_type' => $challenges['access_type'],
            'status' => $challenges['status'],
            'challenge_type' => $challenges['challenge_type'],
            'content_count' => $content_count,
        ]);
        if($challenge){
            foreach($contents as $content){
                if (isset($content['id'])) {
                    $contentResult = ChallengeContent::where('id', $content['id'])->update([
                        "notification_english" => $content['notification_english'],
                        "notification_arabic" => $content['notification_arabic'],
                        "upload_type" => $content['upload_type'],
                        "cover_image" => $content['cover_image'],
                        "url" => $content['url'],
                        "challenge_id" => $challenge_id,
                    ]);
                }else{
                    $contentResult = ChallengeContent::create([
                        "notification_english" => $content['notification_english'],
                        "notification_arabic" => $content['notification_arabic'],
                        "upload_type" => $content['upload_type'],
                        "cover_image" => $content['cover_image'],
                        "url" => $content['url'],
                        "challenge_id" => $challenge_id,
                    ]);
                }

            }
            if($contentResult){
                return response()->json([
                    'success' => true,
                    'msg' => 'Record Updated successfully.'
                ]);
            }else{
                return response()->json([
                    'success' => false,
                    'tab' => 'content data.',
                    'msg' => 'Something went wrong.'
                ]);
            }
        }else{
            return response()->json([
                'success' => false,
                'tab' => 'update data.',
                'msg' => 'Something went wrong.'
            ]);
        }

    }

    public function delete(Request $request)
    {
        $challenge = Challenge::where('id', $request->id)->delete();
        return response()->json([
            'success' => true,
            'msg' => 'Record deleted successfully.'
        ]);
    }

    public function uploadCoverImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("challenge");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeCoverImage(Request $request)
    {
        Challenge::where('cover_image', $request->url)->update([
            'cover_image' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Cover Image removed successfully.'
        ]);
    }

    public function uploadDescriptionImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("challenge");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeDescriptionImage(Request $request)
    {
        Challenge::where('description_image', $request->url)->update([
            'description_image' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Description Image removed successfully.'
        ]);
    }

    public function uploadFinalCoverImage(Request $request)
    {

        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("challenge");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeFinalCoverImage(Request $request)
    {
        Challenge::where('final_cover_image', $request->url)->update([
            'final_cover_image' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Cover Image removed successfully.'
        ]);
    }

    public function uploadFinalUrl(Request $request)
    {

        $data = $request->all();

        if($request->type != ''){
            if (count($data) and $request->type == 'video') {
                $validator = Validator::make($data, [
                    'video' => 'required|mimes:mp4|max:100000',
                ]);
            } else {
                $validator = Validator::make($data, [
                    'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
                ]);
            }
            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                    'data' => array()
                ), 200);
            } else {
                $path = $request->file($request->type)->store("challenge");
                return Response::json(array(
                    'success' => true,
                    'errors' => $validator->getMessageBag()->toArray(),
                    'data' => array('url' => $path)
                ), 200);
            }
        }else{
            $a[] = 'Please select upload type first';
            $b['image'] = $a;
            return Response::json(array(
                'success' => false,
                'errors' => $b,
                'data' => array()
            ), 200);
        }
    }

    public function removeFinalUrl(Request $request)
    {
        Challenge::where('final_url', $request->url)->update([
            'final_url' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function removeCoverUrl(Request $request)
    {
        Challenge::where('cover_image', $request->final_cover_image)->orWhere('final_url', $request->final_url)->update([
            'final_cover_image' => null,
            'final_url' => null,
        ]);
        Storage::delete('' . $request->final_cover_image);
        Storage::delete('' . $request->final_url);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function uploadBadge(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("challenge");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeBadge(Request $request)
    {
        Challenge::where('badge', $request->url)->update([
            'badge' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Badge removed successfully.'
        ]);
    }

    public function manageStatus(Request $request)
    {
        if ($request->status == 1) {
            Challenge::where('id', $request->id)->update([
                'status' => 0
            ]);
        } elseif ($request->status == 0) {
            Challenge::where('id', $request->id)->update([
                'status' => 1
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Status update successfully.'
        ]);
    }

    public function uploadContentCoverImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("challenge-contents");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeContentCoverImage(Request $request)
    {
        ChallengeContent::where('cover_image', $request->url)->update([
            'cover_image' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Cover Image removed successfully.'
        ]);
    }

    public function uploadContentUrl(Request $request)
    {
        $data = $request->all();

        if($request->type != ''){
            if (count($data) and $request->type == 'video') {
                $validator = Validator::make($data, [
                    'video' => 'required|mimes:mp4|max:100000',
                ]);
            } else {
                $validator = Validator::make($data, [
                    'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
                ]);
            }
            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                    'data' => array()
                ), 200);
            } else {
                $path = $request->file($request->type)->store("challenge-contents");
                return Response::json(array(
                    'success' => true,
                    'errors' => $validator->getMessageBag()->toArray(),
                    'data' => array('url' => $path)
                ), 200);
            }
        }else{
            $a[] = 'Please select upload type first';
            $b['image'] = $a;
            return Response::json(array(
                'success' => false,
                'errors' => $b,
                'data' => array()
            ), 200);
        }
    }

    public function removeContentUrl(Request $request)
    {
        ChallengeContent::where('url', $request->url)->update([
            'url' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function removeContentCoverUrl(Request $request)
    {
        ChallengeContent::where('cover_image', $request->cover_image)->orWhere('url', $request->url)->update([
            'cover_image' => null,
            'url' => null,
        ]);
        Storage::delete('' . $request->cover_image);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function removeContent(Request $request)
    {
        $id = $request->id;
        $category = ChallengeContent::where('id', $id)->delete();
        if ($category) {
            Storage::delete('' . $request->url);
            Storage::delete('' . $request->cover_image);
            return response()->json([
                'success' => true,
                'msg' => 'Record deleted successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
}
