<?php

namespace App\Http\Controllers\Admin\AppSetting\Challenge;

use App\Http\Controllers\Controller;
use App\Models\Challenge;
use App\Models\ChallengeSection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\Facades\DataTables;

class ChallengeSectionController extends Controller
{
    public function index()
    {
        return view('admin.app-setting.challenge-section.index');
    }

    public function getChallenge(Request $request){
        if ($request->ajax()) {
            $data = ChallengeSection::get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="'.url('/').'/admin/app-setting/challenge-section/edit/'.$row->id.'" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="'.$row->id.'">Delete</a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
    }

    public function add()
    {
        return view('admin.app-setting.challenge-section.add');
    }

    public function store(Request $request)
    {
        $challengeSection =  ChallengeSection::create([
                'title_english' => $request->title_english,
                'title_arabic' => $request->title_arabic,
                'sub_title_english' => $request->sub_title_english,
                'sub_title_arabic' => $request->sub_title_arabic,
                'display_order' => $request->display_order,
            ]);
        if ($challengeSection) {
            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function edit($id)
    {
        $challengeSection = ChallengeSection::where('id', $id)->first();
        return view('admin.app-setting.challenge-section.edit', compact('challengeSection'));
    }

    public function update(Request $request)
    {
        $challengeSection = ChallengeSection::where('id', $request->id)->update([
            'title_english' => $request->title_english,
            'title_arabic' => $request->title_arabic,
            'sub_title_english' => $request->sub_title_english,
            'sub_title_arabic' => $request->sub_title_arabic,
            'display_order' => $request->display_order,
        ]);
        return response()->json([
            'success' => true,
            'msg' => 'Record updated successfully.'
        ]);
    }

    public function delete(Request $request)
    {
        $challengeSection = ChallengeSection::where('id', $request->id)->delete();
        return response()->json([
            'success' => true,
            'msg' => 'Record deleted successfully.'
        ]);
    }

    public function uploadImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("challenge");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeImage(Request $request)
    {
        Challenge::where('image', $request->url)->update([
            'image' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Icon removed successfully.'
        ]);
    }

    public function manageStatus(Request $request)
    {
        if ($request->status == 1) {
            Challenge::where('id', $request->id)->update([
                'status' => 0
            ]);
        } elseif ($request->status == 0) {
            Challenge::where('id', $request->id)->update([
                'status' => 1
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Status update successfully.'
        ]);
    }
}
