<?php

namespace App\Http\Controllers\Admin\AppSetting\Customer;

use App\Exports\CustomerExport;
use App\Http\Controllers\Controller;
use App\Models\CurrencyPoint;
use App\Models\Customer;
use App\Models\Milestone;
use Illuminate\Http\Request;
use DataTables;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;
use Yajra\DataTables\Facades\DataTables as FacadesDataTables;

class CustomerController extends Controller
{
    public $client;
    public $url;
    
    public function __construct()
    {
        ini_set('max_execution_time', '0');
        $this->client = new Client();
        $this->url = env('SHOPIFY_API_REQUEST_URL') . 'customers.json';
    }

    public function index()
    {
        return view("admin.app-setting.customer.list");
    }

    public function getCustomers(Request $request)
    {
        ini_set('memory_limit', '-1');
        if ($request->ajax()) {
            $data = Customer::latest()->get()->toArray();
            // print_r($data);
            // exit;
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="' . url('/') . '/admin/app-setting/customer/edit/' . $row['customer_id'] . '" class="edit btn btn-primary btn-sm">Edit</a>';
                    return $btn;
                })
                ->editColumn('point', function ($point_data) {
                    $points = $point_data['point'] + $point_data['extra_point'];
                    return $points;
                })
                ->editColumn('level', function ($point_data) {
                    $points = $point_data['point'] + $point_data['extra_point'];
                    $level = $this->getLevel($points);
                    return $level;
                })
                ->rawColumns(['point','level','action'])

                ->make(true);
        }
    }
    public function getLevel($point){

        $max = Milestone::max('from_point');
        if($point >= $max){
            $milestone = Milestone::where('from_point', '=',$max)->select('level_english')->first();
            if($milestone){
                return $milestone->level_english;
            }else{
                return 'Not Available';
            }
        }else{
            $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('level_english')->first();
            if($milestone){
                return $milestone->level_english;
            }else{
                return 'Not Available';
            }
        }
    }
    public function customerSync(Request $request)
    {
        ini_set('max_execution_time', 300);

        $customerCreateData = [];
        $customerUpdateData = [];
        $customerData       = [];
        $customerIdData     = [];
$i = 0;
        while(true){
            // echo $this->url;echo "<br>";
            $res = $this->client->request(
                'GET',
                trim($this->url),
                ['headers' => 
                    [
                        'X-Shopify-Access-Token' => "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",
                    ]
                ]
            );

            if ($res->getStatusCode() == 200) { // 200 OK
                $response_data = $res->getBody()->getContents();
                $customers = json_decode($response_data)->customers;
                foreach($customers as $customer){
                    array_push($customerIdData, $customer->id);
                    $currencyPoint  = CurrencyPoint::where('currency', $customer->currency)->first();
                    if($currencyPoint){
                        $point = $customer->total_spent * $currencyPoint->each_point;
                    }else{
                        $cPoint  = CurrencyPoint::where('currency', 'KWD')->first();
                        if($cPoint){
                            $point = $customer->total_spent * $cPoint->each_point;
                        }else{
                            $point = $customer->total_spent * 1;
                        }
                    }
                    $exist = Customer::where('customer_id', $customer->id)->first();
                    if($exist){
                        $levelPoint = $exist->point + $exist->extra_point;
                        $level = $this->getLevel($levelPoint);
                        $milestone_id = $this->getMilestone($levelPoint);
                        $customerData["customer_id"]        = $customer->id;
                        $customerData["email"]              = $customer->email;
                        $customerData["first_name"]         = $customer->first_name;
                        $customerData["last_name"]          = $customer->last_name;
    
                        $customerData["phone"]              = $customer->phone;
                        $customerData["state"]              = $customer->state;
                        $customerData["total_spent"]        = $customer->total_spent;
                        $customerData["currency"]           = $customer->currency;
                        if($customer->last_order_id != NULL){
                            $customerData["last_order_id"] = $customer->last_order_id ;
                        }
                        $customerData["last_order_name"]    = $customer->last_order_name;
                        $customerData["orders_count"]       = $customer->orders_count;
                        $customerData["point"]              = $point;
                        $customerData["level"]              = $level;
                        $customerData["milestone_id"]       = $milestone_id;
                        $customerData["updated_at"]         =  now();
                        $customerUpdateData[] = $customerData;
                        
                    }else{
                        $newLevel = $this->getLevel(0);
                        $new_milestone_id = $this->getMilestone(0);
                        $customerDetail["customer_id"]      = $customer->id;
                        $customerDetail["email"]            = $customer->email;
                        $customerDetail["first_name"]       = $customer->first_name;
                        $customerDetail["last_name"]        = $customer->last_name;
                        
                        $customerDetail["phone"]            = $customer->phone;
                        $customerDetail["state"]            = $customer->state;
                        $customerDetail["total_spent"]      = $customer->total_spent;
                        $customerDetail["last_order_id"]    = $customer->last_order_id;
                        $customerDetail["last_order_name"]  = $customer->last_order_name;
                        $customerDetail["currency"]         = $customer->currency;
                        $customerDetail["orders_count"]     = $customer->orders_count;
                        $customerDetail["point"]            = $point;
                        $customerDetail["level"]            = $newLevel;
                        $customerDetail["milestone_id"]     = $new_milestone_id;
                        // $customerDetail["created_at"]       =  now();
                        // $customerDetail["updated_at"]       =  now();
                        // $customerDetail["milestone_entry"]  =  now();
                        // Customer::create($customerDetail);
                        $customerCreateData[] = $customerDetail;
                    }
                }
                echo "<pre>";
                echo $i;
                print_r($customerCreateData);
                $i++;
                $response_header =  $res->getHeader('Link');
                // echo "<pre>";
                // echo $i;
                // print_r($response_header);
                // $i++;
                $header =  explode(',',$response_header[0]);

                if(count($header) > 1 && count($header) <= 2){
                    $next =  explode(';',$header[1]);
                    $next_url = str_replace( array( '<', '>' ), ' ', $next[0]);
                    $this->url = $next_url;
                }elseif(count($header) > 0 && count($header) <= 1){
                    $next =  explode(';',$header[0]);
                    $rel =  str_replace( array( 'rel', '=','"' ), ' ', $next[1]);
                    if(trim($rel) == 'next'){
                        $next_url = str_replace( array( '<', '>' ), ' ', $next[0]);
                        $this->url = $next_url;
                    }else{
                        break;
                    }
                }else{
                    break;
                }
            }
        }

        $customerIds =  Customer::get()->pluck('customer_id')->toArray();
        // echo "<pre>";
        // print_r($customerCreateData);die;
        if ($customerCreateData) {
            $result = Customer::insert($customerCreateData);
        }
        if ($customerUpdateData) {
            $result = $this->updateBatch($customerUpdateData);
        }
        if ($customerIdData != null && $customerIds != null) {
            foreach ($customerIds as $customerId) {
                if (!in_array($customerId, $customerIdData)) {
                    $result = Customer::where('customer_id', $customerId)->update([
                        'is_delete' => 1
                    ]);
                }
            }
        }
        if ($result) {
            return response()->json([
                'success' => true,
                'msg' => 'Customer Saved Successfully'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong'
            ]);
        } 
    }

    public function updateBatch($multipleData)
    {
        $tableName = "customers";
        if ($tableName && !empty($multipleData)) {

            // column or fields to update
            $updateColumn = array_keys($multipleData[0]);
            $referenceColumn = $updateColumn[0]; //e.g id
            unset($updateColumn[0]);
            $whereIn = "";

            $q = "UPDATE " . $tableName . " SET ";
            foreach ($updateColumn as $uColumn) {
                $q .=  $uColumn . " = CASE ";

                foreach ($multipleData as $data) {
                    $q .= "WHEN " . $referenceColumn . " = " . $data[$referenceColumn] . " THEN '" . $data[$uColumn] . "' ";
                }
                $q .= "ELSE " . $uColumn . " END, ";
            }
            foreach ($multipleData as $data) {
                $whereIn .= "'" . $data[$referenceColumn] . "', ";
            }
            $q = rtrim($q, ", ") . " WHERE " . $referenceColumn . " IN (" .  rtrim($whereIn, ', ') . ")";
            // Update  
            $affected =  DB::update(DB::raw($q));
            // $affected = DB::update($q);
            return  $affected;
        } else {
            return false;
        }
    }
    public function customerEdit($customer_id)
    {
        $customer = Customer::where('customer_id', $customer_id)->first();
        return view('admin.app-setting.customer.edit', compact('customer'));
    }
    public function customerUpdate(Request $request)
    {
        $customer = Customer::where('customer_id', $request->customer_id)->update([
            'note' => $request->note
        ]);
        return response()->json([
            'success' => true,
            'msg' => 'Note updated successfully.'
        ]);
    }

    public function getMilestone($point){

        $max = Milestone::max('to_point');
        if($point > $max){
            $milestone = Milestone::where('to_point', '=',$max)->select('id', 'level')->first();
            if($milestone){
                return $milestone->id;
            }else{
                return 0;
            }
        }else{
            $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('id', 'level')->first();
            if($milestone){
                return $milestone->id;
            }else{
                return 0;
            }
        }
    }

    public function customerExport()
    {
        ini_set('memory_limit', '-1');
        if(Auth::user()->role == 1){
            return Excel::download(new CustomerExport, 'customer.xlsx');
        }
    }
}
