<?php

namespace App\Http\Controllers\Admin\AppSetting\FlashSale;

use Response;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Controller;
use DataTables;
use App\Models\Menu;
use App\Models\FlashSale;
use App\Models\ExclusiveSection;
use App\Models\Product;
use App\Models\NormalCollection;
use App\Models\CollectionByGoal;
use GuzzleHttp\Client;

class FlashSaleController extends Controller
{

    public function __construct()
    {
        $this->middleware('auth');
        $this->client = new Client();
    }

    public function add()
    {
        return view('admin.app-setting.flash-sale.add');
    }

    public function edit($id)
    {
        $sale = FlashSale::where('id', $id)->first();
        $sale->start_time = date('H:i', strtotime($sale->start_time));
        $sale->end_time = date('H:i', strtotime($sale->end_time));
        return view('admin.app-setting.flash-sale.edit', compact('sale'));
    }

    public function index()
    {
        return view('admin.app-setting.flash-sale.index');
    }

    public function getAllFlashSale(Request $request)
    {
        if ($request->ajax()) {
            $data = FlashSale::latest()->get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="' . url('/') . '/admin/app-setting/flash-sale/edit/' . $row->id . '" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="' . $row->id . '">Delete</a>';
                    return $btn;
                })
                ->rawColumns(['action'])

                ->make(true);
        }
    }

    public function store(Request $request)
    {
        $id = 0;
        $end_date = "";
        $end_time = "";
        if ($request->status) {
            $end_date = $request->end_date;
            $end_time = $request->end_time;
        } else {
            $end_date = NULL;
            $end_time = NULL;
        }
        if (isset($request->id)) {
            $sale = FlashSale::where('id', $request->id)->update([
                'start_date' => $request->start_date,
                'end_date' => $end_date,
                'start_time' => $request->start_time,
                'end_time' => $end_time,
                'title_english' => $request->title_english,
                'title_arabic' => $request->title_arabic,
                'status' => $request->status,
                'is_show_quantity' => $request->is_show_quantity
            ]);
        } else {
            $sale = FlashSale::create([
                'start_date' => $request->start_date,
                'end_date' => $end_date,
                'start_time' => $request->start_time,
                'end_time' => $end_time,
                'title_english' => $request->title_english,
                'title_arabic' => $request->title_arabic,
                'status' => $request->status,
                'is_show_quantity' => $request->is_show_quantity
            ]);
        }
        if (isset($request->id)) {
            $id = $request->id;
        } else {
            $id = $sale->id;
        }
        $sale = FlashSale::where('id', $id)->first();
        if ($sale) {
            $sale->start_time = date('H:i', strtotime($sale->start_time));
            $sale->end_time = date('H:i', strtotime($sale->end_time));
            return response()->json([
                'success' => true,
                'msg' => 'Flash sale saved successfully.',
                'sale' => $sale
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function update(Request $request)
    {
        $menu = Menu::where('id', $request->id)->update([
            'title_english' => $request->title_english,
            'title_arabic' => $request->title_arabic,
            'icon' => $request->icon,
            'link' => $request->link,
        ]);
        return response()->json([
            'success' => true,
            'msg' => 'Record updated successfully.'
        ]);
    }
    public function delete(Request $request)
    {
        $sale = FlashSale::where('id', $request->id)->delete();
        ExclusiveSection::where('flash_sale_id', $request->id)->delete();
        return response()->json([
            'success' => true,
            'msg' => 'Record deleted successfully.'
        ]);
    }


    public function getAllCollection()
    {
        $custom_collection = [];
        $collection = [];
        $res = $this->client->request(
            'GET',
            env('SHOPIFY_API_REQUEST_URL') . 'custom_collections.json'
        );
        if ($res->getStatusCode() == 200) { // 200 OK
            $response_data = $res->getBody()->getContents();
            $custom_collection = json_decode($response_data)->custom_collections;
            foreach ($custom_collection as $key => $col) {
                $collection[$key]['collection_id'] = $col->id;
                $collection[$key]['title'] = $col->title;
            }
            return response()->json([
                'success' => true,
                'collection' => $collection
            ]);
        }
    }

    public function getCollectionProduct(Request $request)
    {
        $product = [];
        $res = $this->client->request(
            'GET',
            env('SHOPIFY_API_REQUEST_URL') . 'collections/' . $request->collection_id . '/products.json'
        );
        if ($res->getStatusCode() == 200) { // 200 OK
            $response_data = $res->getBody()->getContents();
            $shopify_product = json_decode($response_data)->products;
            foreach ($shopify_product as $key => $prod) {
                if ($prod->status == 'active') {
                    $product[$key]['product_id'] = $prod->id;
                    $product[$key]['title'] = $prod->title;
                    if (isset(($prod->image)->src)) {
                        $product[$key]['image'] = ($prod->image)->src;
                    } else {
                        $product[$key]['image'] = "";
                    }

                    $product[$key]['title_english'] = $prod->title;
                    $product[$key]['title_arabic'] = "";
                    $product[$key]['label_english'] = "";
                    $product[$key]['label_arabic'] = "";
                    $product[$key]['display_order'] = "";
                    $product[$key]['error'] = array('title_english' => "", "title_arabic" => "", "label_english" => "", "label_arabic" => "", 'display_order' => "");
                }
            }
            return response()->json([
                'success' => true,
                'product' => $product
            ]);
        }
    }

    public function addExclusiveProduct(Request $request)
    {
        $products = $request->products;
        $flash_id = $request->flash_id['id'];
        $product_view = $request->exclusive_section['product_view'];
        $collection_id = $request->exclusive_section['collection_id'];
        $add_type = $request->exclusive_section['add_type'];
        $exclusive = [];
        foreach ($products as $key => $p) {
            $exclusive[$key]['product_id'] = $p['product_id'];
            $exclusive[$key]['title_english'] = $p['title_english'];
            $exclusive[$key]['title_arabic'] = $p['title_arabic'];
            $exclusive[$key]['label_english'] = $p['label_english'];
            $exclusive[$key]['label_arabic'] = $p['label_arabic'];
            $exclusive[$key]['display_order'] = $p['display_order'];
            $exclusive[$key]['image'] = $p['image'];
            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'products/' . $p['product_id'] . '.json'
            );
            if ($res->getStatusCode() == 200) { // 200 OK
                $response_data = $res->getBody()->getContents();
                $shopify_product = json_decode($response_data)->product;
                $exclusive[$key]['quantity'] = $shopify_product->variants[0]->inventory_quantity;
            }

            $exclusive[$key]['product_view'] = $product_view;
            $exclusive[$key]['add_type'] = $add_type;
            $exclusive[$key]['collection_id'] = $collection_id;
            $exclusive[$key]['flash_sale_id'] = $flash_id;
            $exclusive[$key]['created_at'] = date("Y-m-d H:i:s");
            $exclusive[$key]['updated_at'] = date("Y-m-d H:i:s");
        }
        ExclusiveSection::insert($exclusive);
        return response()->json([
            'success' => true,
            'msg' => "Product saved successfully."
        ]);
    }

    public function getAllShopifyProduct()
    {
        $product = [];
        $res = $this->client->request(
            'GET',
            env('SHOPIFY_API_REQUEST_URL') . 'products.json'
        );
        if ($res->getStatusCode() == 200) { // 200 OK
            $response_data = $res->getBody()->getContents();
            $shopify_product = json_decode($response_data)->products;
            foreach ($shopify_product as $key => $prod) {
                if ($prod->status == 'active') {
                    $product[$key]['product_id'] = $prod->id;
                    $product[$key]['title'] = $prod->title;
                    if (isset(($prod->image)->src)) {
                        $product[$key]['image'] = ($prod->image)->src;
                    } else {
                        $product[$key]['image'] = "";
                    }
                    $product[$key]['title_english'] = $prod->title;
                    $product[$key]['title_arabic'] = "";
                    $product[$key]['label_english'] = "";
                    $product[$key]['label_arabic'] = "";
                    $product[$key]['display_order'] = "";
                    $product[$key]['error'] = array('title_english' => "", "title_arabic" => "", "label_english" => "", "label_arabic" => "", 'display_order' => "");
                }
            }
            return response()->json([
                'success' => true,
                'product' => $product
            ]);
        }
    }

    public function getFlashSaleProduct(Request $request)
    {
        $product = [];
        $product_view = "";
        $collection_id = "";
        $add_type = "";
        $exclusive = ExclusiveSection::where('flash_sale_id', $request->flash_id)->get();
        foreach ($exclusive as $key => $e) {
            $product[$key]['id'] = $e['id'];
            $product[$key]['product_id'] = $e['product_id'];
            $product[$key]['title_english'] = $e['title_english'];
            $product[$key]['title_arabic'] = $e['title_arabic'];
            $product[$key]['label_arabic'] = $e['label_arabic'];
            $product[$key]['label_english'] = $e['label_english'];
            $product[$key]['display_order'] = $e['display_order'];
            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'products/'.$e['product_id'].'.json'
            );
            if ($res->getStatusCode() == 200) { // 200 OK
                $response_data = $res->getBody()->getContents();
                $shopify_product = json_decode($response_data)->product;
            }
            $product[$key]['image'] = isset($shopify_product->image->src) ? $shopify_product->image->src : "";
            $product[$key]['error'] = array('title_english' => "", "title_arabic" => "", "label_english" => "", "label_arabic" => "", 'display_order' => "");
            $product_view = $e['product_view'];
            $collection_id = $e['collection_id'];
            $add_type = $e['add_type'];
        }
        return response()->json([
            'success' => true,
            'product' => $product,
            'product_view' => $product_view,
            'collection_id' => $collection_id,
            'add_type' => $add_type
        ]);
    }

    public function updateExclusiveProduct(Request $request)
    {
        $products = $request->products;
        $flash_id = $request->flash_id['id'];
        $product_view = $request->exclusive_section['product_view'];
        $collection_id = $request->exclusive_section['collection_id'];
        $add_type = $request->exclusive_section['add_type'];
        $exclusive = [];
        ExclusiveSection::where('flash_sale_id', $flash_id)->delete();
        foreach ($products as $key => $p) {
            $exclusive[$key]['product_id'] = $p['product_id'];
            $exclusive[$key]['title_english'] = $p['title_english'];
            $exclusive[$key]['title_arabic'] = $p['title_arabic'];
            $exclusive[$key]['label_english'] = $p['label_english'];
            $exclusive[$key]['label_arabic'] = $p['label_arabic'];
            $exclusive[$key]['display_order'] = $p['display_order'];
            $exclusive[$key]['image'] = $p['image'];
            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'products/' . $p['product_id'] . '.json'
            );
            if ($res->getStatusCode() == 200) { // 200 OK
                $response_data = $res->getBody()->getContents();
                $shopify_product = json_decode($response_data)->product;
                $exclusive[$key]['quantity'] = $shopify_product->variants[0]->inventory_quantity;
            }
            $exclusive[$key]['product_view'] = $product_view;
            $exclusive[$key]['add_type'] = $add_type;
            $exclusive[$key]['collection_id'] = $collection_id;
            $exclusive[$key]['flash_sale_id'] = $flash_id;
            $exclusive[$key]['created_at'] = date("Y-m-d H:i:s");
            $exclusive[$key]['updated_at'] = date("Y-m-d H:i:s");
        }
        ExclusiveSection::insert($exclusive);
        return response()->json([
            'success' => true,
            'msg' => "Record updated successfully."
        ]);
    }
}
