<?php

namespace App\Http\Controllers\Admin\AppSetting;

use Response;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Controller;
use App\Models\Banner;
use App\Models\ChallengeSection;
use App\Models\Collection;
use App\Models\CollectionByGoal;
use App\Models\Menu;
use App\Models\NormalCollection;
use Exception;
use GuzzleHttp\Client;
use Str;

class HomePageController extends Controller
{

    public $client;

    public function __construct()
    {
        $this->middleware('auth');
        $this->client = new Client();
    }

    public function add()
    {
        return view('admin.app-setting.home-page');
    }

    public function uploadImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg,gif|max:10240',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("normal_collection");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function uploadGoalImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file('image')->store("collection_by_goal");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function saveNormalCollection(Request $request)
    {
        $vertical_collection = $request->vertical_collection;
        $horizontal_collection = $request->horizontal_collection;

        foreach ($vertical_collection as $vc) {
            if (isset($vc['id'])) {
                NormalCollection::where('id', $vc['id'])->update([
                    'text_english' => $vc['text_english'],
                    'text_arabic' => $vc['text_arabic'],
                    'collection_image' => $vc['collection_image'],
                    'collection_image_type' => $vc['collection_image_type'],
                ]);
            } else {
                NormalCollection::create([
                    'text_english' => $vc['text_english'],
                    'text_arabic' => $vc['text_arabic'],
                    'collection_image' => $vc['collection_image'],
                    'collection_image_type' => $vc['collection_image_type'],
                ]);
            }
        }
        if (isset($horizontal_collection)) {
            if (isset($horizontal_collection['id'])) {
                NormalCollection::where('id', $horizontal_collection['id'])->update([
                    'text_english' => $horizontal_collection['text_english'],
                    'text_arabic' => $horizontal_collection['text_arabic'],
                    'collection_image' => $horizontal_collection['collection_image'],
                    'collection_image_type' => $horizontal_collection['collection_image_type'],
                ]);
            } else {
                NormalCollection::create([
                    'text_english' => $horizontal_collection['text_english'],
                    'text_arabic' => $horizontal_collection['text_arabic'],
                    'collection_image' => $horizontal_collection['collection_image'],
                    'collection_image_type' => $horizontal_collection['collection_image_type'],
                ]);
            }
        }
        $verticle_collection = NormalCollection::where('collection_image_type', 'vertical')->get()->toArray();
        $horizontal_collection = NormalCollection::where('collection_image_type', 'horizontal')->first();
        $normal_collection['vertical_collection'] = $verticle_collection;
        $normal_collection['horizontal_collection'] = $horizontal_collection;
        foreach ($normal_collection['vertical_collection'] as $key => $vc) {
            $normal_collection['vertical_collection'][$key]['error'] = array("english" => "", "arabic" => "", "image" => "");
        }
        if ($normal_collection['horizontal_collection']) {
            $normal_collection['horizontal_collection']['error'] = array("english" => "", "arabic" => "", "image" => "");
        }
        return response()->json([
            'success' => true,
            'msg' => 'Record saved successfully.',
            'normal_collection' => $normal_collection
        ]);
    }
    public function getHomePageData()
    {
        $banner = [];
        $normal_collection = [];
        $collection_goal = [];
        $banner = Banner::all();
        $verticle_collection = NormalCollection::where('collection_image_type', 'vertical')->get()->toArray();
        $horizontal_collection = NormalCollection::where('collection_image_type', 'horizontal')->first();
        $collection_goal = CollectionByGoal::all();
        foreach ($collection_goal as $cg) {
            $cg['error'] = array("english" => "", "arabic" => "", "order" => "");
        }
        $normal_collection['vertical_collection'] = $verticle_collection;
        $normal_collection['horizontal_collection'] = $horizontal_collection;
        foreach ($normal_collection['vertical_collection'] as $key => $vc) {
            $normal_collection['vertical_collection'][$key]['error'] = array("english" => "", "arabic" => "", "image" => "");
        }
        if ($normal_collection['horizontal_collection']) {
            $normal_collection['horizontal_collection']['error'] = array("english" => "", "arabic" => "", "image" => "");
        }
        foreach ($banner as $bs) {
            if($bs->destination_english == NULL){
                $bs->destination_english = "";
            }
            $bs['error'] = array("type_of_banner" => "", "upload_banner_type" => "", "banner_url" => "", "cover_image" => "", "english" => "", "banner_language" => "", "arabic" => "", "order" => "");
        }
        return response()->json([
            'success' => true,
            'banner' => $banner,
            'collection_by_goal' => $collection_goal,
            'normal_collection' => $normal_collection,
        ]);
    }
    public function removeGoalCollection(Request $request)
    {
        $id = $request->id;
        $collection = CollectionByGoal::where('id', $id)->delete();
        if ($collection) {
            return response()->json([
                'success' => true,
                'msg' => 'Record deleted successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function saveCollectionGoal(Request $request)
    {
        $collection_goal = $request->all();

        foreach ($collection_goal as $cg) {
            if (isset($cg['id'])) {
                CollectionByGoal::where('id', $cg['id'])->update([
                    'title_english' => $cg['title_english'],
                    'title_arabic' => $cg['title_arabic'],
                    'display_order' => $cg['display_order'],
                    'collection_image' => $cg['collection_image'],
                ]);
            } else {
                CollectionByGoal::create([
                    'title_english' => $cg['title_english'],
                    'title_arabic' => $cg['title_arabic'],
                    'display_order' => $cg['display_order'],
                    'collection_image' => $cg['collection_image'],
                ]);
            }
        }
        $collection_goal = CollectionByGoal::all();
        foreach ($collection_goal as $cg) {
            $cg['error'] = array("english" => "", "arabic" => "", "order" => "");
        }
        return response()->json([
            'success' => true,
            'collection_by_goal' => $collection_goal,
            'msg' => 'Record saved successfully.'
        ]);
    }

    public function removeImage(Request $request)
    {
        if ($request->type == "normal_collection") {
            NormalCollection::where('collection_image', $request->url)->update([
                'collection_image' => null
            ]);
        }
        if ($request->type == "banner") {
            Banner::where('banner_url', $request->url)->update([
                'banner_url' => null
            ]);
        }
        if ($request->type == "banner_cover_image") {
            Banner::where('cover_image', $request->url)->update([
                'cover_image' => null
            ]);
        }
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }
    public function removeCollectionGoalImage(Request $request)
    {
        Storage::delete('' . $request->url);
        CollectionByGoal::where('collection_image', $request->url)->update([
            'collection_image' => null
        ]);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }
    public function linkNormalCollection(Request $request)
    {
        $custom_collection = [];
        $is_match = false;
        $normal_collection = NormalCollection::where('id', $request->id)->first();
        if ($normal_collection) {
            $slug = Str::slug($normal_collection->text_english);
            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'custom_collections.json'
            );
            if ($res->getStatusCode() == 200) { // 200 OK
                $response_data = $res->getBody()->getContents();
                $custom_collection = json_decode($response_data)->custom_collections;
                foreach ($custom_collection as $col) {
                    $title = Str::slug(explode("|", $col->title)[0]);
                    if ($title == $slug) {
                        NormalCollection::where('id', $request->id)->update(
                            [
                                'collection_id' => $col->id
                            ]
                        );
                        $is_match = true;
                        return response()->json([
                            'success' => true,
                            'msg' => 'Linked successfully.',
                            'is_linked' => $is_match
                        ]);
                    }
                }
                if (!$is_match) {
                    return response()->json([
                        'success' => true,
                        'msg' => 'Collection does not exist in shopify.',
                        'is_linked' => $is_match
                    ]);
                }
            } else {
                return response()->json([
                    'success' => false,
                    'msg' => 'Something went wrong.'
                ]);
            }
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function linkCollectionGoal(Request $request)
    {
        $custom_collection = [];
        $is_match = false;
        $collection_goal = CollectionByGoal::where('id', $request->id)->first();
        if ($collection_goal) {
            $slug = Str::slug($collection_goal->title_english);
            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'custom_collections.json'
            );
            if ($res->getStatusCode() == 200) { // 200 OK
                $response_data = $res->getBody()->getContents();
                $custom_collection = json_decode($response_data)->custom_collections;
                foreach ($custom_collection as $col) {
                    $title = Str::slug(explode("|", $col->title)[0]);
                    if ($title == $slug) {
                        CollectionByGoal::where('id', $request->id)->update(
                            [
                                'collection_id' => $col->id
                            ]
                        );
                        $is_match = true;
                        return response()->json([
                            'success' => true,
                            'msg' => 'Linked successfully.',
                            'is_linked' => $is_match
                        ]);
                    }
                }
                if (!$is_match) {
                    return response()->json([
                        'success' => true,
                        'msg' => 'Collection does not exist in shopify.',
                        'is_linked' => $is_match
                    ]);
                }
            } else {
                return response()->json([
                    'success' => false,
                    'msg' => 'Something went wrong.'
                ]);
            }
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function removeCoverBannerImage(Request $request)
    {
        Banner::where('cover_image', $request->coverUrl)->orWhere('banner_url', $request->bannerUrl)->update([
            'cover_image' => null,
            'banner_url' => null,
        ]);
        Storage::delete('' . $request->coverUrl);
        Storage::delete('' . $request->bannerUrl);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function uploadCoverImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("banner");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }

    public function uploadBannerImage(Request $request)
    {
        $data = $request->all();
        if (count($data) and $request->type == 'video') {
            $validator = Validator::make($data, [
                'video' => 'required|mimes:mp4|max:100000',
            ]);
        } else {
            $validator = Validator::make($data, [
                'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
            ]);
        }
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file($request->type)->store("banner");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }

    public function removeBanner(Request $request)
    {
        $id = $request->id;
        $banner = Banner::where('id', $id)->delete();
        if ($banner) {
            return response()->json([
                'success' => true,
                'msg' => 'Record deleted successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function saveBanner(Request $request)
    {
        $bannerSections = $request->banner;
        $type = $request->type;
        // echo "<pre>";
        // print_r($bannerSections);die;
        foreach ($bannerSections as $bannerSection) {
            if (isset($bannerSection['id'])) {
                Banner::where('id', $bannerSection['id'])->update([
                    'type_of_banner' => $type,
                    'upload_banner_type' => $bannerSection["upload_banner_type"],
                    'cover_image' => $bannerSection["cover_image"],
                    'banner_url' => $bannerSection["banner_url"],
                    'destination_type' => $bannerSection["destination_type"],
                    'destination' => $bannerSection["destination"],
                    'banner_language' => $bannerSection["banner_language"],
                    'display_order' => $bannerSection["display_order"],
                ]);
            } else {
                Banner::create([
                    'type_of_banner' => $type,
                    'upload_banner_type' => $bannerSection["upload_banner_type"],
                    'cover_image' => $bannerSection["cover_image"],
                    'banner_url' => $bannerSection["banner_url"],
                    'destination_type' => $bannerSection["destination_type"],
                    'destination' => $bannerSection["destination"],
                    'banner_language' => $bannerSection["banner_language"],
                    'display_order' => $bannerSection["display_order"],
                ]);
            }
        }
        $banner = Banner::all();
        foreach ($banner as $bs) {
            $bs['error'] = array("type_of_banner" => "", "upload_banner_type" => "", "banner_url" => "", "cover_image" => "", "english" => "", "banner_language" => "", "arabic" => "", "order" => "");
        }
        return response()->json([
            'success' => true,
            'banner' => $banner,
            'msg' => 'Record saved successfully.'
        ]);
    }

    public function UnlinkCollection(Request $request)
    {
        if ($request->type == 'normal_collection') {
            NormalCollection::where('id', $request->id)->update([
                'collection_id' => 0
            ]);
        } else if ($request->type == 'collection_goal') {
            CollectionByGoal::where('id', $request->id)->update([
                'collection_id' => 0
            ]);
        }

        return response()->json([
            'success' => true,
            'msg' => 'Collection unlink successfully.'
        ]);
    }

    public function getAllCollection()
    {
        try {
            $collection = Collection::select('collection_id', 'title_english')->get();
            return response()->json([
                'success' => true,
                'record' => $collection
            ]);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function getAllChallenge()
    {
        try {
            $challenge = ChallengeSection::select('id', 'title_english')->get();
            return response()->json([
                'success' => true,
                'record' => $challenge
            ]);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function getAllMenu()
    {
        try {
            $menu = Menu::select('id', 'title_english')->get();
            return response()->json([
                'success' => true,
                'record' => $menu
            ]);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
}
