<?php

namespace App\Http\Controllers\Admin\AppSetting\Management;

use App\Exports\UserExport;
use App\Http\Controllers\Controller;
use App\Models\Management;
use App\Models\UserLoginHistory;
use App\User;
use Illuminate\Contracts\Session\Session;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Session as FacadesSession;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Maatwebsite\Excel\Facades\Excel;
use Yajra\DataTables\Facades\DataTables;

class ManagementController extends Controller
{
    public function index()
    {
        return view('admin.app-setting.management.index');
    }
    public function add()
    {
        return view('admin.app-setting.management.add');
    }

    public function getAllUser(Request $request)
    {
        if ($request->ajax()) {
            $data = User::where('role', '=', 0)->latest()->get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="'.url('/').'/admin/app-setting/management/edit/'.$row->id.'" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="'.$row->id.'">Delete</a>';
                    return $btn;
                })
                ->editColumn('status', function ($management) {
                    if ($management->status == 1) {
                        $btn = '<div class="custom-control custom-switch">
                        <input type="checkbox" class="custom-control-input activeSwitch" checked  value="' . $management->status . '" id="customSwitch' . $management->id . '" data-id="' . $management->id . '" data-language="' . $management->language . '">
                        <label class="custom-control-label" for="customSwitch' . $management->id . '"></label>
                      </div>';
                    } else {
                        $btn = '<div class="custom-control custom-switch">
                        <input type="checkbox" class="custom-control-input activeSwitch"  value="' . $management->status . '" id="customSwitch' . $management->id . '" data-id="' . $management->id . '" data-language="' . $management->language . '">
                        <label class="custom-control-label" for="customSwitch' . $management->id . '"></label>
                      </div>';
                    }
                    return $btn;
                })
                ->rawColumns(['status', 'action'])

                ->make(true);
        }
    }

    public function getHistory(Request $request)
    {
        if ($request->ajax()) {
            $data = UserLoginHistory::where('user_id', $request->id)->latest()->get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->make(true);
        }
    }

    public function store(Request $request)
    {

        $data = $request->all();
        $validator = Validator::make($data, [
            'name'      => 'required|unique:users',
            'email'     => 'required|email|unique:users',
            'password'  => 'required',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors'  => $validator->getMessageBag()->toArray(),
                'data'    => array()
            ), 422);
        } else {
            $management = User::create([
                'name'           => $request->name,
                'email'          => $request->email,
                'password'       => Hash::make($request->password),
                'plain_password' =>$request->password,
                'status'         => $request->status,
                'ip'             => $request->ip(),
            ]);
            if ($management) {
                return response()->json([
                    'success'   => true,
                    'msg'       => 'Record saved successfully.'
                ]);
            } else {
                return response()->json([
                    'success' => false,
                    'msg'     => 'Something went wrong.'
                ]);
            }
        }
    }

    public function edit($id)
    {
        $management = User::where('id', $id)->select('id', 'name', 'email', 'plain_password as password','status')->first();
        // $management = User::where('id', $id)->first();
        // echo $management;die;
        // echo Crypt::decrypt($management->password);die;
        return view('admin.app-setting.management.edit', compact('management'));
    }

    public function update(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'name' => 'required|unique:users,name,'.$request->id,
            'email' => 'required|email|unique:users,email,'.$request->id,
            'password' => 'required',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $management = User::where('id', $request->id)->update([
                'name'           => $request->name,
                'email'          => $request->email,
                'password'       =>  Hash::make($request->password),
                'plain_password' => $request->password,
                'status'         => $request->status,
                'ip'             => $request->ip(),
            ]);
            if ($management) {
                return response()->json([
                    'success' => true,
                    'msg' => 'Record updated successfully.'
                ]);
            } else {
                return response()->json([
                    'success' => false,
                    'msg' => 'Something went wrong.'
                ]);
            }
        }

    }
    public function delete(Request $request)
    {
        $management = User::where('id', $request->id)->delete();
        if ($management) {
            return response()->json([
                'success' => true,
                'msg' => 'Record deleted successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function manageStatus(Request $request)
    {
        if ($request->active == 1) {
           $user =  User::where('id', $request->id)->update([
                'status' => 0
            ]);
        } elseif ($request->active == 0) {
            $user = User::where('id', $request->id)->update([
                'status' => 1
            ]);
        }
        if($user){
            if($request->active == 1){
                $sessionDelete = DB::table('sessions')->where('user_id', $request->id)->delete();
            }
            return response()->json([
                'success' => true,
                'msg' => 'Status update successfully.'
            ]);

        }else{
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function export() 
    {
        return Excel::download(new UserExport, 'user.xlsx');
    }
}
