<?php

namespace App\Http\Controllers\Admin\AppSetting\ManualNotification;

use App\Http\Controllers\Controller;
use App\Models\ChallengeSection;
use App\Models\Collection;
use App\Models\ManualNotification;
use App\Models\NormalCollection;
use App\Models\CollectionByGoal;
use App\Models\Menu;
use App\Models\Product;
use App\Models\UserToken;
use Carbon\Carbon;
use DataTables;
use DateTime;
use DateTimeZone;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class ManualNotificationController extends Controller
{
    public function index(){
        return view("admin.app-setting.manual-notification.list");
    }
    public function add(){
        return view("admin.app-setting.manual-notification.add");
    }
    public function store(Request $request){
        $notification = ManualNotification::create([
            "title_english"         => $request->title_english,
            "title_arabic"          => $request->title_arabic,
            "description_english"   => $request->description_english,
            "description_arabic"    => $request->description_arabic,
            "destination_type"      => $request->destination_type,
            "destination"           => $request->destination,
            "date"                  => $request->date,
            "image"                 => $request->image,
        ]);
        if($notification){
            if($notification->date == NULL){
                $response = $this->sendNotification($notification);
                if($response){
                    $notification->status = 1;
                    $notification->save(); 
                    return response()->json([
                        'success' => true,
                        'msg' => 'Record saved successfully.',
                        'record' => $response,
                    ]);
                }else{
                    return response()->json([
                        'success' => false,
                        'msg' => 'Something went wrong!'
                    ]);
                }
            }else{
                return response()->json([
                    'success' => true,
                    'msg' => 'Record saved successfully.'
                ]);
            }
        }else{
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong!'
            ]);
        }
    }
    public function edit($id){
        $notification = ManualNotification::where('id', $id)->first();
        $notification->date = strftime('%Y-%m-%dT%H:%M:%S', strtotime($notification->date));
        return view("admin.app-setting.manual-notification.edit", compact('notification'));
    }
    public function update(Request $request){
        
        $notificationResponse = ManualNotification::where('id', $request->id)->update([
            "title_english"         => $request->title_english,
            "title_arabic"          => $request->title_arabic,
            "description_english"   => $request->description_english,
            "description_arabic"    => $request->description_arabic,
            "destination_type"      => $request->destination_type,
            "destination"           => $request->destination,
            "date"                  => $request->date,
            "image"                 => $request->image,
        ]);
        $notification = ManualNotification::where('id', $request->id)->first();
        if($notificationResponse){
            if($notification->date == NULL && $notification->status == 0){
                
                $response = $this->sendNotification($notification);
                if($response){
                    $notification->status = 1;
                    $notification->save(); 
                    return response()->json([
                        'success' => true,
                        'msg' => 'Record update successfully.',
                        'record' => $response,
                    ]);
                }else{
                    return response()->json([
                        'success' => false,
                        'msg' => 'Something went wrong!'
                    ]);
                }
            }else{
                return response()->json([
                    'success' => true,
                    'msg' => 'Record update successfully.'
                ]);
            }
        }else{
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong!'
            ]);
        }
    }
    public function delete(Request $request){
        $notification = ManualNotification::where('id', $request->id)->first();
        if($notification){
            $notificationDelete = ManualNotification::where('id', $request->id)->delete();
            if($notificationDelete){
                Storage::delete('' . $notification->image);
                return response()->json([
                    'success' => true,
                    'msg' => 'Record deleted successfully.'
                ]);
            }else{
                return response()->json([
                    'success' => false,
                    'msg' => 'Something went wrong !'
                ]);
            }
        }
    }
    public function getAllNotification(Request $request){
        if ($request->ajax()) {
            $data = ManualNotification::get()->map(function($data){
                if($data->destination_type == 'collection'){
                    $collect = Collection::where('collection_id', $data->destination)->first();
                    if($collect){
                        $data->destination = $collect->title_english;
                    }
                }
                if($data->destination_type == 'product'){
                    $product = Product::where('product_id', $data->destination)->first();
                    if($product){
                        $data->destination = $product->title_english;
                    }
                }
                if($data->destination_type == 'challenge'){
                    $challenge = ChallengeSection::where('id', $data->destination)->first();
                    if($challenge){
                        $data->destination = $challenge->title_english;
                    }
                }
                if($data->destination_type == 'menu'){
                    $menu = Menu::where('id', $data->destination)->first();
                    if($menu){
                        $data->destination = $menu->title_english;
                    }
                }
                return $data;
            });
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    if($row->status == 0){
                        $btn = '<a href="' . url('/') . '/admin/app-setting/manual-notification/edit/' . $row->id . '" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="' . $row->id . '">Delete</a>';
                    }else{
                        $btn = '<a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="' . $row->id . '">Delete</a>';
                    }
                    return $btn;
                })
                ->editColumn('status', function ($row){
                    if($row->status == 0){
                        $status = '<span class="btn-pill btn btn-danger">Pending</span>';
                    }else{
                        $status = '<span class="btn-pill btn btn-success">Success</span>';
                    }
                    return $status;
                })
                ->rawColumns(['status','action'])

                ->make(true);
        }
    }
    public function uploadImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg,gif|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file('image')->store("manual-notifiction");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeImage(Request $request)
    {
        Storage::delete('' . $request->url);
        if (isset($request->id) && $request->id) {
            ManualNotification::where('id', $request->id)->where('image', $request->url)->update([
                'image' => null
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function sendNotification($notification){
        $result = NULL;
        $response = [];
        $serverKey = trim(env('SERVER_KEY'));
        if ($notification) {
            $image  = $notification->image ? config('app.url').'/storage/app/' . $notification->image : "";
            $type = 'basic';
            $icon  = "https://image.flaticon.com/icons/png/512/270/270014.png";/*Default Icon*/
            $tokens = UserToken::get();
            $accesstoken = $serverKey;
            $URL = 'https://fcm.googleapis.com/fcm/send';
            $collection_type = "";
            $collection_title = "";
            if($notification->destination_type == 'menu'){
                $menuData = Menu::where('id', $notification->destination)->first();
                $destinationType = $notification->destination_type;
                $destination = $menuData != NULL ? $menuData->link : NULL;
            }else{
                $destinationType = $notification->destination_type;
                $destination = $notification->destination;
                if($notification->destination_type == 'collection'){
                    $normal = NormalCollection::where('collection_id',$notification->destination)->first();
                    if($normal){
                       $collection_type = "normal_collection"; 
                       $collection_title = $normal->text_english; 
                    }else{
                        $goal = CollectionByGoal::where('collection_id',$notification->destination)->first();
                        if($goal){
                           $collection_type = "collection_by_goal"; 
                            $collection_title = $goal->title_english; 
                        }
                    }
                    
                }
            }
            foreach($tokens as $token){
                $notification_id = trim($token->device_token);
                if($token->language_id == 'en'){
                    $title = $notification->title_english;
                    $message = strip_tags($notification->description_english);
                    
                }elseif($token->language_id == 'ar'){
                    $title = $notification->title_arabic;
                    $message = strip_tags($notification->description_arabic);
                }
                
                $post_data = '{
                "to" : "' . $notification_id . '",
                "data" : {
                    "body" : "",
                    "title" : "' . $title . '",
                    "type" : "' . $type . '",
                    "message" : "' . $message . '",
                    "notification_type": "manual",
                    "manual_id": "'.$notification->id.'",
                    "destination_type": "'.$destinationType.'",
                    "destination": "'.$destination.'",
                    "collection_type": "'.$collection_type.'",
                    "collection_title": "'.$collection_title.'",
                    "variant_id": "0" ,
                    "product_id": "0" ,
                },
                "notification" : {
                    "body" : "' . $message . '",
                    "title" : "' . $title . '",
                    "type" : "' . $type . '",
                    "largeIcon"	: "large_icon",
                    "message" : "' . $message . '",
                    "icon" : "' . $icon . '",
                    "image" : "' . $image . '",
                    "sound" : "1",
                    "badge" : "1",
                    "mutable-content": "1",
                    "notification_type": "manual",
                    "manual_id": "'.$notification->id.'",
                    "destination_type": "'.$destinationType.'",
                    "destination": "'.$destination.'",
                    "collection_type": "'.$collection_type.'",
                    "collection_title": "'.$collection_title.'",
                    "variant_id": "0" ,
                    "product_id": "0" ,
                    },
                    
                }';

                $crl = curl_init();
    
                $headr = array();
                $headr[] = 'Content-type: application/json';
                $headr[] = 'Authorization: key=' . $accesstoken;
    
                curl_setopt($crl, CURLOPT_SSL_VERIFYPEER, false);
    
                curl_setopt($crl, CURLOPT_URL, $URL);
                curl_setopt($crl, CURLOPT_HTTPHEADER, $headr);
    
                curl_setopt($crl, CURLOPT_POST, true);
                curl_setopt($crl, CURLOPT_POSTFIELDS, $post_data);
                curl_setopt($crl, CURLOPT_RETURNTRANSFER, true);
    
                $result = curl_exec($crl);

                if ($result === FALSE) {
                    die('FCM Send Error: ' . curl_error($crl));
                } else {
                    // $result = json_decode($result,true);
                    $response[] = $result;
    
                    curl_close($crl);
                }
            }
        }
        return $response;
    }

}
