<?php

namespace App\Http\Controllers\Admin\AppSetting\Menu;

use Response;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Controller;
use DataTables;
use App\Models\Menu;
use App\Models\MenuClickCount;

class MenuController extends Controller
{

    public function __construct()
    {
    }

    public function add()
    {
        return view('admin.app-setting.menu.add');
    }

    public function edit($id)
    {
        $menu = Menu::where('id', $id)->first();
        return view('admin.app-setting.menu.edit', compact('menu'));
    }

    public function index()
    {
        return view('admin.app-setting.menu.index');
    }

    public function getAllMenu(Request $request)
    {
        if ($request->ajax()) {
            $data = Menu::latest()->get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="'.url('/').'/admin/app-setting/menu/edit/'.$row->id.'" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="'.$row->id.'">Delete</a>';
                    return $btn;
                })
                ->editColumn('icon', function ($menu) {
                    $url = url('/') . '/../storage/app/' . $menu->icon;
                    $icon = '<img src="' . $url . '" style="height:40px;width:40px;" />';
                    return $icon;
                })
                ->rawColumns(['icon', 'action'])

                ->make(true);
        }
    }

    public function store(Request $request)
    {
        $menu = Menu::create([
            'title_english' => $request->title_english,
            'title_arabic' => $request->title_arabic,
            'icon' => $request->icon,
            'link' => $request->link,
        ]);
        MenuClickCount::create([
            'menu'=>$request->title_english,
            'count'=>0
        ]);
        if ($menu) {
            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function update(Request $request)
    {
        $menu = Menu::where('id', $request->id)->update([
            'title_english' => $request->title_english,
            'title_arabic' => $request->title_arabic,
            'icon' => $request->icon,
            'link' => $request->link,
        ]);
        return response()->json([
            'success' => true,
            'msg' => 'Record updated successfully.'
        ]);
    }
    public function delete(Request $request)
    {
        $menuDetail = Menu::where('id', $request->id)->first();
        $menu = Menu::where('id', $request->id)->delete();
        if ($menu) {
            Storage::delete('' . $menuDetail->icon);
            return response()->json([
                'success' => true,
                'msg' => 'Record deleted successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function uploadIcon(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("menu");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeIcon(Request $request)
    {
        Menu::where('icon', $request->url)->update([
            'icon' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Icon removed successfully.'
        ]);
    }
}
