<?php

namespace App\Http\Controllers\Admin\AppSetting\Milestone;

use App\Http\Controllers\Admin\Helper\HelperController;
use App\Http\Controllers\Controller;
use App\Models\Currency;
use App\Models\CurrencyPoint;
use App\Models\Customer;
use App\Models\Milestone;
use App\Models\MilestonePoint;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\Facades\DataTables;

class MilestoneController extends Controller
{
    public function add()
    {
        return view('admin.app-setting.milestone.add');
    }

    public function getMilestone(Request $request){
        $milestone = Milestone::get();
        return response()->json([
            'success' => true,
            'milestone' => $milestone
        ]);
    }



    public function store(Request $request)
    {
        $milestones = $request->all();
        foreach($milestones as $milestone){
            if (isset($milestone['id'])) {
                $milestoneResult = Milestone::where('id', $milestone['id'])->update([
                    "level_english"         => $milestone['level_english'],
                    "level_arabic"          => $milestone['level_arabic'],
                    "milestone_image"       => $milestone['milestone_image'],
                    "from_point"            => $milestone['from_point'],
                    "to_point"              => $milestone['to_point'],
                    "prize_english"         => $milestone['prize_english'],
                    "prize_arabic"          => $milestone['prize_arabic'],
                    "note_english"          => $milestone['note_english'],
                    "note_arabic"           => $milestone['note_arabic'],
                    "optional_note_english" => $milestone['optional_note_english'],
                    "optional_note_arabic"  => $milestone['optional_note_arabic'],
                ]);
            }else{
                $milestoneResult = Milestone::create([
                    "level_english"         => $milestone['level_english'],
                    "level_arabic"          => $milestone['level_arabic'],
                    "milestone_image"       => $milestone['milestone_image'],
                    "from_point"            => $milestone['from_point'],
                    "to_point"              => $milestone['to_point'],
                    "prize_english"         => $milestone['prize_english'],
                    "prize_arabic"          => $milestone['prize_arabic'],
                    "note_english"          => $milestone['note_english'],
                    "note_arabic"           => $milestone['note_arabic'],
                    "optional_note_english" => $milestone['optional_note_english'],
                    "optional_note_arabic"  => $milestone['optional_note_arabic'],
                ]);
            }
        }
        if ($milestoneResult) {
            $milestoneData = Milestone::get();
            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.',
                'milestone' => $milestoneData
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function getCustomerMilestone()
    {
        $milestone = Customer::select('customer_id' ,'email', 'extra_point as point')->get();
        return response()->json([
            'success' => true,
            'milestone' => $milestone
        ]);
    }

    public function getCurrency(Request $request)
    {
        $currencies = Currency::get();
        $data = [];
        $customerData = [];
        foreach($currencies as $currency){
            $data = [];
            $data['value'] = $currency->code;
            $data['text'] = $currency->country." (".$currency->code.")";
            $currencyData[] = $data;

        }

        return response()->json([
            'success' => true,
            'currency' => $currencyData
        ]);
    }

    public function getCurrencyDetail(Request $request)
    {

        $existCurrency = CurrencyPoint::where('currency', $request->code)->select('id','currency as code', 'each_value', 'each_point')->first();
        if($existCurrency){
            return response()->json([
                'success' => true,
                'currency' => $existCurrency
            ]);
        }else{
            $currency = Currency::where('code', $request->code)->select('code')->first();
            $currency['each_value'] = "";
            $currency['each_point'] = "";
            return response()->json([
                'success' => true,
                'currency' => $currency
            ]);
        }
    }

    public function storeCurrencyPoint(Request $request)
    {
        $exist = CurrencyPoint::where('currency', $request->code)->first();
        if ($exist) {
            $contentResult = CurrencyPoint::where('currency', $request->code)->update([
                "each_value" => abs($request->each_value),
                "each_point" => abs($request->each_point),
            ]);
        }else{
            $contentResult = CurrencyPoint::create([
                "currency"   => $request->code,
                "each_value" => abs($request->each_value),
                "each_point" => abs($request->each_point),
            ]);
        }
        if($contentResult){
            return response()->json([
                'success'   => true,
                'msg'       => 'Record saved successfully.',
            ]);
        }else{
            return response()->json([
                'success'   => false,
                'msg'       => 'Something went wrong.'
            ]);
        }
    }

    public function getAllCustomer(Request $request)
    {
        // ini_set('memory_limit', '-1');
        $customers = Customer::get();
        $data = [];
        $customerData = [];
        foreach($customers as $customer){
            $data = [];
            $data['value'] = $customer->customer_id;
            $data['text'] = $customer->first_name.' '.$customer->last_name;
            $customerData[] = $data;

        }

        return response()->json([
            'success' => true,
            'customer' => $customerData
        ]);
    }

    public function getCustomerDetail(Request $request)
    {

        // $existMilestone = MilestonePoint::where('customer_id', $request->customer_id)->select('id','customer_id', 'email', 'point as current_point')->first();
        $existMilestone = Customer::where('customer_id', $request->customer_id)->select('id','customer_id', 'email', 'extra_point as current_point')->first();
        if($existMilestone){
            $existMilestone['point'] = "";
            return response()->json([
                'success' => true,
                'milestone' => $existMilestone
            ]);
        }else{
            return response()->json([
                'success' => false,
                'msg' => "customer not exist"
            ]);
        }
    }

    public function storeCustomerMilestone(Request $request)
    {
        $exist = Customer::where('customer_id', $request->customer_id)->first();
        if ($exist) {
            $totalPointBefore = $exist->point + $exist->extra_point;
            $levelBefore = $this->getLevel($totalPointBefore);
            $contentResult = Customer::where('customer_id', $request->customer_id)->update([
                "extra_point" => $exist->extra_point + $request->point,
            ]);
            if($contentResult){
                $existCustomer      = Customer::where('customer_id', $request->customer_id)->first();
                $totalPointAfter    = $existCustomer->point + $existCustomer->extra_point;
                $levelAfter         = $this->getLevel($totalPointAfter);
                $milestone_id       = $this->getMilestoneId($totalPointAfter);
                if (trim($levelBefore) != trim($levelAfter)) {
                    $milestoneEntry = Customer::where('customer_id', $request->customer_id)->update([
                        "milestone_id"      => $milestone_id,
                        "level"             => $levelAfter,
                        "milestone_entry"   => Carbon::now('Asia/Kuwait'),
                    ]);
                }
            }

        }
        if($contentResult){
            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.',
            ]);
        }else{
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function getLevel($point){
        $max = Milestone::max('from_point');
        if($point >= $max){
            $milestone = Milestone::where('from_point', '=',$max)->select('level_english')->first();
            if($milestone){
                return $milestone->level_english;
            }else{
                return 'Not Available';
            }
            // echo $milestone->level;die;
        }else{
            $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('level_english')->first();
            if($milestone){
                return $milestone->level_english;
            }else{
                return 'Not Available';
            }
            // echo $milestone->level;die;
        }
    }

    public function getMilestoneId($point){

        $max = Milestone::max('from_point');
        if($point >= $max){
            $milestone = Milestone::where('from_point', '=',$max)->select('id', 'level_english')->first();
            if($milestone){
                return $milestone->id;
            }else{
                return 0;
            }
        }else{
            $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('id', 'level_english')->first();
            if($milestone){
                return $milestone->id;
            }else{
                return 0;
            }
        }
    }

    public function deleteCustomerMilestone(Request $request)
    {
        $exist = Customer::where('customer_id', $request->customer_id)->first();
        if ($exist) {
            $totalPointBefore   = $exist->point + $exist->extra_point;
            $levelBefore        = $this->getLevel($totalPointBefore);
            $contentResult      = Customer::where('customer_id', $request->customer_id)->update([
                "extra_point" => $exist->extra_point - $request->point,
            ]);
            if($contentResult){
                $existCustomer      = Customer::where('customer_id', $request->customer_id)->first();
                $totalPointAfter    = $existCustomer->point + $existCustomer->extra_point;
                $levelAfter         = $this->getLevel($totalPointAfter);
                $milestone_id       = $this->getMilestoneId($totalPointAfter);

                if (trim($levelBefore) != trim($levelAfter)) {
                    $milestoneEntry = Customer::where('customer_id', $request->customer_id)->update([
                        "milestone_id"      => $milestone_id,
                        "level"             => $levelAfter,
                        "milestone_entry"   => Carbon::now('Asia/Kuwait'),
                    ]);
                }
            }

        }
        // $exist = Customer::where('customer_id', $request->customer_id)->first();
        // if ($exist) {
        //     $contentResult = Customer::where('customer_id', $request->customer_id)->update([
        //         "extra_point" => $exist->extra_point - $request->point,
        //     ]);
        // }
        if($contentResult){
            return response()->json([
                'success' => true,
                'msg' => 'Record remove successfully.',
            ]);
        }else{
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    public function uploadImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("milestone");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removePrizeEnglish(Request $request)
    {
        Milestone::where('prize_english', $request->url)->update([
            'prize_english' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'prize english removed successfully.'
        ]);
    }
    public function removePrizeArabic(Request $request)
    {
        Milestone::where('prize_arabic', $request->url)->update([
            'prize_arabic' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'prize arabic removed successfully.'
        ]);
    }
    public function removeMilestoneImage(Request $request)
    {
        Milestone::where('milestone_image', $request->url)->update([
            'milestone_image' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'prize arabic removed successfully.'
        ]);
    }
    
    public function getCustomerBySeacrh(Request $request)
    {
        $customers = Customer::Where('first_name', 'like', '%' . $request->keyword . '%')->orWhere('last_name', 'like', '%' . $request->keyword . '%')->get();
        $data = [];
        $customerData = [];
        foreach($customers as $customer){
            $data = [];
            $data['value'] = $customer->customer_id;
            $data['text'] = $customer->first_name.' '.$customer->last_name;
            $customerData[] = $data;

        }

        return response()->json([
            'success' => true,
            'customer' => $customerData
        ]);
    }

}

