<?php

namespace App\Http\Controllers\Admin\AppSetting\Product;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\Variant;
use App\Models\Ingredients;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Validator;
use Response;
use Illuminate\Support\Arr;
use DataTables;


class ProductController extends Controller
{
    public $client;
    public function __construct()
    {
        $this->client = new Client();
    }

    public function products(Request $request)
    {

        $productData = [];
        $productCreateData = [];
        $productIdData = [];
        $variantIdData = [];
        $res = $this->client->request(
            'GET',
            env('SHOPIFY_API_REQUEST_URL') . 'products.json'
        );
        if ($res->getStatusCode() == 200) {
            $product_data = $res->getBody()->getContents();
            $products = json_decode($product_data)->products;
            $variantsCreateData = [];
            $variantsUpdateData = [];
            foreach ($products as $product) {

                array_push($productIdData, $product->id);
                $img = [];
                $variantsUpdate = [];
                $variantsCreate = [];
                if (count($product->images) > 0) {
                    foreach ($product->images as $image) {
                        $img[$image->id] = $image->src;
                    }
                }

                $productData['title_english'] = $product->title;
                $productData['title_arabic'] = "";
                $productData['description_english'] = $product->body_html;
                $productData['description_withouthtml_english'] = strip_tags($product->body_html);
                $productData['description_arabic'] = "";
                $productData['description_withouthtml_arabic'] = "";
                $productData['product_id'] = $product->id;
                $productData['image'] = $product->image->src;
                if (count($product->variants) > 0) {
                    $productData['inventory_policy'] = $product->variants[0]->inventory_policy;
                    $productData['stock'] = $product->variants[0]->inventory_quantity;
                    $productData['price'] = $product->variants[0]->price;
                    $productData['strike_price'] = $product->variants[0]->compare_at_price;
                    $productData['variant_id'] = $product->variants[0]->id;
                    if (array_key_exists($product->variants[0]->image_id, $img)) {
                        $productData['image'] = !empty($product->variants[0]->image_id) ? $img[$product->variants[0]->image_id] : $product->image->src;
                    }
                    foreach ($product->variants as $variant) {

                        array_push($variantIdData, $variant->id);

                        $variantCheck = Variant::where('variant_id', $variant->id)->first();
                        $status = NULL;
                        if ($variant->inventory_quantity != NULL) {
                            $status = 'add_to_cart';
                        } else {
                            if ($variant->inventory_policy == 'deny') {
                                $status = 'notify_me';
                            } else if ($variant->inventory_policy == 'continue') {
                                $status = 'add_to_cart';
                            }
                        }
                        if ($variantCheck != null) {
                            $variantsUpdate['inventory_policy'] = $variant->inventory_policy;
                            $variantsUpdate['stock'] = $variant->inventory_quantity;
                            $variantsUpdate['price'] = $variant->price;
                            $variantsUpdate['strike_price'] = $variant->compare_at_price;
                            $variantsUpdate['status']     = $variantCheck->status == 'pre_order' ? 'pre_order' :  $status;
                            $variantsUpdate['variant_id'] = $variant->id;
                            if (array_key_exists($variant->image_id, $img)) {
                                $variantsUpdate['image'] = !empty($variant->image_id) ? $img[$variant->image_id] : $product->image->src;
                            } else {
                                $variantsUpdate['image'] = $product->image->src;
                            }
                            $variantsUpdateData[] = $variantsUpdate;
                        } else {
                            $variantsCreate['option1_english'] = $variant->option1;
                            $variantsCreate['option2_english'] = $variant->option2;
                            $variantsCreate['option3_english'] = $variant->option3;
                            $variantsCreate['option1_arabic'] = "";
                            $variantsCreate['option2_arabic'] = "";
                            $variantsCreate['option3_arabic'] = "";
                            $variantsCreate['inventory_policy'] = $variant->inventory_policy;
                            $variantsCreate['stock'] = $variant->inventory_quantity;
                            $variantsCreate['price'] = $variant->price;
                            $variantsCreate['strike_price'] = $variant->compare_at_price;
                            $variantsCreate['status']     = $status;
                            $variantsCreate['variant_id'] = $variant->id;
                            $variantsCreate['product_id'] = $product->id;

                            if (array_key_exists($variant->image_id, $img)) {
                                $variantsCreate['image'] = !empty($variant->image_id) ? $img[$variant->image_id] : $product->image->src;
                            } else {
                                $variantsCreate['image'] = $product->image->src;
                            }
                            array_push($variantsCreateData, $variantsCreate);
                        }
                    }
                    for ($i = 0; $i < count($product->variants); $i++) {
                        if ($product->variants[$i]->inventory_quantity) {
                            if ($product->variants[$i]->compare_at_price != NULL) {
                                $productData['inventory_policy'] = $product->variants[$i]->inventory_policy;
                                $productData['stock'] = $product->variants[$i]->inventory_quantity;
                                $productData['price'] = $product->variants[$i]->price;
                                $productData['strike_price'] = $product->variants[$i]->compare_at_price;
                                $productData['variant_id'] = $product->variants[$i]->id;
                                if (array_key_exists($product->variants[$i]->image_id, $img)) {
                                    $productData['image'] = !empty($product->variants[$i]->image_id) ? $img[$product->variants[$i]->image_id] : $product->image->src;
                                }
                                break;
                            }
                            $productData['inventory_policy'] = $product->variants[$i]->inventory_policy;
                            $productData['stock'] = $product->variants[$i]->inventory_quantity;
                            $productData['price'] = $product->variants[$i]->price;
                            $productData['strike_price'] = $product->variants[$i]->compare_at_price;
                            $productData['variant_id'] = $product->variants[$i]->id;
                            if (array_key_exists($product->variants[$i]->image_id, $img)) {
                                $productData['image'] = !empty($product->variants[$i]->image_id) ? $img[$product->variants[$i]->image_id] : $product->image->src;
                            }
                        }
                    }
                }
                $productCheck = Product::where('product_id', $productData['product_id'])->first();
                if ($productCheck != null) {
                    $productRes =  Product::where('product_id', $productData['product_id'])->update([
                        'inventory_policy' => $productData['inventory_policy'],
                        'stock' => $productData['stock'],
                        'price' => $productData['price'],
                        'strike_price' => $productData['strike_price'],
                        'variant_id' => $productData['variant_id'],
                        'image' => $productData['image'],
                    ]);
                    if ($productRes) {
                        $this->saveVariant($variantsUpdateData);
                    }
                } else {
                    $productCreateData[] = $productData;
                }
            }
            if ($productCreateData) {
                Product::insert($productCreateData);
            }
            if ($variantsCreateData) {
                Variant::insert($variantsCreateData);
            }
        }
        $variantIds =  Variant::get()->pluck('variant_id')->toArray();
        $productIds =  Product::get()->pluck('product_id')->toArray();
        if ($productIdData != null && $productIds != null) {
            foreach ($productIds as $productId) {
                if (!in_array($productId, $productIdData)) {
                    Product::where('product_id', $productId)->delete();
                }
            }
        }
        if ($variantIdData != null && $variantIds != null) {
            foreach ($variantIds as $variantId) {
                if (!in_array($variantId, $variantIdData)) {
                    Variant::where('variant_id', $variantId)->delete();
                }
            }
        }
        return response()->json([
            'success' => true,
            'msg' => "Product sync successfully."
        ]);
    }

    public function saveVariant($variants)
    {
        if ($variants != null) {
            foreach ($variants as $variant) {

                $variantCheck = Variant::where('variant_id', $variant['variant_id'])->first();
                if ($variantCheck != null) {
                    $variantRes =  Variant::where('variant_id', $variant['variant_id'])->update([
                        'inventory_policy'  => $variant['inventory_policy'],
                        'stock'             => $variant['stock'],
                        'price'             => $variant['price'],
                        'strike_price'      => $variant['strike_price'],
                        'status'            => $variant['status'],
                        'image'            => $variant['image']
                    ]);
                }
            }
        }
    }

    public function getProducts(Request $request)
    {
        if ($request->ajax()) {
            $data = Product::latest()->get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    if($row->is_delete == 0){
                        $btn = '<a href="' . url('/') . '/admin/product/productedit/' . $row->id . '" class="edit btn btn-primary btn-sm">Edit</a>';
                    }else{
                        $btn = '<span class="btn-pill btn btn-danger">Deleted</span>';
                    }
                    return $btn;
                })
                ->editColumn('active', function ($menu) {
                    if($menu->is_delete == 0){
                        if ($menu->active == 1) {
                            $btn = '<div class="custom-control custom-switch">
                            <input type="checkbox" class="custom-control-input activeSwitch" checked  value="' . $menu->active . '" id="customSwitch' . $menu->id . '" data-id="' . $menu->id . '" data-language="' . $menu->language . '">
                            <label class="custom-control-label" for="customSwitch' . $menu->id . '"></label>
                          </div>';
                        } else {
                            $btn = '<div class="custom-control custom-switch">
                            <input type="checkbox" class="custom-control-input activeSwitch"  value="' . $menu->active . '" id="customSwitch' . $menu->id . '" data-id="' . $menu->id . '" data-language="' . $menu->language . '">
                            <label class="custom-control-label" for="customSwitch' . $menu->id . '"></label>
                          </div>';
                        }
                        return $btn;
                    }
                })
                ->editColumn('stock', function ($row) {
                    $stockSum = [];
                    $stocks = Variant::where('product_id', $row->product_id)->pluck('stock');
                    foreach($stocks as $stock)
                    {
                        array_push($stockSum, $stock);
                    }
                    return array_sum($stockSum);
                })
                ->rawColumns(['stock', 'active','action'])

                ->make(true);
        }
    }

    public function index()
    {
        return view('admin.product.product-list');
    }

    public function edit($id)
    {
        $product = Product::where('id', $id)->first();
        return view('admin.product.edit', compact('product'));
    }
    public function update_product(Request $request)
    {
        $id = 0;
        if (isset($request->id)) {
            $sale = Product::where('id', $request->id)->update([
                'title_english' => $request->title_english,
                'title_arabic' => $request->title_arabic,
                'description_english' => $request->description_english,
                'description_withouthtml_english' => strip_tags($request->description_english),
                'description_arabic' => $request->description_arabic,
                'description_withouthtml_arabic' => strip_tags($request->description_arabic),
                'stock' => $request->stock,
                'barcode' => $request->barcode,
            ]);
        }

        $product = Product::where('id', $request->id)->first();
        return response()->json([
            'success' => true,
            'msg' => 'Product Update successfully.',
            'sale' => $product
        ]);
    }

    public function uploadIngrientsImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg,gif|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file('image')->store("ingredients");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeIngrientsImage(Request $request)
    {
        Storage::delete('' . $request->url);
        if (isset($request->product_id) && $request->product_id) {
            Ingredients::where('product_id', $request->product_id)->where('icon', $request->url)->update([
                'icon' => null
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function updateIngredients(Request $request)
    {
        $ingredients = $request->ingredients;
        $count = 0;
        foreach ($ingredients as $ing) {
            if( $count < 4){
                if (isset($ing['id']) && $ing['id']) {
                    Ingredients::where('id', $ing['id'])->update([
                        'title_english' => $ing['title_english'],
                        'title_arabic' => $ing['title_arabic'],
                        'icon' => $ing['icon'],
                    ]);
                } else {
                    Ingredients::create([
                        'title_english' => $ing['title_english'],
                        'product_id' => $request->product_id,
                        'title_arabic' => $ing['title_arabic'],
                        'icon' => $ing['icon'],
                    ]);
                }
            }
            $count++;
        }
        $ingred = Ingredients::where('product_id', $request->product_id)->get()->toArray();
        if (count($ingred)) {
            return response()->json([
                'success' => true,
                'record' => $ingred,
                'msg' => "Record saved successfully."
            ]);
        } else {
            return response()->json([
                'success' => true,
                'record' => [],
                'msg' => "No record found."
            ]);
        }
    }

    public function getAllIngredients(Request $request)
    {
        $ingred = Ingredients::where('product_id', $request->product_id)->get()->toArray();
        if (count($ingred)) {
            return response()->json([
                'success' => true,
                'record' => $ingred,
                'msg' => "Record saved successfully."
            ]);
        } else {
            return response()->json([
                'success' => true,
                'record' => [],
                'msg' => "No record found."
            ]);
        }
    }
    public function removeIngrients(Request $request)
    {
        // echo $request->id;die;
        $ingred = Ingredients::where('id', $request->id)->delete();
        if ($ingred) {
            return response()->json([
                'success' => true,
                'msg' => "Record delete successfully."
            ]);
        } else {
            return response()->json([
                'success' => true,
                'msg' => "Something went wrong."
            ]);
        }
    }
    public function manageStatus(Request $request)
    {
        if ($request->active == 1) {
            Product::where('id', $request->id)->update([
                'active' => 0
            ]);
        } elseif ($request->active == 0) {
            Product::where('id', $request->id)->update([
                'active' => 1
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Status update successfully.'
        ]);
    }
}
