<?php

namespace App\Http\Controllers\Admin\AppSetting\Product;

use App\Http\Controllers\Controller;
use App\Models\ProductVideo;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class ProductVideoController extends Controller
{
    public function uploadProductCoverImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg,gif|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file('image')->store("product-videos");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeProductCoverImage(Request $request)
    {
        Storage::delete('' . $request->url);
        if (isset($request->product_id) && $request->product_id) {
            ProductVideo::where('product_id', $request->product_id)->where('cover_image', $request->url)->update([
                'cover_image' => null
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function uploadProductVideo(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'video' => 'required|mimes:mp4|max:100000',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file('video')->store("product-videos");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }
    public function removeProductVideo(Request $request)
    {
        Storage::delete('' . $request->url);
        if (isset($request->product_id) && $request->product_id) {
            ProductVideo::where('product_id', $request->product_id)->where('video', $request->url)->update([
                'video' => null
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Video removed successfully.'
        ]);
    }

    public function updateProductVideo(Request $request)
    {
        $videos = $request->productvideos;
        foreach ($videos as $video) {
            if (isset($video['id']) && $video['id']) {
                ProductVideo::where('id', $video['id'])->update([
                    'cover_image' => $video['cover_image'],
                    'video' => $video['video'],
                ]);
            } else {
                ProductVideo::create([
                    'product_id' => $request->product_id,
                    'cover_image' => $video['cover_image'],
                    'video' => $video['video'],
                ]);
            }
        }
        $productVideos = ProductVideo::where('product_id', $request->product_id)->get()->toArray();
        if (count($productVideos)) {
            return response()->json([
                'success' => true,
                'record' => $productVideos,
                'msg' => "Record saved successfully."
            ]);
        } else {
            return response()->json([
                'success' => true,
                'record' => [],
                'msg' => "No record found."
            ]);
        }
    }

    public function getAllProductVideos(Request $request)
    {
        $ingred = ProductVideo::where('product_id', $request->product_id)->get()->toArray();
        if (count($ingred)) {
            return response()->json([
                'success' => true,
                'record' => $ingred,
                'msg' => "Record saved successfully."
            ]);
        } else {
            return response()->json([
                'success' => true,
                'record' => [],
                'msg' => "No record found."
            ]);
        }
    }

    public function deleteProductVideo(Request $request)
    {
        $id = $request->id;
        $deleteVideo = ProductVideo::where('id', $id)->delete();
        if ($deleteVideo) {
            Storage::delete('' . $request->video);
            Storage::delete('' . $request->cover_image);
            return response()->json([
                'success' => true,
                'msg' => 'Record deleted successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
}
