<?php

namespace App\Http\Controllers\Admin\AppSetting\ProductCategory;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\ProductCategory;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\Facades\DataTables;

class ProductCategoryController extends Controller
{
    public function index()
    {
        return view('admin.app-setting.product-category.list');
    }

    public function getCategoryData(Request $request)
    {
        if ($request->ajax()) {
            $data = ProductCategory::with('product')->latest()->get()->groupBy('product_id');

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="' . url('/') . '/admin/app-setting/product-category/edit/' . $row[0]['unique_id'] . '" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="' . $row[0]['unique_id'] . '">Delete</a>';
                    return $btn;
                })
                ->addColumn('no_of_review', function ($row) {
                    $count = count($row);
                    return $count;
                })
                ->editColumn('title_english', function ($row) {
                    foreach ($row as $r) {
                        $title_eng = $r->product->title_english;
                        return $title_eng;
                    }
                })
                ->editColumn('title_arabic', function ($row) {
                    foreach ($row as $r) {
                        $title_arabic = $r->product->title_arabic;
                        return $title_arabic;
                    }
                })
                ->rawColumns(['title_english', 'title_arabic', 'no_of_review', 'action'])

                ->make(true);
        }
    }


    public function add()
    {
        $product = Product::active()->where('is_delete', '=', 0)->get();
        return view('admin.app-setting.product-category.add', compact('product'));
    }

    public function store(Request $request)
    {
        $exist = ProductCategory::where('product_id', $request->product)->first();
        if($exist){
            return response()->json([
                'success' => false,
                'status' => true,
                'msg' => 'This product is already selected.'
            ]);
        }else{
            $unique_id = $this->generateRandomString();
            $data = [];
            $categoryData = [];
            foreach ($request->category as $cat) {
                $data['title_english']  = $cat['title_english'];
                $data['title_arabic']   = $cat['title_arabic'];
                $data['product_id']     = $request->product;
                $data['unique_id']      = $unique_id;
                $data['upload_type']    = $cat['upload_type'];
                $data['cover_image']    = $cat['cover_image'];
                $data['image']          = $cat['image'];
                $data['display_order']  = $cat['display_order'];
                $data['language']       = $cat['language'];
                $data['created_at']     = Carbon::now()->toDateTimeString();
                $data['updated_at']     = Carbon::now()->toDateTimeString();
                $categoryData[] = $data;
            }
            $category = ProductCategory::insert($categoryData);
            if ($category) {
                return response()->json([
                    'success' => true,
                    'msg' => 'Record saved successfully.'
                ]);
            } else {
                return response()->json([
                    'success' => false,
                    'status' => false,
                    'msg' => 'Something went wrong.'
                ]);
            }
        }
    }

    function generateRandomString($length = 10)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    public function uploadImage(Request $request)
    {
        $data = $request->all();

        if ($request->type != '') {
            if (count($data) and $request->type == 'video') {
                $validator = Validator::make($data, [
                    'video' => 'required|mimes:mp4|max:100000',
                ]);
            } else {
                $validator = Validator::make($data, [
                    'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
                ]);
            }
            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                    'data' => array()
                ), 200);
            } else {
                $path = $request->file($request->type)->store("product-category");
                return Response::json(array(
                    'success' => true,
                    'errors' => $validator->getMessageBag()->toArray(),
                    'data' => array('url' => $path)
                ), 200);
            }
        } else {
            $a[] = 'Please select upload type first';
            $b['image'] = $a;
            return Response::json(array(
                'success' => false,
                'errors' => $b,
                'data' => array()
            ), 200);
        }
    }

    public function edit($id)
    {
        $productCategory = ProductCategory::where('unique_id', $id)->get();
        foreach ($productCategory as $pc) {
            $pc['error'] = array("image" => "", "cover_image" => "", "english" => "", "arabic" => "", "order" => "");
        }
        return view('admin.app-setting.product-category.update', compact('productCategory'));
    }

    public function update(Request $request)
    {
        $categories = $request->category;
        $product = $request->product;
        $unique_id = $request->category[0]['unique_id'];
        $exist = ProductCategory::where('unique_id', $unique_id)->first();
        if($exist->product_id != $request->product){
            return response()->json([
                'success' => false,
                'status' => true,
                'msg' => 'This product is already selected.'
            ]);
        }else{
            foreach ($categories as $category) {
                if (isset($category['id'])) {
                    ProductCategory::where('id', $category['id'])->update([
                        'title_english' => $category["title_english"],
                        'title_arabic'  => $category["title_arabic"],
                        'image'         => $category["image"],
                        'display_order' => $category["display_order"],
                        'language'      => $category["language"],
                        'product_id'    => $product,
                        'unique_id'     => $unique_id,
                        'upload_type'   => $category['upload_type'],
                        'cover_image'   => $category['cover_image'],
                    ]);
                } else {
                    ProductCategory::create([
                        'title_english' => $category["title_english"],
                        'title_arabic'  => $category["title_arabic"],
                        'image'         => $category["image"],
                        'display_order' => $category["display_order"],
                        'language'      => $category["language"],
                        'product_id'    => $product,
                        'unique_id'     => $unique_id,
                        'upload_type'   => $category['upload_type'],
                        'cover_image'   => $category['cover_image'],
                    ]);
                }
            }
            $productCategory = ProductCategory::where('unique_id', $unique_id)->get();
            foreach ($productCategory as $pc) {
                $pc['error'] = array("image" => "", "english" => "", "arabic" => "", "order" => "");
            }
            return response()->json([
                'success' => true,
                'category' => $productCategory,
                'msg' => 'Record update successfully.'
            ]);
        }
    }

    public function getProducts()
    {

        $products = Product::active()->where('is_delete', '=', 0)->get();
        return response()->json([
            'success' => true,
            'product' => $products,
        ]);
    }

    public function removeCategory(Request $request)
    {
        $id = $request->id;
        $category = ProductCategory::where('id', $id)->delete();
        if ($category) {
            Storage::delete('' . $request->image);
            Storage::delete('' . $request->cover_image);
            return response()->json([
                'success' => true,
                'msg' => 'Record deleted successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function removeImage(Request $request)
    {
        ProductCategory::where('image', $request->url)->update([
            'image' => null
        ]);
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function uploadCoverImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 422);
        } else {
            $path = $request->file('image')->store("product-category");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }

    public function removeCoverImage(Request $request)
    {
        ProductCategory::where('cover_image', $request->cover_image)->update([
            'cover_image' => null,
            // 'image' => null,
        ]);
        Storage::delete('' . $request->cover_image);
        // Storage::delete('' . $request->image);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function removeCoverAndImage(Request $request)
    {
        ProductCategory::where('cover_image', $request->cover_image)->orWhere('image', $request->image)->update([
            'cover_image' => null,
            'image' => null,
        ]);
        Storage::delete('' . $request->cover_image);
        Storage::delete('' . $request->image);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function delete(Request $request)
    {
        $id = $request->id;
        $images = ProductCategory::where('unique_id', $id)->select('cover_image', 'image')->get();
        $category = ProductCategory::where('unique_id', $id)->delete();
        if ($category) {
            $isValid = false;
            foreach ($images as $image) {
                Storage::delete('' . $image->cover_image);
                Storage::delete('' . $image->image);
                $isValid = true;
            }
            if ($isValid) {
                return response()->json([
                    'success' => true,
                    'msg' => 'Record deleted successfully.'
                ]);
            } else {
                return response()->json([
                    'success' => false,
                    'msg' => 'Something went wrong.'
                ]);
            }
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
}
