<?php

namespace App\Http\Controllers\Admin\AppSetting\Recommend;

use Response;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Controller;
use DataTables;
use App\Models\Menu;
use App\Models\Product;
use App\Models\RecommendProduct;
use Exception;

class RecommendController extends Controller
{

    public function __construct()
    {
    }

    public function add()
    {
        return view('admin.app-setting.recommend-product.add');
    }

    public function edit($id)
    {
        $product = [];
        $recommend = RecommendProduct::where('generated_unique_id', $id)->get();
        foreach ($recommend as $re) {
            $product['product_id'] = $re->product_id;
            $product['generated_unique_id'] = $re->generated_unique_id;
            $content[] = array('id' => $re->id, 'content_product_id' => $re->content_product_id, 'display_order' => $re->display_order);
        }
        $product['error'] = "";
        $product['content'] = $content;
        return view('admin.app-setting.recommend-product.edit', compact('product'));
    }

    public function index()
    {
        return view('admin.app-setting.recommend-product.index');
    }

    public function getAllRecommendProduct(Request $request)
    {
        if ($request->ajax()) {
            $data = RecommendProduct::with('product')->latest()->get()->groupBy('product_id');

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="' . url('/') . '/admin/app-setting/recommend-product/edit/' . $row[0]['generated_unique_id'] . '" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="' . $row[0]['generated_unique_id'] . '">Delete</a>';
                    return $btn;
                })
                ->addColumn('recommend_product', function ($row) {
                    $count = count($row);
                    return $count;
                })
                ->editColumn('title_english', function ($row) {
                    foreach ($row as $r) {
                        $title_eng = $r->product->title_english;
                        return $title_eng;
                    }
                })
                ->editColumn('title_arabic', function ($row) {
                    foreach ($row as $r) {
                        $title_arabic = $r->product->title_arabic;
                        return $title_arabic;
                    }
                })
                ->rawColumns(['title_english', 'title_arabic', 'recommend_product', 'action'])

                ->make(true);
        }
    }

    public function store(Request $request)
    {
        $product = [];
        $data = $request->all();
        // echo "<pre>";
        // print_r($data);die;
        $exist = RecommendProduct::where('product_id', $data['product_id'])->first();
        if($exist){
            return response()->json([
                'success' => false,
                'status' => true,
                'msg' => 'This product is already selected.'
            ]);
        }else{
            $generated_unique_id = $this->generateRandomString();
            foreach ($data['content'] as $cont) {
                $product[] = array('product_id' => $data['product_id'], 'content_product_id' => $cont['content_product_id'], 'display_order' => $cont['display_order'], 'generated_unique_id' => $generated_unique_id);
            }
    
            $recommend = RecommendProduct::insert($product);
    
            if ($recommend) {
                return response()->json([
                    'success' => true,
                    'msg' => 'Record saved successfully.'
                ]);
            } else {
                return response()->json([
                    'success' => false,
                    'status' => false,
                    'msg' => 'Something went wrong.'
                ]);
            }
        }
    }
    public function update(Request $request)
    {

        try {

            $data = $request->all();
            // echo "<pre>";
            // print_r($data);die;
            $exist = RecommendProduct::where('generated_unique_id', $data['generated_unique_id'])->first();
            if($exist->product_id != $data['product_id']){
                return response()->json([
                    'success' => false,
                    'status' => true,
                    'msg' => 'This product is already selected.'
                ]);
            }else{
                foreach ($data['content'] as $cont) {
                    if (isset($cont['id'])) {
                        RecommendProduct::where('id', $cont['id'])->update([
                            'content_product_id' => $cont['content_product_id'],
                            'display_order' => $cont['display_order'],
                            'product_id' => $data['product_id']
                        ]);
                    } else {
                        RecommendProduct::create([
                            'product_id' => $data['product_id'],
                            'content_product_id' => $cont['content_product_id'],
                            'display_order' => $cont['display_order'],
                            'generated_unique_id' => $data['generated_unique_id'],
                        ]);
                    }
                }
                $product = [];
                $content = [];
                $recommend = RecommendProduct::where('generated_unique_id', $data['generated_unique_id'])->get();
                foreach ($recommend as $re) {
                    $product['product_id'] = $re->product_id;
                    $product['generated_unique_id'] = $re->generated_unique_id;
                    $content[] = array('id' => $re->id, 'content_product_id' => $re->content_product_id, 'display_order' => $re->display_order);
                }
                $product['error'] = "";
                $product['content'] = $content;
                return response()->json([
                    'success' => true,
                    'msg' => 'Record saved successfully.',
                    'record' => $product
                ]);
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'status' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function delete(Request $request)
    {
        RecommendProduct::where('generated_unique_id', $request->id)->delete();
        return response()->json([
            'success' => true,
            'msg' => 'Record deleted successfully.'
        ]);
    }

    public function getAllProduct()
    {
        try {
            $product = Product::active()->where('is_delete', '=', 0)->get();
            return response()->json([
                'success' => true,
                'record' => $product
            ]);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function removeContent(Request $request)
    {
        try {
            RecommendProduct::where('id', $request->id)->delete();
            $product = [];
            $recommend = RecommendProduct::where('generated_unique_id', $request->generated_unique_id)->get();
            foreach ($recommend as $re) {
                $product['product_id'] = $re->product_id;
                $product['generated_unique_id'] = $re->generated_unique_id;
                $content[] = array('id' => $re->id, 'content_product_id' => $re->content_product_id, 'display_order' => $re->display_order);
            }
            $product['content'] = $content;
            return response()->json([
                'success' => true,
                'record' => $product
            ]);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    function generateRandomString($length = 10)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }
}
