<?php

namespace App\Http\Controllers\Admin\AppSetting\Review;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Response;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Storage;
use Yajra\DataTables\Facades\DataTables;
use App\Models\Product;
use App\Models\RecommendProduct;
use App\Models\ReviewProduct;
use Exception;

class ReviewController extends Controller
{

    public function __construct()
    {
    }

    public function add()
    {
        return view('admin.app-setting.review.add');
    }

    public function edit($id)
    {
        $product = [];
        $recommend = ReviewProduct::where('generated_unique_id', $id)->get();
        foreach ($recommend as $re) {
            $product['product_id'] = $re->product_id;
            $product['generated_unique_id'] = $re->generated_unique_id;
            $content[] = array('id' => $re->id, 'title_english' => $re->title_english, 'title_arabic' => $re->title_arabic, 'text_english' => $re->text_english, 'text_arabic' => $re->text_arabic, 'source_icon' => $re->source_icon ? $re->source_icon : '','upload_source' => $re->upload_source ? $re->upload_source : '', 'display_order' => $re->display_order, 'error' => '');
        }
        $product['content'] = $content;
        return view('admin.app-setting.review.edit', compact('product'));
    }

    public function index()
    {

        return view('admin.app-setting.review.index');
    }

    public function getAllReviewProduct(Request $request)
    {
        if ($request->ajax()) {
            $data = ReviewProduct::with('product')->latest()->get()->groupBy('product_id');

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="' . url('/') . '/admin/app-setting/review-product/edit/' . $row[0]['generated_unique_id'] . '" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="' . $row[0]['generated_unique_id'] . '">Delete</a>';
                    return $btn;
                })
                ->addColumn('no_of_review', function ($row) {
                    $count = count($row);
                    return $count;
                })
                ->editColumn('title_english', function ($row) {
                    foreach ($row as $r) {
                        $title_eng = $r->product->title_english;
                        return $title_eng;
                    }
                })
                ->editColumn('title_arabic', function ($row) {
                    foreach ($row as $r) {
                        $title_arabic = $r->product->title_arabic;
                        return $title_arabic;
                    }
                })
                ->rawColumns(['title_english', 'title_arabic', 'no_of_review', 'action'])

                ->make(true);
        }
    }

    public function uploadImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file('image')->store("review");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }

    public function store(Request $request)
    {
        $product = [];
        $data = $request->all();
        $generated_unique_id = $this->generateRandomString();
        foreach ($data['content'] as $cont) {
            $product[] = array('product_id' => $data['product_id'], 'title_english' => $cont['title_english'], 'title_arabic' => $cont['title_arabic'], 'text_arabic' => $cont['text_arabic'], 'text_english' => $cont['text_english'], 'source_icon' => $cont['source_icon'], 'upload_source' => $cont['upload_source'], 'display_order' => $cont['display_order'], 'generated_unique_id' => $generated_unique_id);
        }

        $review = ReviewProduct::insert($product);

        if ($review) {
            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function update(Request $request)
    {
        try {

            $data = $request->all();
            // echo "<pre>";
            // print_r($data);die;
            foreach ($data['content'] as $cont) {
                if (isset($cont['id'])) {
                    ReviewProduct::where('id', $cont['id'])->update([
                        'title_english' => $cont['title_english'],
                        'title_arabic' => $cont['title_arabic'],
                        'text_english' => $cont['text_english'],
                        'text_arabic' => $cont['text_arabic'],
                        'source_icon' => $cont['source_icon'],
                        'upload_source' => $cont['upload_source'],
                        'display_order' => $cont['display_order'],
                        'product_id' => $data['product_id']
                    ]);
                } else {
                    ReviewProduct::create([
                        'title_english' => $cont['title_english'],
                        'title_arabic' => $cont['title_arabic'],
                        'text_english' => $cont['text_english'],
                        'text_arabic' => $cont['text_arabic'],
                        'source_icon' => $cont['source_icon'],
                        'upload_source' => $cont['upload_source'],
                        'display_order' => $cont['display_order'],
                        'product_id' => $data['product_id'],
                        'generated_unique_id' => $data['generated_unique_id']
                    ]);
                }
            }
            $product = [];
            $review = ReviewProduct::where('generated_unique_id', $data['generated_unique_id'])->get();
            foreach ($review as $re) {
                $product['product_id'] = $re->product_id;
                $product['generated_unique_id'] = $re->generated_unique_id;
                $content[] = array('id' => $re->id, 'title_english' => $re->title_english, 'title_arabic' => $re->title_arabic, 'text_english' => $re->text_english, 'text_arabic' => $re->text_arabic, 'source_icon' => $re->source_icon ? $re->source_icon : '', 'upload_source' => $cont['upload_source'], 'display_order' => $re->display_order, 'error' => '');
            }
            $product['content'] = $content;
            return response()->json([
                'success' => true,
                'msg' => 'Record saved successfully.',
                'record' => $product
            ]);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.',
                'error' => $e->getMessage(),
            ]);
        }
    }
    public function delete(Request $request)
    {
        $reviews = ReviewProduct::where('generated_unique_id', $request->id)->get();
        $reviewDelete = ReviewProduct::where('generated_unique_id', $request->id)->delete();

        if ($reviewDelete) {
            foreach ($reviews as $review) {
                Storage::delete(''. $review->source_icon);
                Storage::delete(''. $review->upload_source);
            }
            return response()->json([
                'success' => true,
                'msg' => 'Record deleted successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong!.'
            ]);
        }
    }

    public function getAllProduct()
    {
        try {
            $product = Product::all();
            return response()->json([
                'success' => true,
                'record' => $product
            ]);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function removeContent(Request $request)
    {
        try {
           $reviews =  ReviewProduct::where('id', $request->id)->first();
            ReviewProduct::where('id', $request->id)->delete();
            Storage::delete(''. $reviews->source_icon);
            Storage::delete(''. $reviews->upload_source);
            $product = [];
            $review = ReviewProduct::where('generated_unique_id', $request->generated_unique_id)->get();
            foreach ($review as $re) {
                $product['product_id'] = $re->product_id;
                $product['generated_unique_id'] = $re->generated_unique_id;
                $content[] = array('id' => $re->id, 'title_english' => $re->title_english, 'title_arabic' => $re->title_arabic, 'text_english' => $re->text_english, 'text_arabic' => $re->text_arabic, 'source_icon' => $re->source_icon ? $re->source_icon : '', 'upload_source' => $re->upload_source ? $re->upload_source : '',  'display_order' => $re->display_order, 'error' => '', 'upload_source_error' => '');
            }
            $product['content'] = $content;
            return response()->json([
                'success' => true,
                'record' => $product
            ]);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function removeImage(Request $request)
    {
        Storage::delete('' . $request->url);
        if (isset($request->id) && $request->id) {
            ReviewProduct::where('id', $request->id)->update([
                'source_icon' => null
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function uploadSource(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file('image')->store("review");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }

    public function removeSource(Request $request)
    {
        Storage::delete('' . $request->url);
        if (isset($request->id) && $request->id) {
            ReviewProduct::where('id', $request->id)->update([
                'upload_source' => null
            ]);
        }
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    function generateRandomString($length = 10)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }
}
