<?php

namespace App\Http\Controllers\Admin\AppSetting\SciencePage;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Response;
use App\Models\SciencePage;
use App\Models\SciencePageContent;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Yajra\DataTables\Facades\DataTables;

class SciencePageController extends Controller
{
    public function index()
    {
        return view('admin.app-setting.science-page.index');
    }

    public function getData(Request $request)
    {
        if ($request->ajax()) {
            $data = SciencePage::get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="' . url('/') . '/admin/app-setting/science-page/edit/' . $row->id . '" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="' . $row->id . '">Delete</a>';
                    return $btn;
                })
                ->editColumn('active', function ($menu) {
                    if ($menu->active == 1) {
                        $btn = '<div class="custom-control custom-switch">
                        <input type="checkbox" class="custom-control-input activeSwitch" checked  value="' . $menu->active . '" id="customSwitch' . $menu->id . '" data-id="' . $menu->id . '" data-language="' . $menu->language . '">
                        <label class="custom-control-label" for="customSwitch' . $menu->id . '"></label>
                      </div>';
                    } else {
                        $btn = '<div class="custom-control custom-switch">
                        <input type="checkbox" class="custom-control-input activeSwitch"  value="' . $menu->active . '" id="customSwitch' . $menu->id . '" data-id="' . $menu->id . '" data-language="' . $menu->language . '">
                        <label class="custom-control-label" for="customSwitch' . $menu->id . '"></label>
                      </div>';
                    }
                    return $btn;
                })
                ->rawColumns(['active', 'action'])

                ->make(true);
        }
    }

    public function add()
    {
        return view('admin.app-setting.science-page.add');
    }

    public function store(Request $request)
    {
        $content = [];
        if ($request->active == 1) {
            $affected = SciencePage::where('active', '=', 1)->where('language', $request->language)->update(array('active' => 0));
        }
        $science_Page = SciencePage::create([
            'title' => $request->title,
            'language' => $request->language,
            'active' => $request->active,
        ]);
        foreach ($request->repeater as $re) {
            $content[] = array('science_page_id' => $science_Page->id, 'data' => $re['data'], 'type' => $re['type']);
        }
        $page = SciencePage::where('active', '=', 1)->where('language', $request->language)->first();
        if (!$page) {
            SciencePage::where('id', $science_Page->id)->where('language', $request->language)->update(["active" => 1]);
        }
        if ($science_Page) {
            SciencePageContent::insert($content);
            return response()->json([
                'success' => true,
                'msg' => "Blog created successfully."
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => "Something went wrong."
            ]);
        }
    }
    public function edit($id)
    {
        $sciencePage = SciencePage::where('id', $id)->with('repeater')->first();
        foreach ($sciencePage->repeater as $r) {
            $r['error'] = "";
        }
        return view('admin.app-setting.science-page.edit', compact('sciencePage'));
    }
    public function update(Request $request)
    {
        if ($request->active == 1) {
            $affected = SciencePage::where('active', '=', 1)->where('language', $request->language)->update(array('active' => 0));
        }
        $science_Page = SciencePage::where('id', $request->id)->update([
            'title'     => $request->title,
            'language'  => $request->language,
            'active'    => $request->active,
        ]);
        foreach ($request->repeater as $re) {
            if (isset($re['id'])) {
                $scienceContent = SciencePageContent::where('id', $re['id'])->update([
                    'data' => $re['data'],
                    'type' => $re['type'],
                ]);
            } else {
                $scienceContent = SciencePageContent::create([
                    'science_page_id'   => $request->id,
                    'data'              => $re['data'],
                    'type'              => $re['type'],
                ]);
            }
        }
        $sciencePage = SciencePage::where('id', $request->id)->with('repeater')->first();
        foreach ($sciencePage->repeater as $r) {
            $r['error'] = "";
        }
        if ($science_Page) {
            return response()->json([
                'success' => true,
                'sciencePage' => $sciencePage,
                'msg' => 'Record updated successfully.'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => "Something went wrong."
            ]);
        }
    }

    public function uploadImage(Request $request)
    {
        $data = $request->all();
        $validator = Validator::make($data, [
            'image' => 'required|image|mimes:jpeg,png,jpg,svg,gif|max:2048',
        ]);
        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array()
            ), 200);
        } else {
            $path = $request->file('image')->store("blog");
            return Response::json(array(
                'success' => true,
                'errors' => $validator->getMessageBag()->toArray(),
                'data' => array('url' => $path)
            ), 200);
        }
    }

    public function delete(Request $request)
    {
        $sciencePageContents = SciencePageContent::where('science_page_id', $request->id)->get();
        $sciencePageDelete = SciencePage::where('id', $request->id)->delete();
        if ($sciencePageDelete) {
            $contentDelete = SciencePageContent::where('science_page_id', $request->id)->delete();
            if ($contentDelete) {
                foreach($sciencePageContents as $content){
                    Storage::delete('' . $content->data);
                }
                return response()->json([
                    'success' => true,
                    'msg' => 'Record deleted successfully.'
                ]);
            } else {
                return response()->json([
                    'success' => false,
                    'msg' => 'Something went wrong!.'
                ]);
            }
        }
    }

    public function removeText(Request $request)
    {

        SciencePageContent::where('id', $request->id)->delete();
        Storage::delete('' . $request->data);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function removeImage(Request $request)
    {
        Storage::delete('' . $request->url);
        return response()->json([
            'success' => true,
            'msg' => 'Image removed successfully.'
        ]);
    }

    public function manageStatus(Request $request)
    {
        SciencePage::where('active', '=', 1)->where('language', $request->language)->update(array('active' => 0));
        SciencePage::where('id', $request->id)->where('language', $request->language)->update([
            'active' => 1
        ]);
        return response()->json([
            'success' => true,
            'msg' => 'Status update successfully.'
        ]);
    }
}
