<?php

namespace App\Http\Controllers\Admin\Collection;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Collection;
use App\Models\CollectionProduct;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\DB;
use DataTables;
use Exception;

class CollectionController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
        $this->client = new Client();
    }

    public function index()
    {
        return view('admin.collection.list');
    }

    public function edit($id)
    {
        $collection = Collection::where('id', $id)->first();
        return view('admin.collection.edit', compact('collection'));
    }

    public function update_collection(Request $request)
    {

        $id = 0;
        if (isset($request->id)) {
            $sale = Collection::where('id', $request->id)->update([
                'title_english' => $request->title_english,
                'title_arabic' => $request->title_arabic,
                'description_english' => $request->description_english,
                'description_arabic' => $request->description_arabic,
                'description_withouthtml_english' => strip_tags($request->description_english),
                'description_withouthtml_arabic' => strip_tags($request->description_arabic),
            ]);
        }

        $result = Collection::where('id', $request->id)->first();
        return response()->json([
            'success' => true,
            'msg' => 'Update successfully.'
        ]);
    }

    public function getCollections(Request $request)
    {
        if ($request->ajax()) {
            $data = Collection::latest()->get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="' . url('/') . '/admin/collection/collectionedit/' . $row->id . '" class="edit btn btn-primary btn-sm">Edit</a>';
                    return $btn;
                })
                ->rawColumns(['action'])

                ->make(true);
        }
    }

    public function collectionSync(Request $request)
    {
        DB::beginTransaction();
        try {
            $collection_product = [];
            $new_collection = [];
            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'custom_collections.json'
            );
            if ($res->getStatusCode() == 200) {
                $response_data = $res->getBody()->getContents();
                $getCollection = json_decode($response_data)->custom_collections;

                $getcollectionIds = Collection::select('collection_id')->get();
                $collection  = collect($getcollectionIds);
                $idString = $collection->implode('collection_id', ',');
                $idArray = explode(',', $idString);

                $name = [];
                foreach ($getCollection as $key => $row) {

                    $productsRes = $this->client->request(
                        'GET',
                        env('SHOPIFY_API_REQUEST_URL') . 'collections/' . $row->id . '/products.json'
                    );
                    if ($productsRes->getStatusCode() == 200) {
                        $response_data = $productsRes->getBody()->getContents();
                        $getProduct = json_decode($response_data)->products;
                        foreach ($getProduct as $key => $product) {
                            $collection_product[] = array('collection_id' => $row->id, 'product_id' => $product->id);
                        }
                    }
                    $coll = [];
                    $coll['collection_id'] = $row->id;

                    $coll['description_english'] = $row->body_html;
                    $coll['description_withouthtml_english'] = strip_tags($row->body_html);

                    $coll['type'] = 'collection';
                    $coll['image'] = isset($row->image) ? $row->image->src : null;

                    $getCollectionData = Collection::where('collection_id', $row->id)->get();
                    if (count($getCollectionData)) {
                        Collection::where('collection_id', $getCollectionData[0]->collection_id)->update($coll);
                    } else {

                        if(str_contains($row->title, '|')){
                            $titleArray = explode('|', $row->title);
                            $new_collection[] = array('collection_id' => $row->id, 'title_english' => $titleArray[0], 'title_arabic' => $titleArray[1], 'description_english' => $row->body_html, 'description_withouthtml_english' => strip_tags($row->body_html));
                        }else{
                            $new_collection[] = array('collection_id' => $row->id, 'title_english' => $row->title, 'title_arabic' => NULL, 'description_english' => $row->body_html, 'description_withouthtml_english' => strip_tags($row->body_html));
                        }

                    }
                    array_push($name, $row->id);
                    if (($k = array_search($row->id, $idArray)) !== false) {
                        unset($idArray[$k]);
                    }
                }

                if (count($idArray)) {
                    Collection::whereIn('collection_id', $idArray)->delete();
                }
                if (count($new_collection)) {
                    Collection::insert($new_collection);
                }
                if (count($collection_product)) {
                    CollectionProduct::truncate();
                    CollectionProduct::insert($collection_product);
                }

                return response()->json([
                    'success' => true,
                    'product' => $getCollection
                ]);
            }
        } catch (Exception $e) {
            DB::rollback();
        }
    }

    /**
     * Delete Collection
     */
    public function delete(Request $request)
    {
        $deleteproduct = Collection::where('id', $request->id)->delete();
        return response()->json([
            'success' => true,
            'msg' => 'Record deleted successfully.'
        ]);
    }
}
