<?php

namespace App\Http\Controllers\Admin\Product;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Product;
use GuzzleHttp\Client;
use DataTables;
class ProductController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
        $this->client = new Client();
    }

    public function index(){
        return view('admin.product.product-list');
    }

    public function edit($id)
    {
        $product = Product::where('id', $id)->first();
        return view('admin.product.edit',compact('product'));
    }
    public function update_product(Request $request){
        
        $id = 0;
        if (isset($request->id)) {
            $sale = Product::where('id', $request->id)->update([
                'title_english' => $request->title_english,
                'title_arabic' => $request->title_arabic,
                'stock' => $request->stock,
            ]);
        }
        
        $product = Product::where('id', $request->id)->first();
        return response()->json([
            'success' => true,
            'msg' => 'Product Update successfully.',
            'sale' => $product
        ]);
        
    }

    public function getProducts(Request $request)
    {
        if ($request->ajax()) {
            $data = Product::latest()->get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $btn = '<a href="' . url('/') . '/admin/product/productedit/' . $row->id . '" class="edit btn btn-primary btn-sm">Edit</a><a href="javascript:void(0)" class="delete btn btn-danger btn-sm ml-2" data-id="' . $row->id . '">Delete</a>';
                    return $btn;
                })
                ->rawColumns(['action'])

                ->make(true);
        }
    }

    public function productSync(Request $request)
    {
        
        $res = $this->client->request(
            'GET',
            env('SHOPIFY_API_REQUEST_URL') . 'products.json'
        );
        if ($res->getStatusCode() == 200) {
            $response_data = $res->getBody()->getContents();
            $shopify_product = json_decode($response_data)->products;


            

            $getproductIds = Product::select('product_id')->get();
            
            $collection  = collect($getproductIds);
            $productIdString = $collection->implode('product_id', ',');
            $idArray = explode(',',$productIdString);

            
            foreach ($shopify_product as $key => $prod) {
                $product = [];
                $product['product_id'] = $prod->id;
                $product['description'] = strip_tags($prod->body_html);
                $product['stock'] = $prod->variants[0]->inventory_quantity;
                $product['type'] = 'product';
                $product['image'] = isset($prod->image)?$prod->image->src:null;
                $product['price'] = $prod->variants[0]->price;
                $product['strike_price'] = $prod->variants[0]->compare_at_price;

                $getProduct = Product::where('product_id',$prod->id)->get();
                if(count($getProduct)){
                    Product::where('product_id',$prod->id)->update($product);
                }
                else{
                    $product['title_english'] = $prod->title;
                    $product['title_arabic'] = NULL;
                    Product::insert($product);
                }
                if(($k = array_search($prod->id, $idArray)) !== false) {
                    unset($idArray[$k]);
                }
            }

            if(count($idArray)){
                for($i=0;$i<count($getproductIds);$i++){
                    if(isset($idArray[$i])){
                        $productId = $idArray[$i];
                        Product::where('product_id',$productId)->delete();
                    }
                }
            }
            
            return response()->json([
                'success' => true,
                'product' => $getproductIds
            ]);
        }

        
    }
/**
 * Delete Product
*/
public function delete(Request $request)
{
    $deleteproduct = Product::where('id',$request->id)->delete();
    return response()->json([
        'success' => true,
        'msg' => 'Record deleted successfully.'
    ]);
}

}
