<?php

namespace App\Http\Controllers\Api\AppSetting;

use Response;
use Validator;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Cart;
use App\Models\Variant;
use App\Models\AddCartProductFrom;
use App\Models\UserToken;
use App\Http\Controllers\Admin\Helper\HelperController;
use Exception;

class CartController extends Controller
{

    public function addToCart(Request $request)
    {
        try {
            $data = $request->all();
            $rules = array(
                'device_id'         => 'required',
                'variant_id'    => 'required',
                'quantity' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $variant = Variant::where('variant_id', $request->variant_id)->first();
                if ($variant) {
                    if (isset($request->from)) {
                        
                            AddCartProductFrom::create([
                                'from' => $request->from,
                                'count' => 1
                            ]);
                        
                    }
                    $cart = Cart::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('is_delete', 0)->first();
                    if ($cart) {
                        Cart::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('is_delete', 0)->update([
                            'quantity' => $cart->quantity + $request->quantity
                        ]);
                        $count = Cart::where('device_id', $request->device_id)->where('is_delete', 0)->where('quantity', '!=', 0)->count();
                        if ($request->language_id == 'ar') {
                            $msg = "تم الاضافة لسلة التسوق!";
                        } elseif ($request->language_id == 'en') {
                            $msg = "Added to the cart successully!";
                        }
                        return response()->json([
                            'count' => $count, 
                            'msg' => $msg, 
                            'success' => true
                        ]);
                    } else {
                        Cart::create([
                            'device_id' => $request->device_id,
                            'variant_id' => $request->variant_id,
                            'product_id' => $variant->product_id,
                            'quantity' => $request->quantity,
                        ]);
                        $count = Cart::where('device_id', $request->device_id)->where('is_delete', 0)->where('quantity', '!=', 0)->count();
                        if ($request->language_id == 'ar') {
                            $msg = "تم الاضافة لسلة التسوق!";
                        } elseif ($request->language_id == 'en') {
                            $msg = "Added to the cart successully!";
                        }
                        return response()->json([
                            'count' => $count, 
                            'msg' => $msg, 
                            'success' => true
                        ]);
                    }
                } else {
                    return Response::json(array(
                        'success' => false,
                        'errors' => array('variant_id' => [
                            'The variant_id does not exist.'
                        ]),
                    ), 422);
                }
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json(['msg' => $msgErr, 'success' => false]);
        }
    }

    public function getAllCartProduct(Request $request)
    {
        $cart = [];
        $cart_data = [];
        $status = "";
        $image = "";
        $variant_name = "";
        $total_price = 0;
        $total_strike_price = 0;
        // try {
            $data = $request->all();
            $rules = array(
                'device_id'         => 'required',
                'language_id'    => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
				$user =  UserToken::where('device_id', $request->device_id)->first();
                    if ($user) {
                        $currency = $user->currency;
                    } else {
                        $currency = 'KWD';
                    }
                if ($request->language_id == 'en') {
                    // $currency = UserToken::where('device_id', $request->device_id)->first();
                    $cart = Cart::where('device_id', $request->device_id)
                        ->where('is_delete', 0)->with(['variant' => function ($q) {
                            $q->with(['product' => function ($query) {
                                $query->select('title_english as title', 'product_id');
                            }]);
                            $q->select('variant_id', 'option1_english as option1', 'option2_english as option2', 'option3_english as option3', 'image', 'price', 'strike_price', 'status', 'inventory_policy', 'product_id');
                        }])->get();
                } else if ($request->language_id == 'ar') {
                    $cart = Cart::where('device_id', $request->device_id)
                        ->where('is_delete', 0)->with(['variant' => function ($q) {
                            $q->with(['product' => function ($query) {
                                $query->select('title_arabic as title', 'product_id');
                            }]);
                            $q->select('variant_id', 'product_id', 'option1_arabic as option1', 'option2_arabic as option2', 'option3_arabic as option3', 'image', 'price', 'strike_price', 'status', 'inventory_policy');
                        }])->get();
                }
                foreach ($cart as $c) {
                    $Product_title = $c['variant']['product']['title'];
                    $title = "";
                    $price = "";
                    $strike_price = "";
                    $variant_name = "";
                    $status = "";
                    $image = "";
                    if ($c['variant']['option1'] != null) {
                        $title = $c['variant']['option1'];
                    }
                    if ($c['variant']['option2'] != null) {
                        $title = $title . '/' . $c['variant']['option2'];
                    }
                    if ($c['variant']['option3'] != null) {
                        $title = $title . '/' . $c['variant']['option3'];
                    }
                    if ($title != 'Default Title' && $title != "") {
                        $variant_name = $title;
                    }
                    if ($c['variant']['price']) {
                        // $price = strval(HelperController::changeCurrency("KWD", $currency->currency, $c['variant']['price'] * $c['quantity']));
                        $price = $this->changeCurrency($currency, $c['variant']['price'] * $c['quantity']);
                        $total_price = $total_price + $c['variant']['price'] * $c['quantity'];
                    }
                    if ($c['variant']['strike_price']) {
                        // $strike_price = strval(HelperController::changeCurrency("KWD", $currency->currency, $c['variant']['strike_price'] * $c['quantity']));
                        $strike_price = $this->changeCurrency($currency, $c['variant']['strike_price'] * $c['quantity']);
                        $total_strike_price = $total_strike_price + $c['variant']['strike_price'] * $c['quantity'];
                    }
                    if ($c['variant']['status'] == 'notify_me') {
                        $status = "sold_out";
                    }
                    if ($c['variant']['status'] == 'add_to_cart') {
                        $status = $c['variant']['status'];
                    }
                    if ($c['variant']['status'] == 'pre_order') {
                        if ($c['variant']['stock']) {
                            $status = 'pre_order';
                        } else {
                            if ($c['variant']['inventory_policy'] == 'deny') {
                                $status = "sold_out";
                            } else if ($c['variant']['inventory_policy'] == 'continue') {
                                $status = 'pre_order';
                            }
                        }
                    }
                    if ($c['variant']['image']) {
                        $image = $c['variant']['image'];
                    } else {
                        $image = $c['variant']['product']['image'];
                    }
                    $cart_data[] = array('variant_id' => $c['variant_id'], 'product_id' => $c['variant']['product_id'], 'title' => $Product_title, 'variant_name' => $variant_name, 'image' => $image, 'quantity' => $c['quantity'], 'price' => "$price", 'strike_price' => "$strike_price", 'actual_price' => $c['variant']['price'] ? $this->changeCurrency($currency, $c['variant']['price']) : "", 'actual_strike_price' => $c['variant']['strike_price'] ? $this->changeCurrency($currency, $c['variant']['strike_price']): "", 'status' => $status);
                }
                return response()->json(['cart' => $cart_data, 'currency' => $currency, 'total_price' => strval($this->changeCurrency($currency, $total_price)),'total_strike_price' => strval($this->changeCurrency($currency, $total_strike_price)), 'subtotal_price' => strval($this->changeCurrency($currency, $total_price)), 'item_count' => count($cart_data), 'success' => true]);
            }
        // } catch (Exception $e) {
        //     return response()->json(['msg' => 'Something went wrong.', 'success' => false]);
        // }
    }

    public function changeCurrency($currency, $price){
        if($currency){
            if($currency == 'KWD'){
                return $price;
            }else{
                $value = HelperController::changeCurrency('KWD', $currency, $price);
                return $value;
            }
        }else{
            return $price;
        }
    }

    public function deleteCartProduct(Request $request)
    {
        try {
            $data = $request->all();
            $rules = array(
                'device_id'         => 'required',
                'variant_id'    => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $cart = Cart::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->update([
                    "is_delete" => 1
                ]);
                if ($request->language_id == 'ar') {
                    $msg = "تم ازالة المنتج من سلة التسوق 😭";
                } elseif ($request->language_id == 'en') {
                    $msg = "Item removed from the cart 😭";
                }
                if ($cart) {
                    return response()->json(['msg' =>  $msg, 'success' => true]);
                } else {
                    if ($request->language_id == 'ar') {
                        $msgEr = "حدث خطأ";
                    } elseif ($request->language_id == 'en') {
                        $msgEr = "Something went wrong";
                    }
                    return response()->json(['msg' => $msgEr, 'success' => false]);
                }
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json(['msg' => $msgErr, 'success' => false]);
        }
    }

    public function quantityChange(Request $request)
    {
        $cart_data = "";
        $total_price = 0;
        $total_strike_price = 0;
        $subtotal_price = 0;
        try {
            $data = $request->all();
            $rules = array(
                'device_id'         => 'required',
                'variant_id'    => 'required',
                'action' => 'required',
                'language_id' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                // $currency = UserToken::where('device_id', $request->device_id)->first();
                $user =  UserToken::where('device_id', $request->device_id)->first();
                if ($user) {
                    $currency = $user->currency;
                } else {
                    $currency = 'KWD';
                }
                $cart = Cart::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('is_delete', 0)->first();
                if ($request->action == 'increase') {
                    Cart::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('is_delete', 0)->update([
                        'quantity' => $cart->quantity + 1
                    ]);
                } else if ($request->action == 'decrease') {
                    Cart::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('is_delete', 0)->update([
                        'quantity' => $cart->quantity - 1
                    ]);
                }
                if ($request->language_id == 'en') {
                    $updated_cart = Cart::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('is_delete', 0)
                        ->with(['variant' => function ($q) {
                            $q->with(['product' => function ($query) {
                                $query->select('title_english as title', 'product_id');
                            }]);
                            $q->select('variant_id', 'option1_english as option1', 'option2_english as option2', 'option3_english as option3', 'image', 'price', 'strike_price', 'status', 'inventory_policy', 'product_id');
                        }])->first();
                } else if ($request->language_id == 'ar') {
                    $updated_cart = Cart::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('is_delete', 0)
                        ->with(['variant' => function ($q) {
                            $q->with(['product' => function ($query) {
                                $query->select('title_arabic as title', 'product_id');
                            }]);
                            $q->select('variant_id', 'option1_arabic as option1', 'option2_arabic as option2', 'option3_arabic as option3', 'image', 'price', 'strike_price', 'status', 'inventory_policy', 'product_id');
                        }])->first();
                }

                if ($updated_cart) {
                    $Product_title = $updated_cart['variant']['product']['title'];
                    $title = "";
                    $price = "";
                    $strike_price = "";
                    $variant_name = "";
                    $status = "";
                    $image = "";
                    if ($updated_cart['variant']['option1'] != null) {
                        $title = $updated_cart['variant']['option1'];
                    }
                    if ($updated_cart['variant']['option2'] != null) {
                        $title = $title . '/' . $updated_cart['variant']['option2'];
                    }
                    if ($updated_cart['variant']['option3'] != null) {
                        $title = $title . '/' . $updated_cart['variant']['option3'];
                    }
                    if ($title != 'Default Title' && $title != "") {
                        $variant_name = $title;
                    }
                    if ($updated_cart['variant']['price']) {
                        $price = strval($this->changeCurrency($currency, $updated_cart['variant']['price'] * $updated_cart['quantity']));
                    }
                    if ($updated_cart['variant']['strike_price']) {
                        $strike_price = strval($this->changeCurrency($currency, $updated_cart['variant']['strike_price'] * $updated_cart['quantity']));
                    }
                    if ($updated_cart['variant']['status'] == 'notify_me') {
                        $status = "sold_out";
                    }
                    if ($updated_cart['variant']['status'] == 'add_to_cart') {
                        $status = $updated_cart['variant']['status'];
                    }
                    if ($updated_cart['variant']['status'] == 'pre_order') {
                        if ($updated_cart['variant']['stock']) {
                            $status = 'pre_order';
                        } else {
                            if ($updated_cart['variant']['inventory_policy'] == 'deny') {
                                $status = "sold_out";
                            } else if ($updated_cart['variant']['inventory_policy'] == 'continue') {
                                $status = 'pre_order';
                            }
                        }
                    }
                    if ($updated_cart['variant']['image']) {
                        $image = $updated_cart['variant']['image'];
                    } else {
                        $image = $updated_cart['variant']['product']['image'];
                    }
                    $cart_price = Cart::where('device_id', $request->device_id)->where('is_delete', 0)
                        ->with(['variant' => function ($q) {
                            $q->select('variant_id', 'price', 'strike_price');
                        }])->get();
                    foreach ($cart_price as $cp) {
                        if ($cp['variant']['price']) {
                            $total_price =  $total_price + $cp['variant']['price'] * $cp['quantity'];
                        }
                        if ($cp['variant']['strike_price']) {
                            $total_strike_price =  $total_strike_price + $cp['variant']['strike_price'] * $cp['quantity'];
                        }
                    }
                    $cart_data = array('variant_id' => $updated_cart['variant_id'], 'product_id' => $updated_cart['variant']['product_id'], 'title' => $Product_title, 'variant_name' => $variant_name, 'image' => $image, 'quantity' => $updated_cart['quantity'], 'price' => $price, 'strike_price' => $strike_price, 'actual_price' => $this->changeCurrency($currency, $updated_cart['variant']['price']), 'actual_strike_price' => $this->changeCurrency($currency, $updated_cart['variant']['strike_price']), 'status' => $status);
                }
                return response()->json(['cart' => $cart_data, 'currency' => $currency, 'total_price' => strval($this->changeCurrency($currency, $total_price)), 'total_strike_price' => strval($this->changeCurrency($currency, $total_strike_price)), 'subtotal_price' => strval($this->changeCurrency($currency, $total_price)), 'item_count' => count($cart_price), 'msg' => 'Quantity updated successfully.', 'success' => true]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json(['msg' => $msgErr, 'success' => false]);
        }
    }
}
