<?php

namespace App\Http\Controllers\Api\AppSetting;

use App\Http\Controllers\Controller;
use App\Models\Challenge;
use App\Models\ChallengeContent;
use App\Models\ChallengeSection;
use App\Models\CompleteChallenge;
use App\Models\SavedChallenge;
use App\Models\UsedChallenge;
use App\Models\ActiveChallenge;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;

class ChallengeController extends Controller
{

    public function getSection(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                'language_id' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                if ($request->language_id == 'en') {
                    $section = ChallengeSection::with(['challenge' => function ($query) {
                        $query->select('id', 'challenge_section_id', 'title_english as title', 'description_english as description', 'cover_image', 'description_image', 'access_type', 'challenge_type');
                    }])->orderBy('display_order', 'asc')->select('id', 'title_english as title', 'sub_title_english as sub_title')->get()->toArray();
                } else if ($request->language_id == 'ar') {
                    $section = ChallengeSection::with(['challenge' => function ($query) {
                        $query->select('id', 'challenge_section_id', 'title_arabic as title', 'description_arabic as description', 'cover_image', 'description_image', 'access_type', 'challenge_type');
                    }])->orderBy('display_order', 'asc')->select('id', 'title_arabic as title', 'sub_title_arabic as sub_title')->get()->toArray();
                }
                foreach ($section as $key => $s) {
                    foreach ($s['challenge'] as $index => $cha) {
                        $section[$key]['challenge'][$index]['cover_image'] = 'storage/app/' . $cha['cover_image'];
                        $section[$key]['challenge'][$index]['description_image'] = 'storage/app/' . $cha['description_image'];
                    }
                }
                return response()->json([
                    'success' => true,
                    'section' => $section
                ]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }

    public function getAllSectionChallenge(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                'language_id' => 'required',
                'section_id' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                if ($request->language_id == 'en') {
                    $section = ChallengeSection::where('id', $request->section_id)->with(['challenge' => function ($query) {
                        $query->where('status', 1)->select('id', 'challenge_section_id', 'title_english as title', 'description_english as description', 'cover_image', 'description_image', 'access_type', 'challenge_type');
                    }])->select('id', 'title_english as title', 'sub_title_english as sub_title')->first();
                } else if ($request->language_id == 'ar') {
                    $section = ChallengeSection::where('id', $request->section_id)->with(['challenge' => function ($query) {
                        $query->where('status', 1)->select('id', 'challenge_section_id', 'title_arabic as title', 'description_arabic as description', 'cover_image', 'description_image', 'access_type', 'challenge_type');
                    }])->select('id', 'title_arabic as title', 'sub_title_arabic as sub_title')->first();
                }
                foreach ($section->challenge as $key => $cha) {
                    $section->challenge[$key]['cover_image'] = 'storage/app/' . $cha['cover_image'];
                }

                return response()->json([
                    'success' => true,
                    'section' => $section
                ]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }

    public function getChallenge(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                'language_id' => 'required',
                'challenge_id' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                if ($request->language_id == 'en') {
                    $challenge = Challenge::where('id', $request->challenge_id)->where('status', '=', '1')->select('id', 'title_english as title', 'description_english as description', 'cover_image', 'description_image', 'status', 'access_type', 'challenge_type')->first();
                } else if ($request->language_id == 'ar') {
                    $challenge = Challenge::where('id', $request->challenge_id)->where('status', '=', '1')->select('id', 'title_arabic as title', 'description_arabic as description', 'cover_image', 'description_image', 'status', 'access_type', 'challenge_type')->first();
                }

                $challenge->cover_image = 'storage/app/' . $challenge->cover_image;
                $challenge->description_image = 'storage/app/' . $challenge->description_image;

                return response()->json([
                    'success' => true,
                    'challenge' => $challenge
                ]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }

    public function getContent(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                'challenge_id' => 'required',
                "language_id" => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $challenge = Challenge::where('id', $request->challenge_id)->first();
                if ($challenge->challenge_type == 'one_time') {
                    $content = ChallengeContent::where('challenge_id', $request->challenge_id)->select('id', 'upload_type', 'cover_image', 'url')->first();
                    $content['cover_image'] = $content['cover_image'] != NULL ? 'storage/app/' . $content['cover_image'] : "";;
                    $content['url'] = 'storage/app/' . $content['url'];
                } elseif ($challenge->challenge_type == 'day_by_day') {
                    $content = ChallengeContent::where('challenge_id', $request->challenge_id)->select('id', 'upload_type', 'cover_image', 'url')->orderBy('id', 'asc')->get()->toArray();
                    foreach ($content as $key => $cha) {
                        $content[$key]['cover_image'] = $cha['cover_image'] != NULL ? 'storage/app/' . $cha['cover_image'] : "";;
                        $content[$key]['url'] = 'storage/app/' . $cha['url'];
                    }
                }
                if ($content) {
                    return response()->json([
                        'success' => true,
                        'challenge_type' => $challenge->challenge_type,
                        'content' => $content
                    ]);
                } else {
                    if ($request->language_id == 'ar') {
                        $msgEr = "حدث خطأ";
                    } elseif ($request->language_id == 'en') {
                        $msgEr = "Something went wrong";
                    }
                    return response()->json([
                        'success' => false,
                        'msg' => $msgEr
                    ]);
                }
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }

    public function getChallengeContent(Request $request)
    {
        $content = ChallengeContent::where('id', $request->content_id)->where('challenge_id', $request->challenge_id)->select('id', 'upload_type', 'cover_image', 'url')->first();
        if ($content) {
            $content['cover_image'] = $content['cover_image'] != NULL ? 'storage/app/' . $content['cover_image'] : "";
            $content['url'] = 'storage/app/' . $content['url'];
            $usedExist =  UsedChallenge::where('device_id', $request->device_id)->where('challenge_id', $request->challenge_id)->where('content_id', $request->content_id)->first();
            $activeExist =  ActiveChallenge::where('device_id', $request->device_id)->where('challenge_id', $request->challenge_id)->first();
            if (!$usedExist) {
                $usedChallenge = UsedChallenge::create([
                    'device_id'     => $request->device_id,
                    'customer_id'   => $request->customer_id,
                    'challenge_id'  => $request->challenge_id,
                    'content_id'    => $request->content_id,
                ]);
            }
            if (!$activeExist) {
                $activeChallenge = ActiveChallenge::create([
                    'device_id'     => $request->device_id,
                    'customer_id'   => $request->customer_id,
                    'challenge_id'  => $request->challenge_id,
                ]);
            }
            return response()->json([
                'success' => true,
                'content' => $content
            ]);
        } else {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }

    public function getFinalData(Request $request)
    {

        if ($request->language_id == 'en') {
            $challenge = Challenge::where('id', $request->challenge_id)->where('status', '=', '1')->select('id', 'title_english as title', 'final_upload_type as upload_type', 'final_cover_image as cover_image', 'final_url as url')->first();
        } else if ($request->language_id == 'ar') {
            $challenge = Challenge::where('id', $request->challenge_id)->where('status', '=', '1')->select('id', 'title_arabic as title', 'final_upload_type as upload_type', 'final_cover_image as cover_image', 'final_url as url')->first();
        }
        $challenge['cover_image'] = $challenge['cover_image'] != NULL ? 'storage/app/' . $challenge['cover_image'] : NULL;;
        $challenge['url'] = 'storage/app/' . $challenge['url'];
        return response()->json([
            'success' => true,
            'final_data' => $challenge
        ]);
    }

    public function getBadge(Request $request)
    {
        $badge = null;
        $challenge = Challenge::where('id', $request->challenge_id)->select('id', 'title_english as title', 'badge')->first();
        if ($challenge) {
            $badge = 'storage/app/' . $challenge->badge;
            return response()->json([
                'success' => true,
                'badge' => $badge
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => "Something went wrong"
            ]);
        }
    }

    public function getActiveChallenge(Request $request)
    {
        $usedChallenges = ActiveChallenge::where('customer_id', $request->customer_id)->orwhere('device_id', $request->device_id)->get();
        $challenge = [];
        if (count($usedChallenges) > 0) {
            $challengeData = [];
            foreach ($usedChallenges as $usedChallenge) {
                $challengeDetails = Challenge::where('id', $usedChallenge->challenge_id)->first();
                if ($challengeDetails) {
                    $challengeData = [];
                    $challengeData['id'] = $challengeDetails->id;
                    $challengeData['title'] = $request->language_id == "en" ? $challengeDetails->title_english : $challengeDetails->title_arabic;
                    $challengeData['cover_image'] = $challengeDetails->id;
                    $challengeData['access_type'] = $challengeDetails->access_type;
                    $challengeData['challenge_type'] = $challengeDetails->challenge_type;

                    $challenge[] = $challengeData;
                }
            }
            foreach ($challenge as $key => $cha) {
                $challenge[$key]['cover_image'] = $cha['cover_image'] != NULL ? 'storage/app/' . $cha['cover_image'] : NULL;
            }
        }
        return response()->json([
            'success' => true,
            'challenge' => $challenge
        ]);
    }

    public function saveChallenge(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "device_id"         => "required",
                "challenge_id"         => "required",
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $exist = SavedChallenge::where("device_id", $request->device_id)->where("challenge_id", $request->challenge_id)->first();
                if ($exist) {
                    return response()->json([
                        'success' => true,
                        'msg' => "Challenge saved already."
                    ]);
                } else {
                    $saveChallenge = SavedChallenge::create([
                        'device_id' => $request->device_id,
                        'customer_id' => $request->customer_id,
                        'challenge_id' => $request->challenge_id,
                    ]);
                    if ($saveChallenge) {
                        return response()->json([
                            'success' => true,
                            'msg' => "Challenge saved successfully"
                        ]);
                    } else {
                        return response()->json([
                            'success' => false,
                            'msg' => "Something went wrong"
                        ]);
                    }
                }
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => "Something went wrong"
            ]);
        }
    }

    public function saveCompleteChallenge(Request $request)
    {
        $data = $request->all();

        $rules = array(
            "device_id"         => "required",
            "customer_id"         => "required",
            "challenge_id"         => "required",
        );

        $validator = Validator::make($data, $rules);

        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
            ), 422);
        } else {
            $exist = CompleteChallenge::where("device_id", $request->device_id)->where("challenge_id", $request->challenge_id)->first();
            if ($exist) {
                return response()->json([
                    'success' => true,
                    'msg' => "Complete Challenge saved already."
                ]);
            } else {
                $completeChallenge = CompleteChallenge::create([
                    'device_id' => $request->device_id,
                    'customer_id' => $request->customer_id,
                    'challenge_id' => $request->challenge_id,
                ]);
                if ($completeChallenge) {
                    return response()->json([
                        'success' => true,
                        'msg' => "Complete Challenge saved successfully"
                    ]);
                } else {
                    return response()->json([
                        'success' => false,
                        'msg' => "Something went wrong"
                    ]);
                }
            }
        }
    }

    public function getSavedChallenge(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "device_id" => "required"
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $customerChallenges = new SavedChallenge();
                $customerChallenges = $customerChallenges->where('device_id', $request->device_id);
                if ($request->customer_id) {
                    $customerChallenges = $customerChallenges->where('customer_id', $request->customer_id);
                }
                $customerChallenges = $customerChallenges->get();
                $challenge = [];
                if (count($customerChallenges) > 0) {
                    $challengeData = [];
                    foreach ($customerChallenges as $customerChallenge) {
                        if ($request->language_id == 'en') {
                            $challegeExist  = Challenge::where('id', $customerChallenge->challenge_id)->select('id', 'title_english as title', 'description_english as description', 'cover_image', 'description_image', 'status', 'access_type', 'challenge_type')->first();
                        } else if ($request->language_id == 'ar') {
                            $challegeExist  = Challenge::where('id', $customerChallenge->challenge_id)->select('id', 'title_arabic as title', 'description_arabic as description', 'cover_image', 'description_image', 'status', 'access_type', 'challenge_type')->first();
                        }

                        if ($challegeExist) {
                            $challegeExist->cover_image = 'storage/app/' . $challegeExist->cover_image;
                            $challegeExist->description_image = 'storage/app/' . $challegeExist->description_image;
                            $challenge[] = $challegeExist;
                        }
                    }
                    return response()->json([
                        'success' => true,
                        'challenge' => $challenge
                    ]);
                } else {
                    return response()->json([
                        'success' => true,
                        'challenge' => $challenge
                    ]);
                }
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong'
            ]);
        }
    }

    public function getCompleteChallenge(Request $request)
    {

        $customerChallenges = CompleteChallenge::where('customer_id', $request->customer_id)->orwhere('device_id', $request->device_id)->get();
        $challenge = [];
        if (count($customerChallenges) > 0) {
            $challengeData = [];
            foreach ($customerChallenges as $customerChallenge) {
                $challegeExist  = Challenge::where('id', $customerChallenge->challenge_id)->first();
                if ($challegeExist) {
                    $challengeData = [];
                    $challengeData['id'] = $challegeExist->id;
                    $challengeData['title'] = $request->language_id == "en" ? $challegeExist->title_english : $challegeExist->title_arabic;

                    $challenge[] = $challengeData;
                }
            }
            return response()->json([
                'success' => true,
                'challenge' => $challenge
            ]);
        } else {
            return response()->json([
                'success' => true,
                'challenge' => $challenge
            ]);
        }
    }
}
