<?php

namespace App\Http\Controllers\Api\AppSetting;

use App\Http\Controllers\Controller;
use App\Models\ShopifyToken;
use App\Models\Cart;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;

class CheckoutController extends Controller
{
    private $encryption_key;
    private $signature_key;


    public function createCheckout(Request $request)
    {

        try {
            $data = $request->all();
            $rules = array(
                'device_id'         => 'required',
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {

                $storeVariantId = [];
                $lineItems = [];
                $cart = Cart::where('device_id', $request->device_id)->where('is_delete',0)->where('quantity', '!=', 0)->get();
                if (count($cart) > 0) {

                    foreach ($cart as $key => $c) {
                        $storeVariantId['variantId'] = 'gid://shopify/ProductVariant/'.$c->variant_id; 
                        $storeVariantId['quantity'] = $c->quantity; 
                        $lineItems[] = $storeVariantId;
                        
                    }
    
                    $url = $this->storeFrontCheckout($lineItems);

                    return response()->json([
                        'success' => true,
                        'url' => $url
                    ]);

                } else {
                    return response()->json([
                        'success' => false,
                        'msg' => "Your cart is empty"
                    ]);
                }
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => "Something went wrong."
            ]);
        }
    }

    // public function createCheckout(Request $request)
    // {

    //     try {
    //         $data = $request->all();
    //         $rules = array(
    //             'device_id'         => 'required',
    //         );

    //         $validator = Validator::make($data, $rules);

    //         if ($validator->fails()) {
    //             return Response::json(array(
    //                 'success' => false,
    //                 'errors' => $validator->getMessageBag()->toArray(),
    //             ), 422);
    //         } else {
    //             $customer_data = [];
    //             $shopify = ShopifyToken::first();
    //             $cart = Cart::where('device_id', $request->device_id)->where('quantity', '!=', 0)->where('is_delete',0)->select('variant_id', 'quantity')->get()->toArray();
    //             if (count($cart)) {
    //                 $field = array(
    //                     "checkout" => array(
    //                         // "email"=> "096.aditya@gmail.com",
    //                         "line_items" => $cart,
    //                         //   "shipping_address"=> array(
    //                         //     "first_name"=> "John",
    //                         //     "last_name"=> "Smith",
    //                         //     "address1"=> "126 York St.",
    //                         //     "city"=> "Ottawa",
    //                         //     "province_code"=> "ON",
    //                         //     "country_code"=> "CA",
    //                         //     "phone"=> "(123)456-7890",
    //                         //     "zip"=> "K1N 5T5"
    //                         //   )
    //                     )
    //                 );
    //                 $data = json_encode($field);
    //                 $url = env('SHOPIFY_API_CHECKOUT_URL') . '/checkouts.json';
    //                 $curl = curl_init();
    //                 curl_setopt_array($curl, array(
    //                     CURLOPT_URL => $url,
    //                     CURLOPT_RETURNTRANSFER => true,
    //                     CURLOPT_CUSTOMREQUEST => "POST",
    //                     CURLOPT_HTTPHEADER => array(
    //                         "accept: application/json",
    //                         "content-type: application/json",
    //                         "X-Shopify-Access-Token: $shopify->access_token"
    //                     ),
    //                     CURLOPT_POSTFIELDS => $data
    //                 ));
    //                 $response = curl_exec($curl);
    //                 $err = curl_error($curl);
    //                 curl_close($curl);
    //                 if ($err) {
    //                     echo $err;
    //                     if ($request->language_id == 'ar') {
    //                         $msgErr = "حدث خطأ";
    //                     } elseif ($request->language_id == 'en') {
    //                         $msgErr = "Something went wrong.";
    //                     }
    //                     return response()->json([
    //                         'success' => false,
    //                         'msg' => $msgErr
    //                     ]);
    //                 } else {
    //                     $response = json_decode($response, true);

    //                     if ($response['checkout']['web_url']) {
    //                         if ($request->email) {
    //                             $customer_data['email'] = $request->email;
    //                             $customer_data['return_to'] = $response['checkout']['web_url'];
    //                             $multipass_token = $this->getMultipassToken($customer_data);
    //                             $url = "https://afrahbeauty.myshopify.com/account/login/multipass/" . $multipass_token;
    //                         } else {
    //                             $url =  $response['checkout']['web_url'];
    //                         }

    //                         $cart = Cart::where('device_id', $request->device_id)->where('quantity', '!=', 0)->where('is_delete',0)->update([
    //                             'token' => $response['checkout']['token']
    //                         ]);
    //                     }
    //                     return response()->json([
    //                         'success' => true,
    //                         'url' => $url
    //                     ]);
    //                 }
    //             } else {
    //                 if ($request->language_id == 'ar') {
    //                     $msg = "سلة التسوق فارغة";
    //                 } elseif ($request->language_id == 'en') {
    //                     $msg = "Your cart is empty";
    //                 }
    //                 return response()->json([
    //                     'success' => false,
    //                     'msg' => $msg
    //                 ]);
    //             }
    //         }
    //     } catch (Exception $e) {
    //         if ($request->language_id == 'ar') {
    //             $msgErr = "حدث خطأ";
    //         } elseif ($request->language_id == 'en') {
    //             $msgErr = "Something went wrong";
    //         }
    //         return response()->json([
    //             'success' => false,
    //             'msg' => $msgErr
    //         ]);
    //     }
    // }

    public function checkCompletedCheckout()
    {
        
        $shopify = ShopifyToken::first();
        $cart = Cart::where('is_delete', 0)->where('quantity', '!=', 0)->whereNotNull('token')->get()->groupBy('token');
        foreach ($cart->toArray() as $key => $c) {
            $url = env('SHOPIFY_API_CHECKOUT_URL') . '/checkouts/' . $key . '.json';
            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_CUSTOMREQUEST => "GET",
                CURLOPT_HTTPHEADER => array(
                    "accept: application/json",
                    "content-type: application/json",
                    "X-Shopify-Access-Token: $shopify->access_token",
                )
            ));
            $response = curl_exec($curl);
            $err = curl_error($curl);
            curl_close($curl);
            if ($response) {
                $response = json_decode($response, true);
                if (isset($response['checkout']['completed_at']) && isset($response['checkout']['order_id'])) {
                    $cart = Cart::where('token', $key)->update([
                        'is_delete' => 1
                    ]);
                }
            }
        }
        return response()->json([
            'success' => true,
        ]);
    }

    public function getMultipassToken($customer_data)
    {
        $key_material = hash("sha256", env('MULTIPASS_SECRET'), true);
        $this->encryption_key = substr($key_material, 0, 16);
        $this->signature_key = substr($key_material, 16, 16);
        $token = $this->generate_token($customer_data);
        return $token;
    }
    public function generate_token($customer_data_hash)
    {

        $customer_data_hash["created_at"] = date("c");

        $ciphertext = $this->encrypt(json_encode($customer_data_hash));

        return strtr(base64_encode($ciphertext . $this->sign($ciphertext)), '+/', '-_');
    }

    public function encrypt($plaintext)
    {
        $iv = openssl_random_pseudo_bytes(16);
        return $iv . openssl_encrypt($plaintext, "AES-128-CBC", $this->encryption_key, OPENSSL_RAW_DATA, $iv);
    }

    public function sign($data)
    {
        return hash_hmac("sha256", $data, $this->signature_key, true);
    }
    public function getOrderNumberByToken(Request $request)
    {
        $delete = Cart::where('device_id', $request->device_id)->update([
            'is_delete' => 1
        ]);
        if($delete){
            return response()->json([
                'success' => true,
                "order_no" => ""
            ]);
        }else{
            return response()->json([
                'success' => false,
                "order_no" => "Something went wrong"
            ]);
        }
    }
    // public function getOrderNumberByToken(Request $request)
    // {
    //     $cart = Cart::where('is_delete', '=', '0')->where('device_id', $request->device_id)->first();
    //     // echo $cart->token;die;
    //     if($cart != NULL){
    //         $shopify = ShopifyToken::first();
    //         $url = env('SHOPIFY_API_CHECKOUT_URL') . '/checkouts/' . $cart->token . '.json';
    //         $curl = curl_init();
    //         curl_setopt_array($curl, array(
    //             CURLOPT_URL => $url,
    //             CURLOPT_RETURNTRANSFER => true,
    //             CURLOPT_CUSTOMREQUEST => "GET",
    //             CURLOPT_HTTPHEADER => array(
    //                 "accept: application/json",
    //                 "content-type: application/json",
    //                 "X-Shopify-Access-Token: $shopify->access_token",
    //             )
    //         ));
    //         $response = curl_exec($curl);
    //         $err = curl_error($curl);
    //         curl_close($curl);
    //         if ($response) {

    //             $response = json_decode($response, true);

    //             if (isset($response['checkout']['completed_at']) && isset($response['checkout']['order_id'])) {

    //                     return response()->json([
    //                         'success' => true,
    //                         "order_no" => $response['checkout']['order']['name']
    //                     ]);
    //             }else{
    //                 return response()->json([
    //                     'success' => false,
    //                     "order_no" => ""
    //                 ]);
    //             }
    //         } else {
    //             return response()->json([
    //                 'success' => false,
    //                 "order_no" => ""
    //             ]);
    //         }
    //     }else{
    //         return response()->json([
    //             'success' => false,
    //             "order_no" => ""
    //         ]);
    //     }
    // }

    public function storeFrontCheckout($query)
    {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://afrahbeauty.myshopify.com/api/2021-07/graphql.json',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => '{"query":"mutation checkoutCreate($input: CheckoutCreateInput!) {\\r\\n  checkoutCreate(input: $input) {\\r\\n    checkout {\\r\\n      id\\r\\n      webUrl\\r\\n    }\\r\\n  }\\r\\n}\\r\\n","variables":{"input":{"lineItems":'.json_encode($query).'}}}',
            CURLOPT_HTTPHEADER => array(
                'X-Shopify-Storefront-Access-Token: 1809d2799af2056fa9af421d81116b80',
                'Content-Type: application/json',
            ),
        ));

        $response = curl_exec($curl);
        // echo $response;die;

        curl_close($curl);
        $checkout = json_decode($response, true);

        return $checkout['data']['checkoutCreate']['checkout']['webUrl'];
    }
}
