<?php

namespace App\Http\Controllers\Api\AppSetting;

use App\Exports\CustomerExport;
use App\Http\Controllers\Controller;
use App\Models\CurrencyPoint;
use App\Models\Customer;
use App\Models\Milestone;
use App\Models\MilestonePoint;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;

class CustomerController extends Controller
{
    public $client;

    public function __construct()
    {
        $this->client = new Client();
        // $this->url = env('SHOPIFY_API_REQUEST_URL') . 'customers.json?page_info=eyJsYXN0X2lkIjo1NDQzMTY2MTA5ODUxLCJsYXN0X3ZhbHVlIjoiMjAyMS0wOC0wMiAwODowMzoxNC4wMDAwMDAifQ&limit=10';
        $this->url = env('SHOPIFY_API_REQUEST_URL') . 'customers.json?limit=250';
    }

    public function getCustomer(Request $request)
    {
        $data = $request->all();

        $rules = array(
            "email" => "required|email",
        );

        $validator = Validator::make($data, $rules);

        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
            ), 422);
        } else {
            $customer = Customer::where('email', $request->email)->first();
            if ($customer) {
                $customer_points = $customer->point + $customer->extra_point;
                $customerPoint = [
                    'customer_id'=> $customer->customer_id,
                    'level'=> NULL,
                    'point'=> $customer_points,
                    'next_point'=> 0,
                    'currency' => NULL,
                    'each_value' => NULL,
                    'each_point' => NULL
                ];
                $currencyPoint = CurrencyPoint::where('currency', $customer->currency)->select('currency', 'each_value', 'each_point')->first();
                $next_point = Milestone::where('from_point' , ">", $customer_points)->select('level_english','from_point')->first();
                $level = $this->getCustomerMilestoneLevel($customer_points, $request->language_id);
                if($next_point){
                    $point = abs($next_point->from_point - $customer_points);
                }else{
                    $point = 0;
                }
                if($currencyPoint){
                    $customerPoint['level'] = $level;
                    $customerPoint['next_point'] = $point;
                    $customerPoint['currency'] = $currencyPoint->currency;
                    $customerPoint['each_value'] = $currencyPoint->each_value;
                    $customerPoint['each_point'] = $currencyPoint->each_point;
                }else{
                    
                    $currencyPointData = CurrencyPoint::where('currency', 'KWD')->select('currency', 'each_value', 'each_point')->first();
                    $customerPoint['level'] = $level;
                    $customerPoint['next_point'] = $point;
                    $customerPoint['currency'] = $currencyPointData->currency;
                    $customerPoint['each_value'] = $currencyPointData->each_value;
                    $customerPoint['each_point'] = $currencyPointData->each_point;
                }
                return response()->json([
                    'success' => true,
                    'customer_id' => $customer->customer_id,
                    'customer_info' => $customer,
                    'customer_milestone' => $customerPoint
                ]);
            } else {
                return response()->json([
                    'success' => false,
                    'msg' => "Email does not exist."
                ]);
            }
        }
    }

    public function getCustomerMilestoneLevel($point, $language){

        $max = Milestone::max('to_point');
        if($point > $max){
            if ($language == 'en') {
                $milestone = Milestone::where('to_point', '=',$max)->select('level_english as level')->first();
            } else if ($language == 'ar') {
                $milestone = Milestone::where('to_point', '=',$max)->select('level_arabic as level')->first();
            }
            if($milestone){
                return $milestone->level;
            }else{
                return 'Not Available';
            }
            // echo $milestone->level;die;
        }else{
            if ($language == 'en') {
                $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('level_english as level')->first();
            } else if ($language == 'ar') {
                $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('level_arabic as level')->first();
            }
            if($milestone){
                return $milestone->level;
            }else{
                return 'Not Available';
            }
            // echo $milestone->level;die;
        }
    }

    public function getLevel($point){

        $max = Milestone::max('from_point');
        if($point >= $max){
            $milestone = Milestone::where('from_point', '=',$max)->select('level_english')->first();
            if($milestone){
                return $milestone->level_english;
            }else{
                return 'Not Available';
            }
        }else{
            $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('level_english')->first();
            if($milestone){
                return $milestone->level_english;
            }else{
                return 'Not Available';
            }
        }
    }

    public function getMilestone($point){

        $max = Milestone::max('from_point');
        if($point >= $max){
            $milestone = Milestone::where('from_point', '=',$max)->select('id', 'level_english')->first();
            if($milestone){
                return $milestone->id;
            }else{
                return 0;
            }
        }else{
            $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('id', 'level_english')->first();
            if($milestone){
                return $milestone->id;
            }else{
                return 0;
            }
        }
    }

    public function getCustomerAllAddress(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "customer_id"  => "required",
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $res = $this->client->request(
                    'GET',
                    env('SHOPIFY_NEW_API_REQUEST_URL') . 'customers/' . $request->customer_id . '/addresses.json'
                );
                if ($res->getStatusCode() == 200) { // 200 OK
                    $response_data = $res->getBody()->getContents();
                    $addresses = json_decode($response_data)->addresses;
                    return response()->json([
                        'success' => true,
                        'address' => $addresses
                    ]);
                } else {
                    return response()->json([
                        'success' => false,
                        'msg' => 'Something went wrong.'
                    ]);
                }
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function getAddressById(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "customer_id"  => "required",
                "address_id"  => "required",
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $res = $this->client->request(
                    'GET',
                    env('SHOPIFY_NEW_API_REQUEST_URL') . 'customers/' . $request->customer_id . '/addresses/' . $request->address_id . '.json'
                );
                if ($res->getStatusCode() == 200) { // 200 OK
                    $response_data = $res->getBody()->getContents();
                    $addresses = json_decode($response_data)->customer_address;
                    return response()->json([
                        'success' => true,
                        'address' => $addresses
                    ]);
                } else {
                    return response()->json([
                        'success' => false,
                        'msg' => 'Something went wrong.'
                    ]);
                }
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function deleteAddress(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "customer_id"  => "required",
                "address_id"  => "required",
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $res = $this->client->request(
                    'DELETE',
                    env('SHOPIFY_NEW_API_REQUEST_URL') . 'customers/' . $request->customer_id . '/addresses/' . $request->address_id . '.json'
                );
                if ($res->getStatusCode() == 200) { // 200 OK
                    $response_data = $res->getBody()->getContents();
                    return response()->json([
                        'success' => true,
                        'msg' => "Address removed successfully."
                    ]);
                } else {
                    return response()->json([
                        'success' => false,
                        'msg' => 'Something went wrong.'
                    ]);
                }
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }
    public function updateAddress(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "customer_id"  => "required",
                "address_id"  => "required",
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $res = $this->client->request(
                    'PUT',
                    env('SHOPIFY_NEW_API_REQUEST_URL') . 'customers/' . $request->customer_id . '/addresses/' . $request->address_id . '.json',
                    [
                        'form_params' => [
                            'address' => $request->address
                        ],

                    ]
                );
                if ($res->getStatusCode() == 200) { // 200 OK
                    $response_data = $res->getBody()->getContents();
                    $addresses = json_decode($response_data)->customer_address;
                    return response()->json([
                        'success' => true,
                        'address' => $addresses
                    ]);
                } else {
                    return response()->json([
                        'success' => false,
                        'msg' => 'Something went wrong.'
                    ]);
                }
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong.'
            ]);
        }
    }

    // public function pagination(){
        
    // }    

    public function pagination(){
        $customerCreateData = [];
        $customerUpdateData = [];
        $customerData       = [];
        $customerIdData     = [];
        $i = 0;
        while(true){
            $res = $this->client->request(
                'GET',
                trim($this->url),
                ['headers' => 
                    [
                        'X-Shopify-Access-Token' => "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",
                    ]
                ]
            );

            if ($res->getStatusCode() == 200) { // 200 OK
                $response_data = $res->getBody()->getContents();
                $customers = json_decode($response_data)->customers;
                foreach($customers as $customer){
                    array_push($customerIdData, $customer->id);
                    $currencyPoint  = CurrencyPoint::where('currency', $customer->currency)->first();
                    if($currencyPoint){
                        $point = $customer->total_spent * $currencyPoint->each_point;
                    }else{
                        $cPoint  = CurrencyPoint::where('currency', 'KWD')->first();
                        if($cPoint){
                            $point = $customer->total_spent * $cPoint->each_point;
                        }else{
                            $point = $customer->total_spent * 1;
                        }
                    }
                    $exist = Customer::where('customer_id', $customer->id)->first();
                    if($exist){
                        $levelPoint = $exist->point + $exist->extra_point;
                        $level = $this->getLevel($levelPoint);
                        $milestone_id = $this->getMilestone($levelPoint);
                        $customerData["customer_id"]        = $customer->id;
                        $customerData["email"]              = $customer->email;
                        $customerData["first_name"]         = $customer->first_name;
                        $customerData["last_name"]          = $customer->last_name;
    
                        $customerData["phone"]              = $customer->phone;
                        $customerData["state"]              = $customer->state;
                        $customerData["total_spent"]        = $customer->total_spent;
                        $customerData["currency"]           = $customer->currency;
                        if($customer->last_order_id != NULL){
                            $customerData["last_order_id"] = $customer->last_order_id ;
                        }
                        $customerData["last_order_name"]    = $customer->last_order_name;
                        $customerData["orders_count"]       = $customer->orders_count;
                        $customerData["point"]              = $point;
                        $customerData["level"]              = $level;
                        $customerData["milestone_id"]       = $milestone_id;
                        $customerData["updated_at"]         =  now();
                        $customerUpdateData[] = $customerData;
                        
                    }else{
                        $newLevel = $this->getLevel(0);
                        $new_milestone_id = $this->getMilestone(0);
                        $customerDetail["customer_id"]      = $customer->id;
                        $customerDetail["email"]            = $customer->email;
                        $customerDetail["first_name"]       = $customer->first_name;
                        $customerDetail["last_name"]        = $customer->last_name;
                        
                        $customerDetail["phone"]            = $customer->phone;
                        $customerDetail["state"]            = $customer->state;
                        $customerDetail["total_spent"]      = $customer->total_spent;
                        $customerDetail["last_order_id"]    = $customer->last_order_id;
                        $customerDetail["last_order_name"]  = $customer->last_order_name;
                        $customerDetail["currency"]         = $customer->currency;
                        $customerDetail["orders_count"]     = $customer->orders_count;
                        $customerDetail["point"]            = $point;
                        $customerDetail["level"]            = $newLevel;
                        $customerDetail["milestone_id"]     = $new_milestone_id;
                        $customerDetail["created_at"]       =  now();
                        $customerDetail["updated_at"]       =  now();
                        $customerDetail["milestone_entry"]  =  now();
                        $customerCreateData[] = $customerDetail;
                    }
                }
                // echo "<pre>";
                // echo $i;
                // print_r($customerCreateData);
                // $i++;
                $response_header =  $res->getHeader('Link');
                $header =  explode(',',$response_header[0]);

                if(count($header) > 1 && count($header) <= 2){
                    $next =  explode(';',$header[1]);
                    $next_url = str_replace( array( '<', '>' ), ' ', $next[0]);
                    $this->url = $next_url;
                }elseif(count($header) > 0 && count($header) <= 1){
                    $next =  explode(';',$header[0]);
                    $rel =  str_replace( array( 'rel', '=','"' ), ' ', $next[1]);
                    if(trim($rel) == 'next'){
                        $next_url = str_replace( array( '<', '>' ), ' ', $next[0]);
                        $this->url = $next_url;
                    }else{
                        break;
                    }
                }else{
                    break;
                }
            }
        }

        $customerIds =  Customer::get()->pluck('customer_id')->toArray();
        if ($customerCreateData) {
            $result = Customer::insert($customerCreateData);
        }
        if ($customerUpdateData) {
            $result = $this->updateBatch($customerUpdateData);
        }
        if ($customerIdData != null && $customerIds != null) {
            foreach ($customerIds as $customerId) {
                if (!in_array($customerId, $customerIdData)) {
                    $result = Customer::where('customer_id', $customerId)->update([
                        'is_delete' => 1
                    ]);
                }
            }
        }
        if ($result) {
            return response()->json([
                'success' => true,
                'msg' => 'Customer Saved Successfully'
            ]);
        } else {
            return response()->json([
                'success' => false,
                'msg' => 'Something went wrong'
            ]);
        } 
    }

    public function updateBatch($multipleData){
        $tableName = "customers";
        if( $tableName && !empty($multipleData) ) {
    
            // column or fields to update
            $updateColumn = array_keys($multipleData[0]);
            $referenceColumn = $updateColumn[0]; //e.g id
            unset($updateColumn[0]);
            $whereIn = "";
    
            $q = "UPDATE ".$tableName." SET "; 
            foreach ( $updateColumn as $uColumn ) {
                $q .=  $uColumn." = CASE ";
    
                foreach( $multipleData as $data ) {
                    $q .= "WHEN ".$referenceColumn." = ".$data[$referenceColumn]." THEN '".$data[$uColumn]."' ";
                }
                $q .= "ELSE ".$uColumn." END, ";
            }
            foreach( $multipleData as $data ) {
                $whereIn .= "'".$data[$referenceColumn]."', ";
            }
            $q = rtrim($q, ", ")." WHERE ".$referenceColumn." IN (".  rtrim($whereIn, ', ').")";
            // Update  
            $affected =  DB::update(DB::raw($q));
            // $affected = DB::update($q);
            return  $affected;
        } else {
            return false;
        }
    }

    public function updateCustomerLevel(){
        
        // $customers = Customer::get();
        // if($customers){
        //     foreach($customers as $customer){
        //         $totalPointBefore = 0;
        //         $levelBefore = NULL;
        //         $totalPointAfter = 0;
        //         $levelAfter = NULL;

        //         $levelBefore = $customer->level;
        //         $totalPoint = $customer->point + $customer->extra_point;
        //         $levelAfter = $this->getLevel($totalPoint);
        //         $milestone_id = $this->getMilestone($totalPoint);

                
        //         if (trim($levelBefore) != trim($levelAfter)) {
        //             // echo $levelAfter;die;
        //             $milestoneEntry = Customer::where('customer_id', $customer->customer_id)->update([
        //                 "level" => $levelAfter,
        //                 "milestone_entry" => Carbon::now('Asia/Kuwait'),
        //             ]);
        //         }
        //     }
        //     echo "yes";
        // }
    }

}
