<?php

namespace App\Http\Controllers\Api\AppSetting;

use Response;
use Validator;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\NotifyMe;
use Exception;

class ManageAlertController extends Controller
{

    public function getAllManageAlert(Request $request)
    {
        $alert = [];
        $alert_data = [];
        $type = "";
        $image = "";
        $variant_name = "";
        try {
            $data = $request->all();
            $rules = array(
                'device_id'         => 'required',
                'language_id'    => 'required',
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                if ($request->language_id == 'en') {
                    $alert = NotifyMe::where('status', 0)->where('device_id', $request->device_id)->with(['variant' => function ($q) {
                        $q->with(['product' => function ($query) {
                            $query->select('title_english as title', 'product_id');
                        }]);
                        $q->select('variant_id', 'option1_english as option1', 'option2_english as option2', 'option3_english as option3', 'image', 'price', 'strike_price', 'status', 'inventory_policy', 'product_id');
                    }])->get();
                } else if ($request->language_id == 'ar') {
                    $alert = NotifyMe::where('status', 0)->where('device_id', $request->device_id)->with(['variant' => function ($q) {
                        $q->with(['product' => function ($query) {
                            $query->select('title_arabic as title', 'product_id');
                        }]);
                        $q->select('variant_id', 'option1_arabic as option1', 'option2_arabic as option2', 'option3_arabic as option3', 'image', 'price', 'strike_price', 'status', 'inventory_policy', 'product_id');
                    }])->get();
                }
                foreach ($alert as $c) {
                    $Product_title = $c['variant']['product']['title'];
                    $title = "";
                    $variant_name = "";
                    $type = "";
                    $image = "";
                    if ($c['variant']['option1'] != null) {
                        $title = $c['variant']['option1'];
                    }
                    if ($c['variant']['option2'] != null) {
                        $title = $title . '/' . $c['variant']['option2'];
                    }
                    if ($c['variant']['option3'] != null) {
                        $title = $title . '/' . $c['variant']['option3'];
                    }
                    if ($title != 'Default Title' && $title != "") {
                        $variant_name = $title;
                    }
                    if ($c['variant']['image']) {
                        $image = $c['variant']['image'];
                    } else {
                        $image = $c['variant']['product']['image'];
                    }
                    $type = $c['type'];
                    if ($c['type'] == 'alert') {
                        $type = "notify_me";
                    }
                    $alert_data[] = array('variant_id' => $c['variant_id'], 'product_id' => $c['variant']['product_id'], 'title' => $Product_title, 'variant_name' => $variant_name, 'image' => $image, 'type' => $type, 'remind_month' => $c['remind_month'] ? $c['remind_month'] : '');
                }
                return response()->json(['alert' => $alert_data, 'success' => true]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json(['msg' => $msgErr, 'success' => false]);
        }
    }

    public function removeAlert(Request $request)
    {

        try {
            $data = $request->all();
            $rules = array(
                'device_id'         => 'required',
                'variant_id'    => 'required',
                'type' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $type = $request->type;
                if ($request->type == 'notify_me') {
                    $type = 'alert';
                }
                $alert = NotifyMe::where('status', 0)->where('device_id', $request->device_id)->where('variant_id',$request->variant_id)->where('type', $type)->delete();
                if ($request->language_id == 'ar') {
                            $msg = "تم ازالة التنبيه";
                        } elseif ($request->language_id == 'en') {
                            $msg = "Alert removed successfully";
                        }
                return response()->json(['msg' => $msg, 'success' => true]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json(['msg' => $msgErr, 'success' => false]);
        }
    }
}
