<?php

namespace App\Http\Controllers\Api\AppSetting;

use Response;
use App\Http\Controllers\Admin\Helper\HelperController;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\Variant;
use App\Models\Customer;
use Exception;
use GuzzleHttp\Client;

class OrderController extends Controller
{
    public $client;
    public function __construct()
    {
        $this->client = new Client();
    }

    public function getCustomerOrder(Request $request)
    {
        try {
            $order_data = [];
            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'orders.json?customer_id=' . $request->customer_id . '&status=any&fields=created_at,closed_at,cancelled_at,cancel_reason,contact_email,fulfillment_status,financial_status,id,name,total-price,line_items,shipping_address'
            );
            if ($res->getStatusCode() == 200) {
                $response_data = $res->getBody()->getContents();
                $orders = json_decode($response_data)->orders;

                $user =  Customer::where('customer_id', $request->customer_id)->first();
                if ($user) {
                    $currency = $user->currency;
                } else {
                    $currency = 'KWD';
                }
                foreach ($orders as $index => $order) {
                    $orders[$index]->total_price = strval($this->changeCurrency($currency, $order->total_price));
                    $orders[$index]->currency = $currency;
                    foreach ($order->line_items as $key => $item) {
                        $variant_title = "";
                        $product_title = "";
                        if ($request->language_id == 'en') {
                            $variant = Variant::where('variant_id', $item->variant_id)->select('option1_english as option1', 'option2_english as option2', 'option3_english as option3', 'image')->first();
                            $product = Product::where('product_id', $item->product_id)->select('title_english as title')->first();
                        } elseif ($request->language_id == 'ar') {
                            $variant = Variant::where('variant_id', $item->variant_id)->select('option1_arabic as option1', 'option2_arabic as option2', 'option3_arabic as option3', 'image')->first();
                            $product = Product::where('product_id', $item->product_id)->select('title_arabic as title')->first();
                        }
                        if ($item->variant_title) {
                            if ($variant->option1 != null) {
                                $variant_title = $variant->option1;
                            }
                            if ($variant->option2 != null) {
                                $variant_title = $variant_title . '/' . $variant->option2;
                            }
                            if ($variant->option3 != null) {
                                $variant_title = $variant_title . '/' . $variant->option3;
                            }
                        } else {
                            $variant_title = $product->title;
                        }
                        $product_title = $product->title;
                        $item->variant_title = $variant_title;
                        $item->title = $product_title;
                        $orders[$index]->line_items[$key] = $item;
                        $orders[$index]->line_items[$key]->image = $variant->image;
                    }
                }

                return response()->json([
                    'success' => true,
                    'orders' => $orders
                ]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }

    public function changeCurrency($currency, $price)
    {
        if ($currency) {
            if ($currency == 'KWD') {
                return $price;
            } else {
                $value = HelperController::changeCurrency('KWD', $currency, $price);
                return $value;
            }
        } else {
            return $price;
        }
    }
}
