<?php

namespace App\Http\Controllers\Api\AppSetting;

use App\Http\Controllers\Admin\Helper\HelperController;
use App\Http\Controllers\Controller;
use App\Models\ExclusiveSection;
use App\Models\FlashSale;
use App\Models\Product;
use App\Models\ProductVideo;
use App\Models\ScanProductCount;
use App\Models\UserToken;
use App\Models\Variant;
use App\Models\Wishlist;
use Response;
use Validator;
use Exception;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Carbon\Carbon;

class ProductDetailController extends Controller
{
    public $client;
    public function __construct()
    {
        $this->client = new Client();
    }
    public function getProductDetail(Request $request)
    {
        $product = Product::with('varients')->with(['how_to_use'=>function($query) use($request){
            $query->where('language',$request->language_id);
        }])->with('reviews')->with('recommanded')->with('ingredients')->where('product_id', $request->product_id)->first();
        $productDetail = [];
        $recommandData = [];
        $rd = [];
        $categoryData = [];
        $cd = [];
        $reviewData = [];
        $rwd = [];
        $imageData = [];
        $imd = [];
        $ingredientData = [];
        $ingd = [];

        if ($product) {
            $title = $request->language_id == 'en' ? $product->title_english : $product->title_arabic;
            $description = $request->language_id == 'en' ? $product->description_english : $product->description_arabic;
            $variant_status = Variant::where('variant_id', $product->variant_id)->first();
            $status = $variant_status->status;
            $wishlist = Wishlist::where('customer_id', $request->customer_id)->where('variant_id', $product->variant_id)->where('status', 1)->first();
            $user =  UserToken::where('device_id', $request->device_id)->first();
            if ($user) {
                $currency = $user->currency;
            } else {
                $currency = 'KWD';
            }
            if ($wishlist) {
                $is_wishlist = true;
            } else {
                $is_wishlist = false;
            }
            if ($product->strike_price) {
                // $strike_price = $product->strike_price;
                $strike_price = $this->changeCurrency($currency, $product->strike_price);
                $label = "SALE";
            } else {
                $label = "";
                $strike_price = "";
            }
            if ($status == 'pre_order') {
                if ($product->stock) {
                    $status = 'pre_order';
                } else {
                    if ($product->inventory_policy == 'deny') {
                        $status = "notify_me";
                    } else if ($product->inventory_policy == 'continue') {
                        $status = 'pre_order';
                    }
                }
            }
            $stock_left = "";
            $exclusiveData = $this->checkProductInExclusiveSection($product->product_id, $request->language_id);
            if ($exclusiveData) {
                if (!empty($exclusiveData[0])) {
                    $label = $exclusiveData[0];
                }
                $no_of_second = $exclusiveData[1];
                if($exclusiveData[2]){
                  if($request->language_id == 'ar'){
                   $stock_left = "فقط" ." ".$product->stock ." "."كمية متبقية"; 
                }else{
                   $stock_left = "Only " . $product->stock . " left in stock"; 
                }  
                }
                
                
            }
            if ($request->language_id == 'en') {
                $variant_title = Variant::where('variant_id', $product->variant_id)->select('option1_english as option1', 'option2_english as option2', 'option3_english as option3')->first();
            } else if ($request->language_id == 'ar') {
                $variant_title = Variant::where('variant_id', $product->variant_id)->select('option1_arabic as option1', 'option2_arabic as option2', 'option3_arabic as option3')->first();
            }
            if ($variant_title) {
                $show_option = true;
                $v_title = "";
                if ($variant_title->option1 == 'Default Title' || $variant_title->option1 == '') {
                    $show_option = false;
                }
                if ($variant_title->option1 != null) {
                    $v_title = $variant_title->option1;
                }
                if ($variant_title->option2 != null) {
                    $v_title = $v_title . '/' . $variant_title->option2;
                }
                if ($variant_title->option3 != null) {
                    $v_title = $v_title . '/' . $variant_title->option3;
                }
            }
            $price = $this->changeCurrency($currency, $product->price);

            $productDetail['product_id']    = $product->product_id;
            $productDetail['image']         = $product->image;
            $productDetail['title']         = $title;
            $productDetail['description']   = $description;
            $productDetail['stock']         = $product->stock;
            $productDetail['price']         = "$price";
            $productDetail['strike_price']  = "$strike_price";
            $productDetail['currency']      = $currency;
            $productDetail['label']         = $label;
            $productDetail['variant_id']    = $product->variant_id;
            $productDetail['status']        = $status;
            $productDetail['show_option']   = $show_option;
            $productDetail['variant_title'] = $v_title;
            $productDetail['seconds']       = $no_of_second;
            $productDetail['stock_left']    = $stock_left;
            $productDetail['is_wishlist']    = $is_wishlist;

            if (count($product->recommanded) > 0) {
                foreach ($product->recommanded as $recommand) {
                    $singleProduct = Product::where('product_id', $recommand->content_product_id)->first();
                    if ($singleProduct->strike_price) {
                        // $strike_price = $singleProduct->strike_price;
                        $strike_price = $this->changeCurrency($currency, $singleProduct->strike_price);
                        $recommandLabel = "SALE";
                    } else {
                        $recommandLabel = "";
                        $strike_price = "";
                    }
                    $exclusive_rlabel = $this->checkProductInExclusiveSection($recommand->content_product_id, $request->language_id);
                    if ($exclusive_rlabel) {
                        if ($exclusive_rlabel[0]) {
                            $recommandLabel = $exclusive_rlabel[0];
                        }
                        $noOfSecond = $exclusive_rlabel[1];
                    }
                    $singleProductPrice = $this->changeCurrency($currency, $singleProduct->price);
                    $recommandData['title'] = $request->language_id == 'en' ? $singleProduct->title_english : $singleProduct->title_arabic;
                    $recommandData['price'] = "$singleProductPrice";
                    $recommandData['image'] = $singleProduct->image;
                    $recommandData['strike_price'] = "$strike_price";
                    $recommandData['product_id'] = $singleProduct->product_id;
                    $recommandData['label'] = $recommandLabel ? $recommandLabel : "";
                    $recommandData['seconds'] = $noOfSecond;
                    $rd[] = $recommandData;
                }
            }
            $productDetail['recommand'] = $rd;
            if (count($product->how_to_use) > 0) {
                foreach ($product->how_to_use as $use) {
                    
                    $categoryData['title'] = $request->language_id == 'en' ? $use->title_english : $use->title_arabic;
                    $categoryData['url'] = $use->image ? 'storage/app/' . $use->image : "";
                    $categoryData['cover_image'] = $use->cover_image ? 'storage/app/' . $use->cover_image : "";
                    $categoryData['upload_type'] = $use->upload_type ?  $use->upload_type : "";
                    
                    $cd[] = $categoryData;
                }
            }
            $productDetail['how_to_use'] = $cd;

            if (count($product->reviews) > 0) {
                foreach ($product->reviews as $review) {
                    $reviewData['title'] =  $request->language_id == 'en' ? $review->title_english : $review->title_arabic;
                    $reviewData['text'] = $request->language_id == 'en' ? $review->text_english : $review->text_arabic;
                    $reviewData['source_icon'] = $review->source_icon ? 'storage/app/' . $review->source_icon : "";
                    $reviewData['upload_source'] = $review->upload_source ? 'storage/app/' . $review->upload_source : "";

                    $rwd[] = $reviewData;
                }
            }
            $productDetail['reviews'] = $rwd;

            if (count($product->ingredients) > 0) {
                foreach ($product->ingredients as $ingre) {
                    $ingredientData['title'] = $request->language_id == 'en' ? $ingre->title_english : $ingre->title_arabic;
                    $ingredientData['image'] = $ingre->icon ? 'storage/app/' . $ingre->icon : "";
                    $path_info = pathinfo($ingre->icon);
                    $ingredientData['image_type'] = $ingre->icon != NULL ? $path_info['extension'] : "";
                    $ingd[] = $ingredientData;
                }
            }
            $productDetail['ingredients'] = $ingd;

            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'products/' . $request->product_id . '/images.json'
            );
            if ($res->getStatusCode() == 200) {
                $response_data = $res->getBody()->getContents();
                $productImages = json_decode($response_data)->images;

                foreach ($productImages as $image) {
                    $imageData['id'] = $image->id;
                    $path_info = pathinfo($image->src);
                    if (explode("?", $path_info['extension'])[0] != 'gif') {
                        $imageData['upload_banner_type'] = 'image';
                    } else {
                        $imageData['upload_banner_type'] = explode("?", $path_info['extension'])[0];
                    }
                    $imageData['banner_url'] = $image->src;

                    $imageData['cover_image'] = "";
                    $imageData['destination'] = "";
                    $imageData['banner_language'] = "english";
                    $imd[] = $imageData;
                }
            }

            $product_video = ProductVideo::where('product_id', $request->product_id)->get()->toArray();
            if (count($product_video)) {
                foreach ($product_video as $pvideo) {
                    $imageData['id'] = $pvideo['id'];
                    $imageData['upload_banner_type'] = 'video';
                    $imageData['banner_url'] = $pvideo['video'] ? 'storage/app/' . $pvideo['video'] : '';

                    $imageData['cover_image'] = $pvideo['cover_image'] ? 'storage/app/' . $pvideo['cover_image'] : '';
                    $imageData['destination'] = "";
                    $imageData['banner_language'] = "english";
                    $imd[] = $imageData;
                }
            }
            $productDetail['banner'] = $imd;
        }
        return response()->json([
            'success' => true,
            'product' => $productDetail
        ]);
    }

    public function changeCurrency($currency, $price){
        if($currency){
            if($currency == 'KWD'){
                return $price;
            }else{
                $value = HelperController::changeCurrency('KWD', $currency, $price);
                return $value;
            }
        }else{
            return $price;
        }
    }

    public function checkProductInExclusiveSection($product_id, $language_id)
    {
        $label = "";
        $no_of_second = 0;
        $show_quantity = 0;
        $now = Carbon::now('Asia/Kuwait')->format('Y-m-d');
        $now_time = Carbon::now('Asia/Kuwait')->format('H:i:s');
        $flash_id = 0;
        $flash = FlashSale::where('start_date', '<=', $now)->get();
        foreach ($flash as $key => $f) {
            if ($f->status) {
                if ($f->start_date == $now) {
                    if ($f->start_time <= $now_time) {
                        if ($f->end_date >= $now) {
                            if ($f->end_date == $now) {
                                if ($f->end_time >= $now_time) {
                                    $flash_id = $f->id;
                                    break;
                                }
                            } else {
                                $flash_id = $f->id;
                                break;
                            }
                        }
                    }
                } else {
                    if ($f->end_date >= $now) {
                        if ($f->end_date == $now) {
                            if ($f->end_time >= $now_time) {
                                $flash_id = $f->id;
                                break;
                            }
                        } else {
                            $flash_id = $f->id;
                            break;
                        }
                    }
                }
            } else {
                if ($f->start_date == $now) {
                    if ($f->start_time <= $now_time) {
                        $flash_id = $f->id;
                        break;
                    }
                } else {
                    $flash_id = $f->id;
                    break;
                }
            }
        }
        if ($flash_id) {
            $flash_sale_new = FlashSale::where('id', $flash_id)->with('exclusive_section')->first();
            if ($flash_sale_new) {
                $exclusive = ExclusiveSection::where('flash_sale_id', $flash_sale_new->id)->where('product_id', $product_id)->first();
                if ($exclusive) {
                    if ($language_id == 'en') {
                        $label = $exclusive->label_english;
                    } else {
                        $label = $exclusive->label_arabic;
                    }
                    if ($flash_sale_new->status) {
                        $d1 = strtotime($flash_sale_new->end_date." ".$flash_sale_new->end_time);
                        $d2 = strtotime($now." ".$now_time);
                        $totalSecondsDiff = abs($d1-$d2); 
                        $no_of_second = $totalSecondsDiff;
                    } else {
                        $no_of_second = 0;
                    }
                    if($flash_sale_new->is_show_quantity){
                        $show_quantity = 1;
                    }
                }
            }
        }
        return [$label, $no_of_second,$show_quantity];
    }

    public function scanProductDetail(Request $request)
    {
        try {
            $data = $request->all();
            $rules = array(
                'language_id' => 'required',
                'barcode' => 'required',
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $status = [
                    '0' => 'inactive',
                    '1' => 'active',
                ];
                $product = Product::where('barcode', '=', trim($request->barcode))->where('is_delete', '=', 0)->first();
                if ($product) {
                    if ($request->language_id == 'en') {
                        $how_to_use = Product::where('barcode', '=', $request->barcode)->with(['how_to_use' => function ($query) use($request) {
                            $query->where('language',$request->language_id)->select('product_id', 'title_english as title', 'upload_type', 'cover_image', 'image as url');
                        }])->select('title_english as title', 'description_english as description', 'image', 'product_id')->first();
                    } else if ($request->language_id == 'ar') {
                        $how_to_use = Product::where('barcode', '=', $request->barcode)->with(['how_to_use' => function ($query) use($request) {
                            $query->where('language',$request->language_id)->select('product_id', 'title_arabic as title', 'upload_type', 'cover_image', 'image as url');
                        }])->select('title_arabic as title', 'description_arabic as description', 'image', 'product_id')->first();
                    }
                    $how_to_use['status'] = $status[$product->active];
                    foreach ($how_to_use['how_to_use'] as $how) {
                        if ($how['cover_image'] == null) {
                            $how['cover_image'] = "";
                        } else {
                            $how['cover_image'] = 'storage/app/' . $how['cover_image'];
                        }
                        if ($how['url'] == null) {
                            $how['url'] = "";
                        } else {
                            $how['url'] = 'storage/app/' . $how['url'];
                        }
                    }
                    // $scanCountExist = ScanProductCount::where('product_id', $product->product_id)->first();
                    // if( $scanCountExist){
                    //     ScanProductCount::where('product_id', $product->product_id)->update([
                    //         'scan_correct'=> $scanCountExist->scan_correct + 1,
                    //     ]);
                    // }else{
                    //     ScanProductCount::create([
                    //         'product_id' => $product->product_id,
                    //         'scan_correct'=> 1,
                    //     ]);
                    // }
                    ScanProductCount::insert([
                        'product_id'    => $product->product_id,
                        'name'          => 'scan_correct',
                        'created_at'    => Carbon::now('Asia/Kuwait'),
                        'updated_at'    => Carbon::now('Asia/Kuwait')
                    ]);
                    return response()->json([
                        'success' => true,
                        'product' => $how_to_use
                    ]);
                } else {
                    if ($request->language_id == 'ar') {
                        $msg =  "هذا ليس منتج من أفراح بيوتي 😲";
                    } elseif ($request->language_id == 'en') {
                        $msg = "This Not an Afrah product 😲!";
                    }

                    // $scanCountExist = ScanProductCount::where('scan_correct', 0)->where('scan_wrong', '>', 0)->first();
                    // if( $scanCountExist){
                    //     ScanProductCount::where('id', $scanCountExist->id)->update([
                    //         'scan_wrong'=> $scanCountExist->scan_wrong + 1,
                    //     ]);
                    // }else{
                    //     ScanProductCount::create([
                    //         'scan_wrong'=> 1,
                    //     ]);
                    // }

                    ScanProductCount::insert([
                        'product_id'    => NULL,
                        'name'          => 'scan_wrong',
                        'created_at'    => Carbon::now('Asia/Kuwait'),
                        'updated_at'    => Carbon::now('Asia/Kuwait')
                    ]);
                    
                    return response()->json([
                        'success' => false,
                        'msg' => $msg
                    ]);
                }
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msg = "هذا ليس منتج من أفراح بيوتي 😲";
            } elseif ($request->language_id == 'en') {
                $msg = "This Not an Afrah product 😲!";
            }
            return response()->json([
                'success' => false,
                'msg' => $msg
            ]);
        }
    }
}
