<?php

namespace App\Http\Controllers\Api\AppSetting;

use App\Http\Controllers\Controller;
use App\Models\CurrencyPoint;
use App\Models\Customer;
use App\Models\Milestone;
use App\Models\MilestonePoint;
use GuzzleHttp\Client;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;

class RegisterController extends Controller
{
    public $client;

    public function __construct()
    {
        $this->client = new Client();
    }
    public function createCustomer(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "first_name"    => "required",
                "last_name"     => "required",
                "email"         => "required|email",
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $customerData['customer'] = $data;
                $customerDetail = [];

                $customerRes = $this->client->request(
                    'POST',
                    env('SHOPIFY_API_REQUEST_URL') . 'customers.json',
                    [
                        'form_params' => $customerData
                    ]
                );
                if ($customerRes->getStatusCode() == 201) {
                    $response_data = $customerRes->getBody()->getContents();
                    $customerDetail = json_decode($response_data)->customer;
                }
                return response()->json([
                    'success' => true,
                    'customer' => $customerDetail
                ]);
            }
        } catch (Exception $e) {
            if ($e->getResponse()->getBody()) {
                $de = json_decode($e->getResponse()->getBody()->getContents());
                if ($de->errors) {
                    return response()->json([
                        'success' => false,
                        'errors' => $de->errors,
                    ]);
                } else {
                    if ($request->language_id == 'ar') {
                        $msgErr = "حدث خطأ";
                    } elseif ($request->language_id == 'en') {
                        $msgErr = "Something went wrong";
                    }
                    return response()->json([
                        'success' => false,
                        'msg' => $msgErr
                    ]);
                }
            } else {
                if ($request->language_id == 'ar') {
                        $msgErr = "حدث خطأ";
                    } elseif ($request->language_id == 'en') {
                        $msgErr = "Something went wrong";
                    }
                return response()->json([
                    'success' => false,
                    'msg' => $msgErr,
                ]);
            }
        }
    }

    public function signInWithSocial(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "email" => "required|email"
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'msg' => "The email must be a valid email address."
                ), 422);
            } else {
                $customer = Customer::where('email', $request->email)->first();
                if ($customer) {
                    $customer_points = $customer->point + $customer->extra_point;
                    $customerPoint = [
                        'customer_id'=> $customer->customer_id,
                        'level'=> NULL,
                        'point'=> $customer_points,
                        'next_point'=> 0,
                        'currency' => NULL,
                        'each_value' => NULL,
                        'each_point' => NULL
                    ];
                    $currencyPoint = CurrencyPoint::where('currency', $customer->currency)->select('currency', 'each_value', 'each_point')->first();
                    $next_point = Milestone::where('from_point' , ">", $customer_points)->select('level_english','from_point')->first();
                    $level = $this->getLevel($customer_points, $request->language_id);
                    if($next_point){
                        $point = abs($next_point->from_point - $customer_points);
                    }else{
                        $point = 0;
                    }
                    if($currencyPoint){
                        $customerPoint['level'] = $level;
                        $customerPoint['next_point'] = $point;
                        $customerPoint['currency'] = $currencyPoint->currency;
                        $customerPoint['each_value'] = $currencyPoint->each_value;
                        $customerPoint['each_point'] = $currencyPoint->each_point;
                    }else{
                        
                        $currencyPointData = CurrencyPoint::where('currency', 'KWD')->select('currency', 'each_value', 'each_point')->first();
                        $customerPoint['level'] = $level;
                        $customerPoint['next_point'] = $point;
                        $customerPoint['currency'] = $currencyPointData->currency;
                        $customerPoint['each_value'] = $currencyPointData->each_value;
                        $customerPoint['each_point'] = $currencyPointData->each_point;
                    }
                    return response()->json([
                        'success' => true,
                        'customer_info' => $customer,
                        'customer_milestone' => $customerPoint
                    ]);
                } else {
                    if ($request->language_id == 'ar') {
                        $msg = "الحساب غير صحيح";
                    } elseif ($request->language_id == 'en') {
                        $msg = "Invalid user";
                    }
                    return response()->json([
                        'success' => false,
                        'msg' => $msg
                    ]);
                }
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                        $msgErr = "حدث خطأ";
                } elseif ($request->language_id == 'en') {
                        $msgErr = "Something went wrong";
                }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }

    public function signUpWithSocial(Request $request)
    {
        try {
            $data = $request->all();

            $rules = array(
                "email" => "required|email"
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'msg' => "The email must be a valid email address.",
                ), 422);
            } else {
                $customer = Customer::where('email', $request->email)->first();
                if ($customer) {
                    return response()->json([
                        'success' => false,
                        'msg' => "User already exists"
                    ]);
                } else {
                    $data['verified_email'] = true;
                    $customerData['customer'] = $data;
                    $customerDetail = [];

                    $customerRes = $this->client->request(
                        'POST',
                        env('SHOPIFY_API_REQUEST_URL') . 'customers.json',
                        [
                            'form_params' => $customerData
                        ]
                    );
                    if ($customerRes->getStatusCode() == 201) {
                        $response_data = $customerRes->getBody()->getContents();
                        $customerDetail = json_decode($response_data)->customer;
                    }

                    // echo $customerDetail->currency;die;

                    $currencyPoint = CurrencyPoint::where('currency', $customerDetail->currency)->select('currency', 'each_value', 'each_point')->first();
                    $next_point = Milestone::where('from_point' , ">", 0)->select('level_english','from_point')->first();
                    $level = $this->getLevel(0, $request->language_id);
                    if($next_point){
                        $point = abs($next_point->from_point - 0);
                    }else{
                        $point = 0;
                    }
                    $customerPoint = [
                        'customer_id'=> $customerDetail->id,
                        'level'=> $level,
                        'point'=> 0,
                        'next_point'=> $point,
                        'currency' => NULL,
                        'each_value' => NULL,
                        'each_point' => NULL
                    ];
                    if($currencyPoint){
                        $customerPoint['level'] = $level;
                        $customerPoint['next_point'] = $point;
                        $customerPoint['currency'] = $currencyPoint->currency;
                        $customerPoint['each_value'] = $currencyPoint->each_value;
                        $customerPoint['each_point'] = $currencyPoint->each_point;
                    }else{
                        $currencyPointData = CurrencyPoint::where('currency', 'KWD')->select('currency', 'each_value', 'each_point')->first();
                        $customerPoint['level'] = $level;
                        $customerPoint['currency'] = $currencyPointData->currency;
                        $customerPoint['each_value'] = $currencyPointData->each_value;
                        $customerPoint['each_point'] = $currencyPointData->each_point;
                    }

                    return response()->json([
                        'success' => true,
                        'customer' => $customerDetail,
                        'customer_milestone' => $customerPoint
                    ]);
                }
            }
        } catch (Exception $e) {

            if ($e->getResponse()->getBody()) {
                $de = json_decode($e->getResponse()->getBody()->getContents());
                if ($de->errors) {
                    if ($de->errors->email) {
                        if ($request->language_id == 'ar') {
                            $msgErr = "الحساب مسجل مسبقاً";
                        } elseif ($request->language_id == 'en') {
                            $msgErr = "User already exists";
                        }
                        return response()->json([
                            'success' => false,
                            'msg' => $msgErr,
                        ]);
                    } else {
                        return response()->json([
                            'success' => false,
                            'errors' => $de->errors,
                        ]);
                    }
                } else {
                    if ($request->language_id == 'ar') {
                        $msgErr = "حدث خطأ";
                    } elseif ($request->language_id == 'en') {
                        $msgErr = "Something went wrong";
                    }
                    return response()->json([
                        'success' => false,
                        'msg' => $msgErr,
                    ]);
                }
            } else {
                if ($request->language_id == 'ar') {
                        $msgErr = "حدث خطأ";
                    } elseif ($request->language_id == 'en') {
                        $msgErr = "Something went wrong";
                    }
                    return response()->json([
                        'success' => false,
                        'msg' => $msgErr,
                    ]);
            }
        }
    }

    public function getLevel($point,$language){

        $max = Milestone::max('from_point');
        if($point >= $max){
            if ($language == 'en') {
                $milestone = Milestone::where('from_point', '=',$max)->select('level_english as level')->first();
            } else if ($language == 'ar') {
                $milestone = Milestone::where('from_point', '=',$max)->select('level_arabic as level')->first();
            }
            if($milestone){
                return $milestone->level;
            }else{
                return 'Not Available';
            }
            // echo $milestone->level;die;
        }else{
            if ($language == 'en') {
                $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('level_english as level')->first();
            } else if ($language == 'ar') {
                $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('level_arabic as level')->first();
            }
            if($milestone){
                return $milestone->level;
            }else{
                return 'Not Available';
            }
            // echo $milestone->level;die;
        }
    }
}
