<?php

namespace App\Http\Controllers\Api\AppSetting;

use Response;
use Validator;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\ChallengeContent;
use App\Models\CollectionByGoal;
use App\Models\UserToken;
use App\Models\NotifyMe;
use Carbon\Carbon;
use App\Models\Variant;
use App\Models\Product;
use App\Models\ManualNotification;
use App\Models\Menu;
use App\Models\NormalCollection;
use App\Models\RemindNotification;
use App\Models\UsedChallenge;
use Exception;
use Illuminate\Support\Facades\Log;

class UserTokenController extends Controller
{

    public function saveToken(Request $request)
    {
        try {
            $data = $request->all();
            $rules = array(
                'device_id' => 'required',
                'device_token' => 'required',
                'language_id' => 'required',
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $user = UserToken::where('device_id', $request->device_id)->first();
                if ($user) {
                    UserToken::where('device_id', $request->device_id)->update([
                        'device_token' => $request->device_token,
                        'language_id' => $request->language_id,
                        'currency' => $request->currency
                    ]);
                } else {
                    UserToken::create([
                        'device_id' => $request->device_id,
                        'device_token' => $request->device_token,
                        'language_id' => $request->language_id,
                        'currency' => $request->currency
                    ]);
                }
                if ($request->language_id == 'ar') {
                    $msg = "تم الحفظ بنجاح";
                } elseif ($request->language_id == 'en') {
                    $msg = "Token saved successfully";
                }
                return response()->json([
                    'success' => true,
                    'msg' => $msg
                ]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }
    public function getCurrency(Request $request)
    {
        try {
            $data = $request->all();
            $rules = array(
                'device_id' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $user = UserToken::where('device_id', $request->device_id)->first();
                if ($user) {
                    return response()->json([
                        'success' => true,
                        'currency' => $user->currency
                    ]);
                } else {
                    if ($request->language_id == 'ar') {
                        $msg = "لم يتم اختيار اي عملة بعد";
                    } elseif ($request->language_id == 'en') {
                        $msg = "User don't have any currency";
                    }
                    return response()->json([
                        'success' => false,
                        'msg' => $msg
                    ]);
                }
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }
    public function changeCurrency(Request $request)
    {
        try {
            $data = $request->all();
            $rules = array(
                'device_id' => 'required',
                'currency' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                
                // var_dump($user_curr);die;
                // echo $user_curr;die;
                UserToken::where('device_id', $request->device_id)->update(
                    [
                        'currency' => $request->currency
                        ]
                    );
                $user_curr = UserToken::where('device_id', $request->device_id)->first();
                if ($request->language_id == 'ar') {
                    $msg = "تم تغيير العملة بنجاح";
                } elseif ($request->language_id == 'en') {
                    $msg = "Currency changed successfully";
                }
                return response()->json([
                    'success' => true,
                    'msg' => $msg,
                    'currency' => $user_curr->currency
                ]);
            }
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'msg' => $e->getMessage()
            ]);
        }
    }
    public function sendNotification()
    {

        $serverKey = trim(env('SERVER_KEY'));

        // echo $serverKey;die;

        $users = NotifyMe::where('status', '!=', 1)->get();
        $response = [];

        if ($users) {
            foreach ($users as $key => $user) {
                if (trim($user->type) == 'alert') {
                    $response[] =  $this->sendAlertNotification($user, $serverKey);
                } else {
                    $response[] =  $this->sendRemindMeNotification($user, $serverKey);
                }
            }
        }
        $response[] = $this->sendManualNotification();
        $response[] = $this->sendChallengeNotification();
        return response()->json(['message' => 'Notification send successfully', 'data' => $response]);
    }

    public function sendAlertNotification($user, $serverKey)
    {
        $result = NULL;
        $variant = Variant::where('variant_id', $user->variant_id)->where('stock', '!=', 0)->first();
        if ($variant) {
            $token = UserToken::where('device_id', $user->device_id)->first();
            if($token){
                $product = Product::where('product_id', $variant->product_id)->first();
                $notification_id = trim($token->device_token);
                $variant_title = "";
                if ($variant->option1_english != null) {
                    $variant_title = $variant->option1_english;
                }
                if ($variant->option2_english != null) {
                    $variant_title = $variant_title . '/' . $variant->option2_english;
                }
                if ($variant->option3_english != null) {
                    $variant_title = $variant_title . '/' . $variant->option3_english;
                }
                $title = 'Back in stock! 🤩';
                if ($variant_title == 'Default Title') {
                    $message = $product->title_english . ' ' . 'back in stock!';
                } else {
                    $message = $product->title_english . ' (' . $variant_title . ')' . ' ' . 'back in stock!';
                }
                $image  = $variant->image ? $variant->image : $product->image;
                $icon  = "https://image.flaticon.com/icons/png/512/270/270014.png";/*Default Icon*/
                $type = 'basic';
                $variant_id = $variant->variant_id;
                $product_id = $variant->product_id;
                // $accesstoken = "AAAAzNblAPg:APA91bGlsh4x1v6BoRSCNQMPf0YkSuODmrdQPabyojyNsnezxNIVoIL_DTAWWZvQBHEFLB6Ap30U_gJ-cY_ykQPrF53_pA-6eZxPunLRmmgSeAzEueaEV1_z38TTI_Ae5sECbR7SLhi7";
                $accesstoken = $serverKey;
    
                $URL = 'https://fcm.googleapis.com/fcm/send';
    
                $post_data = '{
                "to" : "' . $notification_id . '",
                "data" : {
                    "body" : "",
                    "title" : "' . $title . '",
                    "type" : "' . $type . '",
                    "message" : "' . $message . '",
                    "variant_id": "' . $variant_id . '" 
                    "product_id": "' . $product_id . '" 
                    "notification_type": "alert",
                    "manual_id": "0",
                    "variant_id": "' . $variant_id . '" ,
                    "product_id": "' . $product_id . '" ,
                },
                "notification" : {
                    "body" : "' . $message . '",
                    "title" : "' . $title . '",
                    "type" : "' . $type . '",
                    "largeIcon"	: "large_icon",
                    "message" : "' . $message . '",
                    "icon" : "' . $icon . '",
                    "image" : "' . $image . '",
                    "sound" : "1",
                    "badge" : "1"
                    "mutable-content": "1",
                    "notification_type": "alert",
                    "manual_id": "0",
                    "variant_id": "' . $variant_id . '" ,
                    "product_id": "' . $product_id . '" ,
                    },
                    
                }';
    
                $crl = curl_init();
    
                $headr = array();
                $headr[] = 'Content-type: application/json';
                $headr[] = 'Authorization: key=' . $accesstoken;
    
                curl_setopt($crl, CURLOPT_SSL_VERIFYPEER, false);
    
                curl_setopt($crl, CURLOPT_URL, $URL);
                curl_setopt($crl, CURLOPT_HTTPHEADER, $headr);
    
                curl_setopt($crl, CURLOPT_POST, true);
                curl_setopt($crl, CURLOPT_POSTFIELDS, $post_data);
                curl_setopt($crl, CURLOPT_RETURNTRANSFER, true);
    
                $result = curl_exec($crl);
    
    
                if ($result === FALSE) {
                    die('FCM Send Error: ' . curl_error($crl));
                } else {
                    // $result = json_decode($result,true);
    
                    $user->status = 1;
                    $user->save();
                    $response[] = $result;
    
                    curl_close($crl);
                }

            }
        }
        return $result;
    }

    public function sendRemindMeNotification($user, $serverKey)
    {
        $updated_at = $user->updated_at;
        $today      = Carbon::now()->format('Y-m-d');
        $add_month  = $user->remind_month;
        $start_date =  $updated_at->addMonths($add_month)->format('Y-m-d');

        $result = NULL;

        if ($start_date == $today) {
            $variant            = Variant::where('variant_id', $user->variant_id)->first();
            $token              = UserToken::where('device_id', $user->device_id)->first();
            $product            = Product::where('product_id', $variant->product_id)->first();

            if($token){
                $notification_id    = trim($token->device_token);
                $variant_title = "";
                if ($variant->option1_english != null) {
                    $variant_title = $variant->option1_english;
                }
                if ($variant->option2_english != null) {
                    $variant_title = $variant_title . '/' . $variant->option2_english;
                }
                if ($variant->option3_english != null) {
                    $variant_title = $variant_title . '/' . $variant->option3_english;
                }
                $title = 'Notification';
                if ($variant_title == 'Default Title') {
                    $message = "You want to see " . $product->title_english;
                } else {
                    $message = "You want to see " . $product->title_english . ' (' . $variant_title . ')';
                }
                $image  = $variant->image ? $variant->image : $product->image;
                $icon  = "https://image.flaticon.com/icons/png/512/270/270014.png";/*Default Icon*/
                $type = 'basic';
                $variant_id = $variant->variant_id;
                $product_id = $variant->product_id;
                // $accesstoken = "AAAAzNblAPg:APA91bGlsh4x1v6BoRSCNQMPf0YkSuODmrdQPabyojyNsnezxNIVoIL_DTAWWZvQBHEFLB6Ap30U_gJ-cY_ykQPrF53_pA-6eZxPunLRmmgSeAzEueaEV1_z38TTI_Ae5sECbR7SLhi7";
                $accesstoken = $serverKey;
    
                $URL = 'https://fcm.googleapis.com/fcm/send';
    
                $post_data = '{
                "to" : "' . $notification_id . '",
                "data" : {
                    "body" : "",
                    "title" : "' . $title . '",
                    "type" : "' . $type . '",
                    "message" : "' . $message . '",
                    "image" : "' . $image . '",
                    "notification_type": "remind_me",
                    "manual_id": "0",
                    "variant_id": "' . $variant_id . '" ,
                    "product_id": "' . $product_id . '" ,
                },
                "notification" : {
                    "body" : "' . $message . '",
                    "title" : "' . $title . '",
                    "type" : "' . $type . '",
                    "message" : "' . $message . '",
                    "icon" : "' . $icon . '",
                    "image" : "' . $image . '",
                    "sound" : "default",
                    "mutable-content": "1",
                    "notification_type": "remind_me",
                    "manual_id": "0",
                    "variant_id": "' . $variant_id . '" ,
                    "product_id": "' . $product_id . '" ,
                    },
    
                }';
    
                $crl = curl_init();
    
                $headr = array();
                $headr[] = 'Content-type: application/json';
                $headr[] = 'Authorization: key=' . $accesstoken;
    
                curl_setopt($crl, CURLOPT_SSL_VERIFYPEER, false);
    
                curl_setopt($crl, CURLOPT_URL, $URL);
                curl_setopt($crl, CURLOPT_HTTPHEADER, $headr);
    
                curl_setopt($crl, CURLOPT_POST, true);
                curl_setopt($crl, CURLOPT_POSTFIELDS, $post_data);
                curl_setopt($crl, CURLOPT_RETURNTRANSFER, true);
    
                $result = curl_exec($crl);
    
    
                if ($result === FALSE) {
                    die('FCM Send Error: ' . curl_error($crl));
                } else {
    
                    $user->updated_at = Carbon::now()->toDateTimeString();
                    $user->save();
    
                    curl_close($crl);
                }
            }
        }
        return $result;
    }

    public function notifyMe(Request $request)
    {
        $data = $request->all();
        $rules = array(
            'device_id'         => 'required',
            'variant_id'    => 'required',
        );

        $validator = Validator::make($data, $rules);

        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
            ), 422);
        } else {
            $notify = NotifyMe::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('status', 0)->first();
            if ($notify) {  
                 if ($request->language_id == 'ar') {
                        $msg = "شكراً! سوف يتم ابلاغك فور توفر المنتج ❤️";
                    } elseif ($request->language_id == 'en') {
                        $msg = "Thanks! you will get notified once the product in stock again!❤️";
                    }
                return response()->json(['success' => true, 'msg' => $msg]);
            } else {
                $product_id = NULL;
                $variant = Variant::where('variant_id', $request->variant_id)->first();
                if($variant){
                    $product_id = $variant->product_id;
                    NotifyMe::create(
                        [
                            'device_id'     => $request->device_id,
                            'variant_id'    => $request->variant_id,
                            'product_id'    => $product_id,
                            'type'          => 'alert',
                        ]
                    );
                    if ($request->language_id == 'ar') {
                            $msg = "شكراً! سوف يتم ابلاغك فور توفر المنتج ❤️";
                        } elseif ($request->language_id == 'en') {
                            $msg = "Thanks! you will get notified once the product in stock again!❤️";
                        }
                    return response()->json(['success' => true, 'msg' => $msg]);
                }else{
                    if ($request->language_id == 'ar') {
                        $msgErr = "حدث خطأ";
                    } elseif ($request->language_id == 'en') {
                        $msgErr = "Something went wrong";
                    }
                    return response()->json(['success' => false, 'msg' => $msgErr]);
                }
            }
        }
    }

    public function remindMe(Request $request)
    {
        $data = $request->all();
        $rules = array(
            'device_id'         => 'required',
            'variant_id'    => 'required',
            'remind_month'  => 'required',
        );

        $validator = Validator::make($data, $rules);

        if ($validator->fails()) {
            return Response::json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray(),
            ), 422);
        } else {
            $remind = NotifyMe::where('device_id', $request->device_id)->where('variant_id', $request->variant_id)->where('type', 'remind_me')->where('status', 0)->first();
            if ($remind) {
                NotifyMe::where('id', $remind->id)->update(
                    [
                        'remind_month'  => $request->remind_month,
                    ]
                );
                if ($request->language_id == 'ar') {
                        $msg = "شكراً! سوف يتم تذكيرك بشراء المنتج ❤️";
                    } elseif ($request->language_id == 'en') {
                        $msg = "Thanks! you, we will remind u again!❤️";
                    }
                return response()->json(['success' => true, 'msg' => $msg]);
            } else {
                $product_id = NULL;
                $variant = Variant::where('variant_id', $request->variant_id)->first();
                if($variant){
                    $product_id = $variant->product_id;
                }
                $create = NotifyMe::create(
                    [
                        'device_id'     => $request->device_id,
                        'variant_id'    => $request->variant_id,
                        'product_id'    => $product_id,
                        'type'          => 'remind_me',
                        'remind_month'  => $request->remind_month,
                    ]
                );
                if($create){
                    // $productId = Variant::where('variant_id', $request->variant_id)->first();
                    // $remindNotifyExist = RemindNotification::where('product_id', $productId->product_id)->first();
                    // if ($remindNotifyExist) {
                    //     RemindNotification::where('id', $remindNotifyExist->id)->update(
                    //         [
                    //             'count'  => $remindNotifyExist->count + 1,
                    //         ]
                    //     );
                    // } else {
                    //     $create = RemindNotification::create(
                    //         [
                    //             'product_id'    => $productId->product_id,
                    //             'count'    => 1,
                    //         ]
                    //     );
                       
                    // }
                }
                 if ($request->language_id == 'ar') {
                        $msg = "شكراً! سوف يتم تذكيرك بشراء المنتج ❤️";
                    } elseif ($request->language_id == 'en') {
                        $msg = "Thanks! you, we will remind about this product!❤️";
                    }
                return response()->json(['success' => true, 'msg' => $msg]);
            }
        }
    }
    public function sendManualNotification()
    {
        $result = NULL;
        $response = [];
        $serverKey = trim(env('SERVER_KEY'));
        $notifications = ManualNotification::where('status', '=', 0)->get();

        if ($notifications) {
            $type = 'basic';
            $accesstoken = $serverKey;
            $URL = 'https://fcm.googleapis.com/fcm/send';
            $icon  = "https://image.flaticon.com/icons/png/512/270/270014.png";/*Default Icon*/
            $tokens = UserToken::get();

            foreach ($notifications as $notification) {
                $today          = Carbon::now('Asia/Kuwait');
                $date           = $notification->date;
                $greaterEqual   = $today->gte($date);

                if ($greaterEqual) {
                    $image      = $notification->image ? 'https://blankappsteam.com/shoppifyAfraj/' . '/storage/app/' . $notification->image : "";
                    $destinationType = "";
                    $destination = "";
                    $collection_type = "";
                    $collection_title = "";
                    if($notification->destination_type == 'menu'){
                    $menuData = Menu::where('id', $notification->destination)->first();
                    $destinationType = $notification->destination_type;
                    $destination = $menuData != NULL ? $menuData->link : NULL;
                    }else{
                        $destinationType = $notification->destination_type;
                        $destination = $notification->destination;
                        if($notification->destination_type == 'collection'){
                            $normal = NormalCollection::where('collection_id',$notification->destination)->first();
                            if($normal){
                               $collection_type = "normal_collection"; 
                               $collection_title = $normal->text_english; 
                            }else{
                                $goal = CollectionByGoal::where('collection_id',$notification->destination)->first();
                                if($goal){
                                   $collection_type = "collection_by_goal"; 
                                    $collection_title = $goal->title_english; 
                                }
                            }
                            
                        }
                    }    
                    foreach ($tokens as $token) {
                        $notification_id = trim($token->device_token);
                        if ($token->language_id == 'en') {
                            $title = $notification->title_english;
                            $message = trim($notification->description_english);
                        } elseif ($token->language_id == 'ar') {
                            $title = $notification->title_arabic;
                            $message = trim($notification->description_arabic);
                        }

                        $post_data = '{
                        "to" : "' . $notification_id . '",
                        "data" : {
                            "body" : "",
                            "title" : "' . $title . '",
                            "type" : "' . $type . '",
                            "message" : "' . $message . '",
                            "notification_type": "manual",
                            "manual_id": "'.$notification->id.'",
                            "destination_type": "'.$destinationType.'",
                            "destination": "'.$destination.'",
                            "collection_type": "'.$collection_type.'",
                            "collection_title": "'.$collection_title.'",
                            "variant_id": "0" ,
                            "product_id": "0" ,
                        },
                        "notification" : {
                            "body" : "' . $message . '",
                            "title" : "' . $title . '",
                            "type" : "' . $type . '",
                            "largeIcon"	: "large_icon",
                            "message" : "' . $message . '",
                            "icon" : "' . $icon . '",
                            "image" : "' . $image . '",
                            "sound" : "1",
                            "badge" : "1",
                            "mutable-content": "1",
                            "notification_type": "manual",
                            "manual_id": "'.$notification->id.'",
                            "destination_type": "'.$destinationType.'",
                            "destination": "'.$destination.'",
                            "collection_type": "'.$collection_type.'",
                            "collection_title": "'.$collection_title.'",
                            "variant_id": "0" ,
                            "product_id": "0" ,
                            },
                            
                        }';

                        $crl = curl_init();

                        $headr = array();
                        $headr[] = 'Content-type: application/json';
                        $headr[] = 'Authorization: key=' . $accesstoken;

                        curl_setopt($crl, CURLOPT_SSL_VERIFYPEER, false);

                        curl_setopt($crl, CURLOPT_URL, $URL);
                        curl_setopt($crl, CURLOPT_HTTPHEADER, $headr);

                        curl_setopt($crl, CURLOPT_POST, true);
                        curl_setopt($crl, CURLOPT_POSTFIELDS, $post_data);
                        curl_setopt($crl, CURLOPT_RETURNTRANSFER, true);

                        $result = curl_exec($crl);

                        if ($result === FALSE) {
                            die('FCM Send Error: ' . curl_error($crl));
                        } else {
                            // $result = json_decode($result,true);
                            $response[] = $result;

                            curl_close($crl);
                        }
                    }
                    $notification->status = 1;
                    $notification->save();
                }
            }
            return $response;
        }
    }

    public function sendChallengeNotification()
    {
        $result = NULL;
        $response = [];
        $serverKey = trim(env('SERVER_KEY'));
        $challenges = UsedChallenge::where('status', '=', 1)->get();

        if ($challenges) {
            $type = 'basic';
            $accesstoken = $serverKey;
            $URL = 'https://fcm.googleapis.com/fcm/send';
            $icon  = "https://image.flaticon.com/icons/png/512/270/270014.png";/*Default Icon*/


            foreach ($challenges as $challenge) {

                $today = Carbon::now();
                $date = $challenge->created_at;
                $difference = $today->diff($date);
                $diffInHours   = abs($difference->h); //24

                if ($diffInHours >= 24) {
                    $content = ChallengeContent::where('id', '>', $challenge->content_id)->where('challenge_id', $challenge->challenge_id)
                        ->oldest('id')
                        ->first();
                    if ($content) {
                        $nextContentExist = UsedChallenge::where('device_id', $challenge->device_id)->where('challenge_id', $content->challenge_id)->where('content_id', $content->id)->first();
                        if ($nextContentExist) {
                            continue;
                        } else {
                            $image      = $content->cover_image != NULL ? config('app.url') . '/storage/app/' . $content->cover_image : config('app.url') . '/storage/app/' . $content->url;
                            $token = UserToken::where('device_id', $challenge->device_id)->first();
                            $notification_id = trim($token->device_token);
                            $title = "Today challenge";
                            $message = $token->device_token == "en" ? $content->notification_english : $content->notification_arabic;

                            $post_data = '{
                                "to" : "' . $notification_id . '",
                                "data" : {
                                    "body" : "",
                                    "title" : "' . $title . '",
                                    "type" : "' . $type . '",
                                    "message" : "' . $message . '",
                                },
                                "notification" : {
                                    "body" : "' . $message . '",
                                    "title" : "' . $title . '",
                                    "type" : "' . $type . '",
                                    "largeIcon"	: "large_icon",
                                    "message" : "' . $message . '",
                                    "icon" : "' . $icon . '",
                                    "image" : "' . $image . '",
                                    "sound" : "1",
                                    "badge" : "1"
                                    "mutable-content": "1"
                                    },
                                    
                                }';

                            $crl = curl_init();

                            $headr = array();
                            $headr[] = 'Content-type: application/json';
                            $headr[] = 'Authorization: key=' . $accesstoken;

                            curl_setopt($crl, CURLOPT_SSL_VERIFYPEER, false);

                            curl_setopt($crl, CURLOPT_URL, $URL);
                            curl_setopt($crl, CURLOPT_HTTPHEADER, $headr);

                            curl_setopt($crl, CURLOPT_POST, true);
                            curl_setopt($crl, CURLOPT_POSTFIELDS, $post_data);
                            curl_setopt($crl, CURLOPT_RETURNTRANSFER, true);

                            $result = curl_exec($crl);

                            if ($result === FALSE) {
                                die('FCM Send Error: ' . curl_error($crl));
                            } else {
                                // $result = json_decode($result,true);
                                $response[] = $result;

                                curl_close($crl);
                                $prev = ChallengeContent::where('id', '<', $challenge->content_id)->where('challenge_id', $challenge->challenge_id)
                                    ->latest('id')
                                    ->first();
                                if ($prev) {
                                    $preContentExist = UsedChallenge::where('device_id', $challenge->device_id)->where('challenge_id', $prev->challenge_id)->where('content_id', $prev->id)->first();
                                    $preContentExist->status = 0;
                                    $preContentExist->save();
                                }
                            }
                        }
                    }
                }
            }
            return $response;
        }
    }
}
