<?php

namespace App\Http\Controllers\Api\AppSetting;

use App\Http\Controllers\Admin\Helper\HelperController;
use Response;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\FlashSale;
use App\Models\ExclusiveSection;
use App\Models\UserToken;
use GuzzleHttp\Client;

class VariantsController extends Controller
{
    public $client;

    public function __construct()
    {
        $this->client = new Client();
    }

    public function getProductVariants(Request $request)
    {
        $product = [];
        $variant = [];
        if ($request->language_id == 'en') {
            $product = Product::where('product_id', $request->product_id)->active()->where('is_delete', '=', 0)->with(['varients' => function ($q) {
                $q->select('product_id', 'option1_english as option1', 'option2_english as option2', 'option3_english as option3', 'image', 'inventory_policy', 'stock', 'price', 'strike_price', 'variant_id', 'status')->where('is_delete', '=', 0)->get();
            }])->select('product_id', 'title_english as title')->first();
        } else if ($request->language_id == 'ar') {
            $product = Product::where('product_id', $request->product_id)->active()->where('is_delete', '=', 0)->with(['varients' => function ($q) {
                $q->select('product_id', 'option1_arabic as option1', 'option2_arabic as option2', 'option3_arabic as option3', 'image', 'inventory_policy', 'stock', 'price', 'strike_price', 'variant_id', 'status')->where('is_delete', '=', 0)->get();
            }])->select('product_id', 'title_arabic as title')->first();
        }
        $user =  UserToken::where('device_id', $request->device_id)->first();
        if ($user) {
            $currency = $user->currency;
        } else {
            $currency = 'KWD';
        }
        if($product){
            foreach ($product->varients as $p) {
                $title = "";
                $status = "";
                if ($p['option1'] != null) {
                    $title = $p['option1'];
                }
                if ($p['option2'] != null) {
                    $title = $title . '/' . $p['option2'];
                }
                if ($p['option3'] != null) {
                    $title = $title . '/' . $p['option3'];
                }
                if ($p['status'] == 'notify_me') {
                    $status = "sold_out";
                }
                if ($p['status'] == 'add_to_cart') {
                    $status = $p['status'];
                }
                if ($p['status'] == 'pre_order') {
                    if ($p['stock']) {
                        $status = 'pre_order';
                    } else {
                        if ($p['inventory_policy'] == 'deny') {
                            $status = "sold_out";
                        } else if ($p['inventory_policy'] == 'continue') {
                            $status = 'pre_order';
                        }
                    }
                }
                $product_status = $this->checkProductInExclusiveSection($request->product_id, $request->language_id);
                if ($product_status) {
                    if ($p['stock']) {
                        $stock_left = "Only " . $p['stock'] . " left in stock";
                    } else {
                        $stock_left = "Out of stock";
                    }
                } else {
                    $stock_left = "";
                }
                $price = $this->changeCurrency($currency, $p['price']);
                $strike_price = $this->changeCurrency($currency, $p['strike_price']);
                $variant[] = array('variant_id' => $p['variant_id'], 'title' => $title, 'image' => $p['image'], 'price' => $price ? $price : '', 'strike_price' => $strike_price ? $strike_price : '', 'currency' => $currency, 'status' => $status, 'stock_left' => $stock_left);
            }
        }
        return response()->json([
            'success' => true,
            'option' => $variant
        ]);
    }

    public function changeCurrency($currency, $price)
    {
        if ($currency) {
            if ($currency == 'KWD') {
                return $price;
            } else {
                $value = HelperController::changeCurrency('KWD', $currency, $price);
                return $value;
            }
        } else {
            return $price;
        }
    }

    public function checkProductInExclusiveSection($product_id, $language_id)
    {
        $product_status = false;
        $now = date('Y-m-d');
        $now_time = date('H:i:s');
        $flash_id = 0;
        $flash = FlashSale::where('start_date', '<=', $now)->get();
        foreach ($flash as $key => $f) {
            if ($f->status) {
                if ($f->start_date == $now) {
                    if ($f->start_time <= $now_time) {
                        if ($f->end_date >= $now) {
                            if ($f->end_date == $now) {
                                if ($f->end_time >= $now_time) {
                                    $flash_id = $f->id;
                                    break;
                                }
                            } else {
                                $flash_id = $f->id;
                                break;
                            }
                        }
                    }
                } else {
                    if ($f->end_date >= $now) {
                        if ($f->end_date == $now) {
                            if ($f->end_time >= $now_time) {
                                $flash_id = $f->id;
                                break;
                            }
                        } else {
                            $flash_id = $f->id;
                            break;
                        }
                    }
                }
            } else {
                if ($f->start_date == $now) {
                    if ($f->start_time <= $now_time) {
                        $flash_id = $f->id;
                        break;
                    }
                } else {
                    $flash_id = $f->id;
                    break;
                }
            }
        }
        if ($flash_id) {
            $flash_sale_new = FlashSale::where('id', $flash_id)->with('exclusive_section')->first();
            if ($flash_sale_new) {
                $exclusive = ExclusiveSection::where('flash_sale_id', $flash_sale_new->id)->where('product_id', $product_id)->first();
                if ($exclusive) {
                    $product_status = true;
                } else {
                    $product_status = false;
                }
            }
        }
        return $product_status;
    }
}
