<?php

namespace App\Http\Controllers\Api\AppSetting;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Admin\Helper\HelperController;
use App\Models\Wishlist;
use App\Models\UserToken;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;

class WishlistController extends Controller
{
    public function saveWishlist(Request $request)
    {

        try {
            $data = $request->all();

            $rules = array(
                'customer_id' => 'required',
                'variant_id'  => 'required',
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $wishlist = Wishlist::where('customer_id', $request->customer_id)->where('variant_id', $request->variant_id)->first();
                if ($wishlist) {
                    if ($wishlist->status == 0) {
                        $status = 1;
                        Wishlist::where('customer_id', $request->customer_id)->where('variant_id', $request->variant_id)->update([
                            'status' => $status
                        ]);
                        return response()->json(['msg' => 'Added to Wishlist successully.', 'success' => true]);
                    } elseif ($wishlist->status == 1) {
                        $status = 0;
                        Wishlist::where('customer_id', $request->customer_id)->where('variant_id', $request->variant_id)->update([
                            'status' => $status
                        ]);
                        if ($request->language_id == 'ar') {
                            $msg = "تم الازالة من قائمة الامنيات";
                        } elseif ($request->language_id == 'en') {
                            $msg = "Removed from the Wishlist successully";
                        }
                        return response()->json(['msg' => $msg, 'success' => true]);
                    }
                } else {
                    if ($request->language_id == 'ar') {
                            $msg = "تم الاضافة لقائمة الامنيات";
                        } elseif ($request->language_id == 'en') {
                            $msg = "Added to Wishlist successully";
                        }
                    Wishlist::create([
                        'customer_id'   => $request->customer_id,
                        'variant_id'    => $request->variant_id,
                    ]);
                    return response()->json(['msg' => $msg, 'success' => true]);
                }
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json(['msg' => $msgErr, 'success' => false]);
        }
    }

    public function getWishlistItem(Request $request)
    {
        try {
            $data = $request->all();
            $rules = array(
                'customer_id' => 'required',
                'language_id' => 'required',
                'device_id' => 'required'
            );

            $validator = Validator::make($data, $rules);

            if ($validator->fails()) {
                return Response::json(array(
                    'success' => false,
                    'errors' => $validator->getMessageBag()->toArray(),
                ), 422);
            } else {
                $wishlist_data = [];
                $wishlist = [];
                $show_option = true;
                $user =  UserToken::where('device_id', $request->device_id)->first();
                if ($user) {
                    $currency = $user->currency;
                } else {
                    $currency = 'KWD';
                }
                if ($request->language_id == 'en') {
                    $wishlist = Wishlist::where('customer_id', $request->customer_id)->where('status', 1)->with(['variant' => function ($query) {
                        $query->with(['product' => function ($q) {
                            $q->select('title_english as title', 'product_id', 'variant_id');
                        }])->select('option1_english as option1', 'option2_english as option2', 'option3_english as option3', 'image', 'price', 'strike_price', 'variant_id', 'product_id', 'status', 'inventory_policy', 'stock');
                    }])->get();
                } elseif ($request->language_id == 'ar') {
                    $wishlist = Wishlist::where('customer_id', $request->customer_id)->where('status', 1)->with(['variant' => function ($query) {
                        $query->with(['product' => function ($q) {
                            $q->select('title_arabic as title', 'product_id', 'variant_id');
                        }])->select('option1_arabic as option1', 'option2_arabic as option2', 'option3_arabic as option3', 'image', 'price', 'strike_price', 'variant_id', 'product_id', 'status', 'inventory_policy', 'stock');
                    }])->get();
                }

                foreach ($wishlist as $w) {
                    $product_title = $w['variant']['product']['title'];
                    $price = $w['variant']['price'];
                    $strike_price = $w['variant']['strike_price'];
                    $image = $w['variant']['image'];
                    $status = $w['variant']['status'];
                    $show_option = true;
                    $title = "";
                    $is_wishlist = true;
                    if ($w['variant']['option1'] == 'Default Title') {
                        $show_option = false;
                    }
                    if ($w['variant']['option1'] != null) {
                        $title = $w['variant']['option1'];
                    }
                    if ($w['variant']['option2'] != null) {
                        $title = $title . '/' . $w['variant']['option2'];
                    }
                    if ($w['variant']['option3'] != null) {
                        $title = $title . '/' . $w['variant']['option3'];
                    }
                    if ($strike_price) {
                        $strike_price = strval($this->changeCurrency($currency, $strike_price));
                        $label = "SALE";
                    } else {
                        $label = "";
                        $strike_price = "";
                    }
                    if($price){
                        $price = strval($this->changeCurrency($currency, $price));
                    }
                    if ($status == 'pre_order') {
                        if ($w['stock']) {
                            $status = 'pre_order';
                        } else {
                            if ($w['variant']['inventory_policy'] == 'deny') {
                                $status = "notify_me";
                            } else if ($w['variant']['inventory_policy']  == 'continue') {
                                $status = 'pre_order';
                            }
                        }
                    }
                    $exclusive_label = app('App\Http\Controllers\Api\AppSetting\CollectionController')->checkProductInExclusiveSection($w['variant']['product_id'], $request->language_id);
                    if ($exclusive_label) {
                        $label = $exclusive_label;
                    }
                    $wishlist_data[] = array('product_id' => $w['variant']['product_id'], 'title' => $product_title, 'variant_title' => $title, 'variant_id' => $w['variant']['variant_id'], 'image' => $image, 'price' => $price, 'strike_price' => $strike_price, 'currency'=>$currency, 'show_option' => $show_option, 'label' => $label, 'status' => $status, 'is_wishlist' => $is_wishlist);
                }
                return response()->json([
                    'success' => true,
                    'wishlist' => $wishlist_data
                ]);
            }
        } catch (Exception $e) {
            if ($request->language_id == 'ar') {
                $msgErr = "حدث خطأ";
            } elseif ($request->language_id == 'en') {
                $msgErr = "Something went wrong";
            }
            return response()->json([
                'success' => false,
                'msg' => $msgErr
            ]);
        }
    }

    public function changeCurrency($currency, $price)
    {
        if ($currency) {
            if ($currency == 'KWD') {
                return $price;
            } else {
                $value = HelperController::changeCurrency('KWD', $currency, $price);
                return $value;
            }
        } else {
            return $price;
        }
    }
}
