<?php

namespace App\Http\Controllers\Api\Search;

use App\Http\Controllers\Admin\Helper\HelperController;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Product;
use App\Models\Collection;
use App\Models\CollectionProduct;
use App\Models\SearchTerm;
use App\Models\UserToken;
use Carbon\Carbon;
use GuzzleHttp\Client;

class SearchController extends Controller
{

    public function __construct()
    {
        $this->client = new Client();
    }

    public function searchProduct(Request $request)
    {

        if ($request->language_id && $request->title) {
            $remaininglimit = 20;
            $ProductidArray = [];
            if ($request->language_id == 'en') {
                
                $ProductidArray = Product::where('title_english', 'LIKE', '%' . $request->title . '%')
                    ->orWhere('title_arabic', 'LIKE', '%' . $request->title . '%')->distinct()->pluck('product_id')->toArray();

                    $ProductidArray = array_unique(array_merge($ProductidArray, Product::whereNotIn('product_id', $ProductidArray)->where('active', '=', 1)->where('description_withouthtml_english', 'LIKE', '%' . $request->title . '%')
                    ->orWhere('description_withouthtml_arabic', 'LIKE', '%' . $request->title . '%')->take($remaininglimit)->distinct()->pluck('product_id')->toArray()));
                    

                $remaininglimit -= count($ProductidArray);

                $collectionResult3 = Collection::select('collection_id as id', 'title_english as title', 'type', 'image', 'price', 'strike_price')->where('title_english', 'LIKE', '%' . $request->title . '%')
                    ->orWhere('title_arabic', 'LIKE', '%' . $request->title . '%')->take($remaininglimit)->distinct()->get()->toArray();

                $ProductidArray = $this->getCollectionsProducts($collectionResult3, $ProductidArray);

                $remaininglimit -= count($ProductidArray);

                $collectionResult4 = Collection::select('collection_id as id', 'title_english as title', 'type', 'image', 'price', 'strike_price')->where('description_withouthtml_english', 'LIKE', '%' . $request->title . '%')
                    ->orWhere('description_withouthtml_arabic', 'LIKE', '%' . $request->title . '%')->take($remaininglimit)->distinct()->get()->toArray();

                $ProductidArray = $this->getCollectionsProducts($collectionResult4, $ProductidArray);

                $remaininglimit -= count($ProductidArray);
            } else if ($request->language_id == 'ar') {

                $ProductidArray = Product::where('title_arabic', 'LIKE', '%' . $request->title . '%')
                    ->orWhere('title_english', 'LIKE', '%' . $request->title . '%')->distinct()->pluck('product_id')->toArray();

                $ProductidArray = array_unique(array_merge($ProductidArray, Product::whereNotIn('product_id', $ProductidArray)->where('description_withouthtml_arabic', 'LIKE', '%' . $request->title . '%')
                    ->orWhere('description_withouthtml_english', 'LIKE', '%' . $request->title . '%')->take($remaininglimit)->distinct()->pluck('product_id')->toArray()));

                $remaininglimit -= count($ProductidArray);

                $collectionResult3 = Collection::select('collection_id as id', 'title_english as title', 'type', 'image', 'price', 'strike_price')->where('title_arabic', 'LIKE', '%' . $request->title . '%')
                    ->orWhere('title_english', 'LIKE', '%' . $request->title . '%')->take($remaininglimit)->distinct()->get()->toArray();

                $ProductidArray = $this->getCollectionsProducts($collectionResult3, $ProductidArray);

                $remaininglimit -= count($ProductidArray);

                $collectionResult4 = Collection::select('collection_id as id', 'title_english as title', 'type', 'image', 'price', 'strike_price')->where('description_withouthtml_arabic', 'LIKE', '%' . $request->title . '%')
                    ->orWhere('description_withouthtml_english', 'LIKE', '%' . $request->title . '%')->take($remaininglimit)->distinct()->get()->toArray();

                $ProductidArray = $this->getCollectionsProducts($collectionResult4, $ProductidArray);

                $remaininglimit -= count($ProductidArray);
            }

            $allResult = $this->getProductSResult($request->language_id, $ProductidArray, $request->device_id);
            $this->saveSearchTerm($request->title);
            return response()->json([
                'success' => true,
                'search_result' => $allResult
            ]);
        } else {
            return response()->json([
                'success' => false,
                'search_result' => []
            ]);
        }
    }
    public  function  getProductSResult($language_id, $ProductidArray, $device_id)
    {
        $allResult = [];
        $ids_ordered = implode(',', $ProductidArray);

        if ($ids_ordered != '') {
            if ($language_id == 'en') {
                $allResult = Product::active()->where('is_delete', '=', 0)->whereIn('product_id', $ProductidArray)->select('product_id as id', 'title_english as title', 'type', 'image', 'price', 'strike_price', 'variant_id')->orderByRaw("FIELD(product_id, $ids_ordered)")->get()->toArray();
            } else if ($language_id == 'ar') {
                $allResult = Product::active()->where('is_delete', '=', 0)->whereIn('product_id', $ProductidArray)->select('product_id as id', 'title_arabic as title', 'type', 'image', 'price', 'strike_price', 'variant_id')->orderByRaw("FIELD(product_id, $ids_ordered)")->get()->toArray();
            }
        }
        $user =  UserToken::where('device_id', $device_id)->first();
        if ($user) {
            $currency = $user->currency;
        } else {
            $currency = 'KWD';
        }
        foreach ($allResult as $index => $result) {
            $label = "";
            if ($result['type'] == 'product') {
                $exclusive_label = app('App\Http\Controllers\Api\AppSetting\CollectionController')->checkProductInExclusiveSection($result['id'], $language_id);
                if ($exclusive_label) {
                    $label = $exclusive_label;
                } elseif($allResult[$index]['strike_price'] !=null) {
                    $label = "SALE";
                }else{
                   $label = ""; 
                }
                $allResult[$index]['label'] = $label;
            } else {
                $allResult[$index]['label'] = '';
            }
            if ($allResult[$index]['strike_price'] == null) {
                $allResult[$index]['strike_price'] = "";
            }else{
                $strikePrice = $this->changeCurrency($currency, $allResult[$index]['strike_price']);
                $allResult[$index]['strike_price'] = "$strikePrice";
            }
            if ($allResult[$index]['price'] == null) {
                $allResult[$index]['price'] = "";
            }else{
                $priceValue = $this->changeCurrency($currency, $allResult[$index]['price']);
                $allResult[$index]['price'] = "$priceValue";
            }
            if ($allResult[$index]['image'] == null) {
                $allResult[$index]['image'] = "";
            }
            $allResult[$index]['currency'] = $currency;
        }

        return  $allResult;
    }
    public  function getCollectionsProducts($collectionResult1, $prodIdArray)
    {
        $productIdArray = [];
        $productIdArray = array_merge($productIdArray, $prodIdArray);

        if (count($collectionResult1)) {
            $collection  = collect($collectionResult1);
            $collectionIdString = $collection->implode('id', ',');
            $CollectionidArray = explode(',', $collectionIdString);

            if (count($CollectionidArray)) {

                foreach ($CollectionidArray as $col_id) {
                    $productIdArray = array_unique(array_merge($productIdArray, CollectionProduct::where('collection_id', $col_id)->pluck('product_id')->toArray()));
                }
            }
        }
        return  $productIdArray;
    }
    public function searchProduct_copy(Request $request)
    {
        $getProducts = [];
        if ($request->language_id && $request->title) {
            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'products.json'
            );
            $exactMatchedproductidsArray = [];
            if ($res->getStatusCode() == 200) {
                $response_data = $res->getBody()->getContents();
                $shopify_product = json_decode($response_data)->products;
                foreach ($shopify_product as $key => $row) {
                    $requestTitle = '<p>' . $request->title . '</p>';
                    if ($row->body_html == $requestTitle || $request->title == $row->title) {
                        $pro['id'] = $row->id;
                        $pro['title'] = $row->title;
                        $pro['type'] = 'product';
                        if (isset($row->image)) {
                            $pro['image'] = $row->image->src;
                        } else {
                            $pro['image'] = '';
                        }
                        if (isset($row->variants)) {
                            $pro['price'] = $row->variants[0]->price;
                            $pro['strike_price'] = $row->variants[0]->compare_at_price;
                        } else {
                            $pro['price'] = '0.00';
                            $pro['strike_price'] = '0.00';
                        }
                        array_push($getProducts, $pro);
                        array_push($exactMatchedproductidsArray, $row->id);
                    }
                }
            }
            if ($request->language_id == 'en') {
                $Products = Product::select('id', 'title_english as title', 'product_id')->whereNotIn('product_id', $exactMatchedproductidsArray)->where('title_english', 'LIKE', '%' . $request->title . '%')->take(10)->get();
            } else if ($request->language_id == 'ar') {
                $Products = Product::select('id', 'title_arabic as title', 'product_id')->whereNotIn('product_id', $exactMatchedproductidsArray)->where('title_arabic', 'LIKE', '%' . $request->title . '%')->take(10)->get();
            }

            foreach ($Products as $row) {
                $pro['id'] = $row->id;
                $pro['title'] = $row->title;
                $pro['type'] = 'product';

                $res = $this->client->request(
                    'GET',
                    env('SHOPIFY_API_REQUEST_URL') . 'products/' . $row->product_id . '.json'
                );
                if ($res->getStatusCode() == 200) {
                    $response_data = $res->getBody()->getContents();
                    $shopify_product = json_decode($response_data)->product;
                    if (isset($shopify_product->image)) {
                        $pro['image'] = $shopify_product->image->src;
                    } else {
                        $pro['image'] = '';
                    }
                    if (isset($shopify_product->variants)) {
                        $pro['price'] = $shopify_product->variants[0]->price;
                        $pro['strike_price'] = $shopify_product->variants[0]->compare_at_price;
                    } else {
                        $pro['price'] = '0.00';
                        $pro['strike_price'] = '0.00';
                    }
                } else {
                    $pro['image'] = '';
                    $pro['price'] = '0.00';
                    $pro['strike_price'] = '0.00';
                }

                array_push($getProducts, $pro);
            }


            $res = $this->client->request(
                'GET',
                env('SHOPIFY_API_REQUEST_URL') . 'custom_collections.json'
            );
            if ($res->getStatusCode() == 200) {
                $response_data = $res->getBody()->getContents();
                $getCollection = json_decode($response_data)->custom_collections;


                $collection_count = 0;
                foreach ($getCollection as $key => $collection) {
                    if ($collection_count == 10) {
                        break;
                    }
                    if (strpos($collection->title, "|") != "") {
                        $input = preg_quote($request->title, '~');
                        // $input = preg_quote('مش', '~'); 
                        $titleArray = explode('|', $collection->title);
                        $result = preg_grep('~' . $input . '~', $titleArray);
                        if (count($result)) {
                            $collec['id'] = $collection->id;
                            if ($request->language_id == 'en') {
                                $title = $result[0];
                                $collec['title'] = $title;
                            } else if ($request->language_id == 'ar') {
                                $title = $result[1];
                                $collec['title'] = $title;
                            }
                            $collec['type'] = 'collection';
                            if (isset($collection->image)) {
                                $collec['image'] = $collection->image->src;
                            } else {
                                $collec['image'] = '';
                            }
                            $collec['price'] = '0.00';
                            $collec['strike_price'] = '0.00';
                            array_push($getProducts, $collec);
                        }
                        $collection_count += 1;
                    }
                }
            }




            return response()->json([
                'success' => true,
                'search_result' => $getProducts
            ]);
        } else {
            return response()->json([
                'success' => false,
                'search_result' => []
            ]);
        }
    }
    public function  getCollections(Request $request)
    {
        $product = [];
        $res = $this->client->request(
            'GET',
            env('SHOPIFY_API_REQUEST_URL') . 'custom_collections.json'
        );
        if ($res->getStatusCode() == 200) {
            $response_data = $res->getBody()->getContents();
            $getCollection = json_decode($response_data)->custom_collections;

            $request->language_id = 'ar';
            $allCollection = [];
            foreach ($getCollection as $key => $collection) {
                if (strpos($collection->title, "|") != "") {
                    $input = preg_quote('مش', '~');
                    $titleArray = explode('|', $collection->title);
                    $result = preg_grep('~' . $input . '~', $titleArray);
                    if (count($result)) {
                        $collec['id'] = $collection->id;
                        if ($request->language_id == 'en') {
                            $title = $result[0];
                            $collec['title'] = $title;
                        } else if ($request->language_id == 'ar') {
                            $title = $result[1];
                            $collec['title'] = $title;
                        }
                        $collec['type'] = 'collection';
                        array_push($allCollection, $collec);
                    }
                }
            }
        }
        print_r(json_encode($getCollection));
    }

    public function saveSearchTerm($term){
        // $exist = SearchTerm::where('name', trim($term))->first();
        // if($exist){
        //     SearchTerm::where('id', $exist->id)->update([
        //         'count' => $exist->count + 1
        //     ]);
        // }else{
        //     SearchTerm::create([
        //         'name' => $term,
        //         'count' => 1
        //     ]);
        // }
        SearchTerm::insert([
            'name' => $term,
            'count' => 1,
            'created_at' => Carbon::now('Asia/Kuwait'),
            'updated_at' => Carbon::now('Asia/Kuwait')
        ]);
    }

    public function changeCurrency($currency, $price){
        if($currency){
            if($currency == 'KWD'){
                return $price;
            }else{
                $value = HelperController::changeCurrency('KWD', $currency, $price);
                return $value;
            }
        }else{
            return $price;
        }
    }
}
