<?php

namespace App\Http\Controllers\Api\WebHook;

use Response;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use GuzzleHttp\Client;
use App\Http\Controllers\Controller;
use App\Models\Cart;
use App\Models\Collection;
use App\Models\Product;
use App\Models\Variant;
use Carbon\Carbon;
use App\Models\CollectionProduct;
use App\Models\CurrencyPoint;
use App\Models\Customer;
use App\Models\Milestone;
use App\Models\Wishlist;
use Exception;

class WebHookController extends Controller
{

    public $shared_secret = "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";
    public $client;

    public function __construct()
    {
        $this->client = new Client();
    }

    public function verify_webhook($data, $hmac_header)

    {
        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, $this->shared_secret, true));
        return hash_equals($hmac_header, $calculated_hmac);
    }
    public function productCreate()
    {
        try{
            $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
            $data = file_get_contents('php://input');
            $verified = $this->verify_webhook($data, $hmac_header);
    
            if ($verified) {
                $response = $data;
                $result = json_decode($response);
                $product = (array)$result;
                // Log::info(print_r($product, true));
    
                $variantIdData      = [];
                $productData        = [];
                $variantsCreateData = [];
                $variantsCreate     = [];
                $img                = [];
    
    
                if (count($product) > 0 && $product['status'] == 'active') {
                    if (count($product['images']) > 0) {
                        foreach ($product['images'] as $image) {
                            $img[$image->id] = $image->src;
                        }
                    }
                    // Log::info(print_r($img, true));
                    $productData['title_english'] = $product['title'];
                    $productData['title_arabic'] = "";
                    $productData['description_english'] = $product['body_html'];
                    $productData['description_withouthtml_english'] = strip_tags($product['body_html']);
                    $productData['description_arabic'] = "";
                    $productData['description_withouthtml_arabic'] = "";
                    $productData['product_id'] = $product['id'];
                    $productData['image'] = $product['image'] != NULL ? $product['image']->src : NULL;
    
                    if (count($product['variants']) > 0) {
                        $productData['inventory_policy'] = $product['variants'][0]->inventory_policy;
                        $productData['stock'] = $product['variants'][0]->inventory_quantity;
                        $productData['price'] = $product['variants'][0]->price;
                        $productData['strike_price'] = $product['variants'][0]->compare_at_price > 0 ? $product['variants'][0]->compare_at_price : NULL;
                        $productData['variant_id'] = $product['variants'][0]->id;
                        if (array_key_exists($product['variants'][0]->image_id, $img)) {
                            $productData['image'] = !empty($product['variants'][0]->image_id) ? $img[$product['variants'][0]->image_id] : (!empty($product['image']) ? $product['image']->src : NULL);
                        }
    
                        foreach ($product['variants'] as $variant) {
    
                            array_push($variantIdData, $variant->id);
                            $status = NULL;
                            if ($variant->inventory_quantity != NULL) {
                                $status = 'add_to_cart';
                            } else {
                                if ($variant->inventory_policy == 'deny') {
                                    $status = 'notify_me';
                                } else if ($variant->inventory_policy == 'continue') {
                                    $status = 'add_to_cart';
                                }
                            }
                            $variantsCreate['option1_english'] = $variant->option1;
                            $variantsCreate['option2_english'] = $variant->option2;
                            $variantsCreate['option3_english'] = $variant->option3;
                            $variantsCreate['option1_arabic'] = "";
                            $variantsCreate['option2_arabic'] = "";
                            $variantsCreate['option3_arabic'] = "";
                            $variantsCreate['inventory_policy'] = $variant->inventory_policy;
                            $variantsCreate['stock'] = $variant->inventory_quantity;
                            $variantsCreate['price'] = $variant->price;
                            $variantsCreate['strike_price'] = $variant->compare_at_price > 0 ? $variant->compare_at_price : NULL;
                            $variantsCreate['status']     = $status;
                            $variantsCreate['variant_id'] = $variant->id;
                            $variantsCreate['product_id'] = $product['id'];
    
                            if (array_key_exists($variant->image_id, $img)) {
                                $variantsCreate['image'] = !empty($variant->image_id) ? $img[$variant->image_id] : (!empty($product['image']) ? $product['image']['src'] : NULL);
                            } else {
                                $variantsCreate['image'] = !empty($product['image']) ? $product['image']->src : NULL;
                            }
                            array_push($variantsCreateData, $variantsCreate);
                        }
                    }
    
                    for ($i = 0; $i < count($product['variants']); $i++) {
                        if ($product['variants'][$i]->inventory_quantity) {
                            if ($product['variants'][$i]->compare_at_price != NULL) {
                                $productData['inventory_policy'] = $product['variants'][$i]->inventory_policy;
                                $productData['stock'] = $product['variants'][$i]->inventory_quantity;
                                $productData['price'] = $product['variants'][$i]->price;
                                $productData['strike_price'] = $product['variants'][$i]->compare_at_price > 0 ? $product['variants'][$i]->compare_at_price : NULL;
                                $productData['variant_id'] = $product['variants'][$i]->id;
    
                                if (array_key_exists($product['variants'][$i]->image_id, $img)) {
                                    $productData['image'] = !empty($product['variants'][$i]->image_id) ? $img[$product['variants'][$i]->image_id] : (!empty($product['image']) ? $product['image']->src : NULL);
                                }
                                break;
                            }
    
                            $productData['inventory_policy'] = $product['variants'][$i]->inventory_policy;
                            $productData['stock'] = $product['variants'][$i]->inventory_quantity;
                            $productData['price'] = $product['variants'][$i]->price;
                            $productData['strike_price'] = $product['variants'][$i]->compare_at_price > 0 ? $product['variants'][$i]->compare_at_price : NULL;
                            $productData['variant_id'] = $product['variants'][$i]->id;
    
                            if (array_key_exists($product['variants'][$i]->image_id, $img)) {
                                $productData['image'] = !empty($product['variants'][$i]->image_id) ? $img[$product['variants'][$i]->image_id] : (!empty($product['image']) ? $product['image']->src : NULL);
                            }
                        }
                    }
                    // Log::info(print_r(["product data"], true));
                    // Log::info(print_r($productData, true));
                    // Log::info(print_r(["variant data"], true));
                    // Log::info(print_r($variantsCreateData, true));
                    $product = Product::create($productData);
                    if ($product) {
                        $variant = Variant::insert($variantsCreateData);
    
                        // $variantIds =  Variant::where('product_id', $productData['product_id'])->get()->pluck('variant_id')->toArray();
                        // if ($variantIdData != null && $variantIds != null) {
                        //     foreach ($variantIds as $variantId) {
                        //         if (!in_array($variantId, $variantIdData)) {
                        //             Variant::where('variant_id', $variantId)->delete();
                        //         }
                        //     }
                        // }
                    }
                    echo 'created';
                }
            } else {
                $response = "something went wrong.";
            }
        }catch(Exception $e){
            echo 'success';
        }

    }

    //product update

    public function productUpdate()
    {
        try{
            $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
            $data = file_get_contents('php://input');
            $verified = $this->verify_webhook($data, $hmac_header);
    
            if ($verified) {
                $response = $data;
                $result = json_decode($response);
                $product = (array)$result;
                // Log::info(print_r($product, true));
    
                $variantIdData = [];
                $productData = [];
                $variantsUpdateData = [];
                $variantsUpdate = [];
                $img = [];
    
                if (count($product) > 0 && $product['status'] == 'active') {
                    if (count($product['images']) > 0) {
                        foreach ($product['images'] as $image) {
                            $img[$image->id] = $image->src;
                        }
                    }
                    // Log::info(print_r($img, true));
                    $productData['title_english'] = $product['title'];
                    $productData['title_arabic'] = "";
                    $productData['description_english'] = $product['body_html'];
                    $productData['description_withouthtml_english'] = strip_tags($product['body_html']);
                    $productData['description_arabic'] = "";
                    $productData['description_withouthtml_arabic'] = "";
                    $productData['product_id'] = $product['id'];
                    $productData['image'] = $product['image'] != NULL ? $product['image']->src : NULL;
    
                    if (count($product['variants']) > 0) {
                        $productData['inventory_policy'] = $product['variants'][0]->inventory_policy;
                        $productData['stock'] = $product['variants'][0]->inventory_quantity;
                        $productData['price'] = $product['variants'][0]->price;
                        $productData['strike_price'] = $product['variants'][0]->compare_at_price > 0 ? $product['variants'][0]->compare_at_price : NULL;
                        $productData['variant_id'] = $product['variants'][0]->id;
                        if (array_key_exists($product['variants'][0]->image_id, $img)) {
                            $productData['image'] = !empty($product['variants'][0]->image_id) ? $img[$product['variants'][0]->image_id] : (!empty($product['image']) ? $product['image']->src : NULL);
                        }
    
                        foreach ($product['variants'] as $variant) {
    
                            array_push($variantIdData, $variant->id);
                            $status = NULL;
                            if ($variant->inventory_quantity != NULL) {
                                $status = 'add_to_cart';
                            } else {
                                if ($variant->inventory_policy == 'deny') {
                                    $status = 'notify_me';
                                } else if ($variant->inventory_policy == 'continue') {
                                    $status = 'add_to_cart';
                                }
                            }
                            $variantsUpdate['option1_english'] = $variant->option1;
                            $variantsUpdate['option2_english'] = $variant->option2;
                            $variantsUpdate['option3_english'] = $variant->option3;
                            $variantsUpdate['option1_arabic'] = "";
                            $variantsUpdate['option2_arabic'] = "";
                            $variantsUpdate['option3_arabic'] = "";
                            $variantsUpdate['inventory_policy'] = $variant->inventory_policy;
                            $variantsUpdate['stock'] = $variant->inventory_quantity;
                            $variantsUpdate['price'] = $variant->price;
                            $variantsUpdate['strike_price'] = $variant->compare_at_price > 0 ? $variant->compare_at_price : NULL;
                            $variantsUpdate['status']     = $status;
                            $variantsUpdate['variant_id'] = $variant->id;
                            $variantsUpdate['product_id'] = $product['id'];
    
                            if (array_key_exists($variant->image_id, $img)) {
                                $variantsUpdate['image'] = !empty($variant->image_id) ? $img[$variant->image_id] : (!empty($product['image']) ? $product['image']['src'] : NULL);
                            } else {
                                $variantsUpdate['image'] = !empty($product['image']) ? $product['image']->src : NULL;
                            }
                            array_push($variantsUpdateData, $variantsUpdate);
                        }
                    }
    
                    for ($i = 0; $i < count($product['variants']); $i++) {
                        if ($product['variants'][$i]->inventory_quantity) {
                            if ($product['variants'][$i]->compare_at_price != NULL) {
                                $productData['inventory_policy'] = $product['variants'][$i]->inventory_policy;
                                $productData['stock'] = $product['variants'][$i]->inventory_quantity;
                                $productData['price'] = $product['variants'][$i]->price;
                                $productData['strike_price'] = $product['variants'][$i]->compare_at_price > 0 ? $product['variants'][$i]->compare_at_price : NULL;
                                $productData['variant_id'] = $product['variants'][$i]->id;
    
                                if (array_key_exists($product['variants'][$i]->image_id, $img)) {
                                    $productData['image'] = !empty($product['variants'][$i]->image_id) ? $img[$product['variants'][$i]->image_id] : (!empty($product['image']) ? $product['image']->src : NULL);
                                }
                                break;
                            }
    
                            $productData['inventory_policy'] = $product['variants'][$i]->inventory_policy;
                            $productData['stock'] = $product['variants'][$i]->inventory_quantity;
                            $productData['price'] = $product['variants'][$i]->price;
                            $productData['strike_price'] = $product['variants'][$i]->compare_at_price > 0 ? $product['variants'][$i]->compare_at_price : NULL;
                            $productData['variant_id'] = $product['variants'][$i]->id;
    
                            if (array_key_exists($product['variants'][$i]->image_id, $img)) {
                                $productData['image'] = !empty($product['variants'][$i]->image_id) ? $img[$product['variants'][$i]->image_id] : (!empty($product['image']) ? $product['image']->src : NULL);
                            }
                        }
                    }
                    // Log::info(print_r(["product data"], true));
                    // Log::info(print_r($productData, true));
                    // Log::info(print_r(["variant data"], true));
                    // Log::info(print_r($variantsUpdateData, true));
                    $product = Product::where('product_id', $productData['product_id'])->update([
                        'inventory_policy' => $productData['inventory_policy'],
                        'stock' => $productData['stock'],
                        'price' => $productData['price'],
                        'strike_price' => $productData['strike_price'],
                        'variant_id' => $productData['variant_id'],
                        'image' => $productData['image'],
                    ]);
                    if ($product) {
                        $variant = $this->updateVariants($variantsUpdateData);
    
                        $variantIds =  Variant::where('product_id', $productData['product_id'])->get()->pluck('variant_id')->toArray();
                        Log::info(print_r(["api data"], true));
                        Log::info(print_r($variantIdData, true));
                        Log::info(print_r(["db  data"], true));
                        Log::info(print_r($variantIds, true));
                        if ($variantIdData != null && $variantIds != null) {
                            foreach ($variantIds as $variantId) {
                                if (!in_array($variantId, $variantIdData)) {
                                    Variant::where('variant_id', $variantId)->update([
                                        'is_delete' => 1
                                    ]);
                                    Cart::where('variant_id', $variantId)->update([
                                        'is_delete' => 1
                                    ]);
                                    Wishlist::where('variant_id', $variantId)->update([
                                        'status' => 0
                                    ]);
                                }
                            }
                        }
                    }
                    echo 'updated';
                }
            }
        }catch(Exception $e){
            echo 'success';
        }

    }

    public function updateVariants($variants)
    {
        if ($variants != null) {
            foreach ($variants as $variant) {
                $variantCheck = Variant::where('variant_id', $variant['variant_id'])->first();
                if ($variantCheck != null) {
                    $variantRes =  Variant::where('variant_id', $variant['variant_id'])->update([
                        'inventory_policy'  => $variant['inventory_policy'],
                        'stock'             => $variant['stock'],
                        'price'             => $variant['price'],
                        'strike_price'      => $variant['strike_price'],
                        'status'            => $variant['status'],
                        'image'             => $variant['image'],
                    ]);
                } else {
                    $variantRes =  Variant::create([
                        'option1_english'   => $variant['option1_english'],
                        'option2_english'   => $variant['option2_english'],
                        'option3_english'   => $variant['option3_english'],
                        'option1_arabic'    => $variant['option1_arabic'],
                        'option2_arabic'    => $variant['option2_arabic'],
                        'option3_arabic'    => $variant['option3_arabic'],
                        'inventory_policy'  => $variant['inventory_policy'],
                        'stock'             => $variant['stock'],
                        'price'             => $variant['price'],
                        'strike_price'      => $variant['strike_price'],
                        'status'            => $variant['status'],
                        'image'             => $variant['image'],
                        'variant_id'        => $variant['variant_id'],
                        'product_id'        => $variant['product_id'],
                    ]);
                }
            }
        }
    }

    public function productDelete()
    {
        try{
            $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
            $data = file_get_contents('php://input');
            $verified = $this->verify_webhook($data, $hmac_header);
    
            if ($verified) {
                $response = $data;
                $result = json_decode($response);
                $product = (array)$result;
                // Log::info(print_r($product, true));
            }
    
            if (count($product) > 0) {
                $productVariants = Variant::where('product_id', $product['id'])->get();
                $productDel = Product::where('product_id', $product['id'])->update([
                    'is_delete' => 1
                ]);
                if ($productDel) {
                    $variantDel = Variant::where('product_id', $product['id'])->update([
                        'is_delete' => 1
                    ]);
                    foreach($productVariants as $variant){
                        Cart::where('variant_id', $variant->variant_id)->update([
                            'is_delete' => 1
                        ]);
                        Wishlist::where('variant_id', $variant->variant_id)->update([
                            'status' => 0
                        ]);
                    }
                }
            }
            echo 'deleted';
        }catch(Exception $e){
            echo 'success';
        }

    }
    public function createCollection()
    {
        try{
            $collection_data = [];
            $collection_product = [];
            $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
            $data = file_get_contents('php://input');
            $verified = $this->verify_webhook($data, $hmac_header);
    
            if ($verified) {
                $response = $data;
                $result = json_decode($response);
                $collection = (array)$result;
                $titleArray = explode('|', $collection['title']);
                $collection_data['collection_id'] = $collection['id'];
                $collection_data['title_english'] = isset($titleArray[0]) ? $titleArray[0] : $collection['title'];
                $collection_data['title_arabic'] = isset($titleArray[1]) ? $titleArray[1] : $collection['title'];
                $collection_data['description_english'] = $collection['body_html'];
                $collection_data['description_withouthtml_english'] = strip_tags($collection['body_html']);
                $collection_data['type'] = 'collection';
                $collection_data['image'] = isset($collection['image']) ? $collection['image']->src : null;
    
                if ($collection_data) {
                    $createCollectionResult = Collection::create($collection_data);
                }
    
                // Log::info(print_r(['collection data for create'], true));
                // Log::info(print_r($collection_data, true));
    
                $productsRes = $this->client->request(
                    'GET',
                    env('SHOPIFY_API_REQUEST_URL') . 'collections/' . $collection['id'] . '/products.json'
                );
                if ($productsRes->getStatusCode() == 200) {
                    $response_data = $productsRes->getBody()->getContents();
                    $getProduct = json_decode($response_data)->products;
                    if ($getProduct) {
                        foreach ($getProduct as $key => $product) {
                            $collection_product[] = array('collection_id' => $collection['id'], 'product_id' => $product->id, 'created_at' => now(), 'updated_at' => now());
                        }
    
                        // Log::info(print_r(['collection product data for create'], true));
                        // Log::info(print_r($collection_product, true));
                        if ($createCollectionResult) {
                            if ($collection_product) {
                                CollectionProduct::insert($collection_product);
                            }
                        }
                    }
                }
            } else {
                $response = 'something wrong';
            }
        }catch(Exception $e){
            echo 'success';
        }
    }
    public function updateCollection()
    {
        try{
            $collection_data = [];
            $collection_product = [];
            $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
            $data = file_get_contents('php://input');
            $verified = $this->verify_webhook($data, $hmac_header);
    
            if ($verified) {
                $response = $data;
                $result = json_decode($response);
                $collection = (array)$result;
                // Log::info(print_r(['collection update data'], true));
                // Log::info(print_r($collection, true));
                $titleArray = explode('|', $collection['title']);
                $collection_data['collection_id'] = $collection['id'];
                $collection_data['title_english'] = isset($titleArray[0]) ? $titleArray[0] : $collection['title'];
                $collection_data['type'] = 'collection';
                $collection_data['image'] = isset($collection['image']) ? $collection['image']->src : null;
    
                if ($collection_data) {
                    $updateCollectionResult =  Collection::where('collection_id', $collection['id'])->update($collection_data);
                    CollectionProduct::where('collection_id', $collection['id'])->delete();
                }
                // Log::info(print_r(['collection data for update'], true));
                // Log::info(print_r($collection_data, true));
                $productsRes = $this->client->request(
                    'GET',
                    env('SHOPIFY_API_REQUEST_URL') . 'collections/' . $collection['id'] . '/products.json'
                );
                if ($productsRes->getStatusCode() == 200) {
                    $response_data = $productsRes->getBody()->getContents();
                    $getProduct = json_decode($response_data)->products;
    
                    if ($getProduct) {
                        foreach ($getProduct as $key => $product) {
                            $collection_product[] = array('collection_id' => $collection['id'], 'product_id' => $product->id, 'created_at' => now(), 'updated_at' => now());
                        }
    
                        // Log::info(print_r(['collection product data for update'], true));
                        // Log::info(print_r($collection_product, true));
                        if ($updateCollectionResult) {
                            if ($collection_product) {
                                CollectionProduct::insert($collection_product);
                            }
                        }
                    }
                }
            } else {
                $response = 'something wrong';
            }
        }catch(Exception $e){
            echo 'success';
        }
    }
    public function deleteCollection()
    {
        try{
            $collection_data = [];
            $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
            $data = file_get_contents('php://input');
            $verified = $this->verify_webhook($data, $hmac_header);
    
            if ($verified) {
                $response = $data;
                $result = json_decode($response);
                $collection = (array)$result;
                // Log::info(print_r(['collection delete'], true));
                // Log::info(print_r($collection, true));
                $collection_data['collection_id'] = $collection['id'];
                Collection::where('collection_id', $collection['id'])->delete();
                CollectionProduct::where('collection_id', $collection['id'])->delete();
            } else {
                $response = 'something wrong';
            }
        }catch(Exception $e){
            echo 'success';
        }
    }

    public function customerCreate()
    {
        try{
            $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
            $data = file_get_contents('php://input');
            $verified = $this->verify_webhook($data, $hmac_header);
    
            if ($verified) {
                $response = $data;
                $result = json_decode($response);
                $customer = (array)$result;
                // Log::info(print_r(['customer create data'], true));
                // Log::info(print_r($customer, true));
                if (count($customer) > 0) {
                    $level = $this->getLevel(0);
                    $milestone_id = $this->getMilestone(0);
                    $result = Customer::create([
                        "customer_id"     => $customer['id'],
                        "email"           => $customer['email'],
                        "first_name"      => $customer['first_name'],
                        "last_name"       => $customer['last_name'],
    
                        "phone"           => $customer['phone'],
                        "state"           => $customer['state'],
                        "total_spent"     => $customer['total_spent'],
                        "last_order_id"   => $customer['last_order_id'],
                        "last_order_name" => $customer['last_order_name'],
                        "currency"        => $customer['currency'],
                        "orders_count"    => $customer['orders_count'],
                        "milestone_entry" => Carbon::now('Asia/Kuwait'),
                        "level"           =>  $level,
                        "milestone_id"    =>  $milestone_id,
                    ]);
    
                    if ($result) {
                        echo 'created';
                    } else {
                        echo  'Something went wrong on creation!';
                    }
                } else {
                    echo  'customer data is empty';
                }
            } else {
                echo  'Something went wrong!';
            }

        }catch(Exception $e){
            echo 'success';
        }

    }

    public function customerUpdate()
    {
        try{
            $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
            $data = file_get_contents('php://input');
            $verified = $this->verify_webhook($data, $hmac_header);
    
            if ($verified) {
                $response = $data;
                $result = json_decode($response);
                $customer = (array)$result;
                Log::info(print_r(['customer update data'], true));
                Log::info(print_r($customer, true));
                if (count($customer) > 0) {
                    $levelBefore = NULL;
                    $totalPointAfter = 0;
                    $levelAfter = NULL;
    
                    $exist = Customer::where("customer_id", $customer['id'])->first();
                    if($exist){
                        $levelBefore = $exist->level;
                        $currencyPoint  = CurrencyPoint::where('currency', $customer['currency'])->first();
                        if($currencyPoint){
                            $point = $customer['total_spent'] * $currencyPoint->each_point;
                        }else{
                            $cPoint    = CurrencyPoint::where('currency', 'KWD')->first();
                            if($cPoint){
                                $point = $customer['total_spent'] * $cPoint->each_point;
                            }else{
                                $point = $customer['total_spent'] * 1;
                            }
                        }
                        $result = Customer::where("customer_id", $customer['id'])->update([
                            "email"           => $customer['email'],
                            "first_name"      => $customer['first_name'],
                            "last_name"       => $customer['last_name'],
        
                            "phone"           => $customer['phone'],
                            "state"           => $customer['state'],
                            "total_spent"     => $customer['total_spent'],
                            "last_order_id"   => $customer['last_order_id'],
                            "last_order_name" => $customer['last_order_name'],
                            "currency"        => $customer['currency'],
                            "orders_count"    => $customer['orders_count'],
                            "point"           => $point,
                        ]);
                    }
    
                    if ($result) {
                        $updatedCustomer = Customer::where("customer_id", $customer['id'])->first();
                        $totalPointAfter = $updatedCustomer->point + $updatedCustomer->extra_point;
                        $levelAfter = $this->getLevel($totalPointAfter);
                        $milestone_id = $this->getMilestone($totalPointAfter);
    
                        if (trim($levelBefore) != trim($levelAfter)) {
                            // echo $levelAfter;die;
                            $milestoneEntry = Customer::where('customer_id', $customer['id'])->update([
                                "milestone_id"      => $milestone_id,
                                "level"             => $levelAfter,
                                "milestone_entry"   => Carbon::now('Asia/Kuwait'),
                            ]);
                        }
                        echo 'updated';
                    } else {
                        echo  'Something went wrong on updation!';
                    }
                } else {
                    echo  'customer data is empty';
                }
            } else {
                echo  'Something went wrong!';
            }
        }catch(Exception $e){
            echo 'success';
        }

    }

    public function deleteCustomer()
    {
        $collection_data = [];
        $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
        $data = file_get_contents('php://input');
        $verified = $this->verify_webhook($data, $hmac_header);

        if ($verified) {
            $response = $data;
            $result = json_decode($response);
            $customer = (array)$result;
            Log::info(print_r(['customer delete data'], true));
            Log::info(print_r($customer, true));
            // $result = Customer::where('customer_id', $customer['id'])->delete();
            // if ($result) {
            //     echo 'delete';
            // } else {
            //     echo  'Something went wrong on delete!';
            // }
        } else {
            $response = 'something wrong';
        }
    }

    public function getLevel($point){
        $max = Milestone::max('from_point');
        if($point >= $max){
            $milestone = Milestone::where('from_point', '=',$max)->select('level_english')->first();
            if($milestone){
                return $milestone->level_english;
            }else{
                return 'Not Available';
            }
        }else{
            $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('level_english')->first();
            if($milestone){
                return $milestone->level_english;
            }else{
                return 'Not Available';
            }
        }
    }

    public function getMilestone($point){

        $max = Milestone::max('from_point');
        if($point >= $max){
            $milestone = Milestone::where('from_point', '=',$max)->select('id', 'level_english')->first();
            if($milestone){
                return $milestone->id;
            }else{
                return 0;
            }
        }else{
            $milestone = Milestone::where('from_point', '<=',$point)->where('to_point', '>=',$point)->select('id', 'level_english')->first();
            if($milestone){
                return $milestone->id;
            }else{
                return 0;
            }
        }
    }
}
