<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\UserLoginHistory;
use App\Providers\RouteServiceProvider;
use App\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Support\Facades\Hash;
use Carbon\Carbon;
use BrowserDetect;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/admin/dashboard';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
    }

    protected function attemptLogin(Request $request)
    {

        // object
        $agent = new \Jenssegers\Agent\Agent;

        $device = $agent->device();
        $browser = $agent->browser();

        // $device = NULL;

        // #Desktop/Laptop
        // if((new \Jenssegers\Agent\Agent())->isDesktop()){
        //     $device = 'Desktop';
        // }

        // #Mobile
        // if((new \Jenssegers\Agent\Agent())->isMobile()){
        //     $device = 'Mobile';
        // }

        // #Tablet
        // if((new \Jenssegers\Agent\Agent())->isTablet()){
        //     $device = 'Tablet';
        // }


        $exist = $this->guard()->attempt(
            $this->credentials($request), $request->filled('remember')
        );

        if($exist){
            $user = User::where('email', $request->email)->first();
            $u = User::where('email', $request->email)->update([
                'device' => $device,
                'last_login' => now()
            ]);
            if($user){
                $c = UserLoginHistory::create([
                    'user_id' => $user->id,
                    'login' => Carbon::now('Asia/Kuwait')->format('Y-m-d H:i:s'),
                    'device' => $device,
                    'browser' => $browser,
                ]);
            }

            return $exist;

        }else{
            return $exist;
        }


    }

    protected function credentials(Request $request)
    {
        // return $request->only($this->username(), 'password','status');
        $credential =  $request->only($this->username(), 'password');
        $credential['status'] = 1;
        return $credential;
    }

    public function logout(Request $request)
    {
        $agent = new \Jenssegers\Agent\Agent;
        $device = $agent->device();
        $browser = $agent->browser();

        $id = Auth::id();
        $update = User::where('id', $id)->update([
            'last_logout' => now()
        ]);
        if($update){
            $c = UserLoginHistory::create([
                'user_id' => $id,
                'logout' => Carbon::now('Asia/Kuwait')->format('Y-m-d H:i:s'),
                'device' => $device,
                'browser' => $browser,
            ]);
            $this->guard()->logout();
    
            $request->session()->invalidate();
    
            $request->session()->regenerateToken();
    
            if ($response = $this->loggedOut($request)) {
                var_dump($response);die;
                return $response;
            }
    
            return $request->wantsJson()
                ? new JsonResponse([], 204)
                : redirect('/login');
        }
    }
}
